@@ -18,7 +18,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with gnuastro. If not, see <http://www.gnu.org/licenses/>.
+along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 **********************************************************************/
 #include <config.h>
 
@@ -31,17 +31,17 @@ along with gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <unistd.h>
 #include <sys/stat.h>
 
-#include "checkset.h"
+#include <gnuastro/checkset.h>
 
 
 #define FIXEDFORFILE        "The value of `%s` (given as %s)"
 #define FIXEDFOROPTION      "The value of `--%s (-%c)` (given as %s)"
-#define NOTEMSG_NOTNUMBER   "not readable as number."
-#define NOTEMSG_EQZEROORONE "should be ==0 or ==1."
-#define NOTEMSG_LEQZERO     "should be >=0."
-#define NOTEMSG_LARGERZERO  "should be >0."
-#define NOTEMSG_4OR8        "should be either 4 or 8."
-#define NOTEMSG_SMALLERONE  "should be >=0 and <=1."
+#define NOTEMSG_NOTNUMBER   "not readable as number"
+#define NOTEMSG_EQZEROORONE "should be ==0 or ==1"
+#define NOTEMSG_LEQZERO     "should be >=0"
+#define NOTEMSG_LARGERZERO  "should be >0"
+#define NOTEMSG_4OR8        "should be either 4 or 8"
+#define NOTEMSG_SMALLERONE  "should be >=0 and <=1"
 
 
 #define CHECKFULLNUMBER   if(*tailptr!='\0'){				\
@@ -56,13 +56,52 @@ along with gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 
 
+/****************************************************************
+ ************      Check and convert strings    *****************
+ ****************************************************************/
+/* See if a given string is a floating point number, if so, put it in
+   the number.*/
+int
+strisdouble(char *string, double *out)
+{
+  char *tailptr;
+  double tmp=*out;
+  *out=strtod(string, &tailptr);
+
+  /* If the tail pointer (tailptr) is the string NULL character, then
+     the string was a single double precision floating point
+     number. However, if it is any other character, then put the
+     initial value of out back inside of it and return 0. */
+  if(*tailptr=='\0') return 1;
+  *out=tmp;
+  return 0;
+}
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 /****************************************************************
  *****************      Check the numbers    ********************
  ****************************************************************/
 void
-intzeroorone(char *optarg, int *var, char *lo, char so, char *spack,
-	     char *filename, size_t lineno)
+gal_checkset_int_zero_or_one(char *optarg, int *var, char *lo, char so,
+                             char *spack, char *filename, size_t lineno)
 {
   long tmp;
   char *tailptr;
@@ -86,8 +125,8 @@ intzeroorone(char *optarg, int *var, char *lo, char so, char *spack,
 
 
 void
-int4or8(char *optarg, int *var, char *lo, char so, char *spack,
-        char *filename, size_t lineno)
+gal_checkset_int_4_or_8(char *optarg, int *var, char *lo, char so, char *spack,
+                        char *filename, size_t lineno)
 {
   long tmp;
   char *tailptr;
@@ -111,8 +150,8 @@ int4or8(char *optarg, int *var, char *lo, char so, char *spack,
 
 
 void
-intelzero(char *optarg, int *var, char *lo, char so, char *spack,
-	  char *filename, size_t lineno)
+gal_checkset_int_el_zero(char *optarg, int *var, char *lo, char so, char *spack,
+                         char *filename, size_t lineno)
 {
   long tmp;
   char *tailptr;
@@ -135,8 +174,8 @@ intelzero(char *optarg, int *var, char *lo, char so, char *spack,
 
 
 void
-intlzero(char *optarg, int *var, char *lo, char so, char *spack,
-	 char *filename, size_t lineno)
+gal_checkset_int_l_zero(char *optarg, int *var, char *lo, char so, char *spack,
+                        char *filename, size_t lineno)
 {
   long tmp;
   char *tailptr;
@@ -159,8 +198,9 @@ intlzero(char *optarg, int *var, char *lo, char so, char *spack,
 
 
 void
-intsmallerequalto(char *optarg, int *var, char *lo, char so, char *spack,
-                  char *filename, size_t lineno, long maxvalue)
+gal_checkset_int_smaller_equal_to(char *optarg, int *var, char *lo, char so,
+                                  char *spack, char *filename, size_t lineno,
+                                  long maxvalue)
 {
   long tmp;
   char *tailptr;
@@ -183,8 +223,8 @@ intsmallerequalto(char *optarg, int *var, char *lo, char so, char *spack,
 
 
 void
-longelzero(char *optarg, long *var, char *lo, char so, char *spack,
-	  char *filename, size_t lineno)
+gal_checkset_long_el_zero(char *optarg, long *var, char *lo, char so,
+                          char *spack, char *filename, size_t lineno)
 {
   long tmp;
   char *tailptr;
@@ -207,8 +247,8 @@ longelzero(char *optarg, long *var, char *lo, char so, char *spack,
 
 
 void
-anylong(char *optarg, long *var, char *lo, char so, char *spack,
-	char *filename, size_t lineno)
+gal_checkset_any_long(char *optarg, long *var, char *lo, char so, char *spack,
+                      char *filename, size_t lineno)
 {
   char *tailptr;
   *var=strtol(optarg, &tailptr, 0);
@@ -226,8 +266,8 @@ anylong(char *optarg, long *var, char *lo, char so, char *spack,
    negative values, then we check that value before placing it in the
    size_t pointer.*/
 void
-sizetelzero(char *optarg, size_t *var, char *lo, char so, char* spack,
-	    char *filename, size_t lineno)
+gal_checkset_sizet_el_zero(char *optarg, size_t *var, char *lo, char so,
+                           char* spack, char *filename, size_t lineno)
 {
   long tmp;
   char *tailptr;
@@ -248,10 +288,10 @@ sizetelzero(char *optarg, size_t *var, char *lo, char so, char* spack,
 
 
 
-/* See explanation for sizetlzero above. */
+/* See explanation for gal_checkset_sizet_l_zero above. */
 void
-sizetlzero(char *optarg, size_t *var, char *lo, char so, char* spack,
-	   char *filename, size_t lineno)
+gal_checkset_sizet_l_zero(char *optarg, size_t *var, char *lo, char so,
+                          char* spack, char *filename, size_t lineno)
 {
   long tmp;
   char *tailptr;
@@ -275,8 +315,8 @@ sizetlzero(char *optarg, size_t *var, char *lo, char so, char* spack,
 
 /* Positive and odd size_t. */
 void
-sizetpodd(char *optarg, size_t *var, char *lo, char so, char* spack,
-          char *filename, size_t lineno)
+gal_checkset_sizet_p_odd(char *optarg, size_t *var, char *lo, char so,
+                         char* spack, char *filename, size_t lineno)
 {
   long tmp;
   char *tailptr;
@@ -289,7 +329,7 @@ sizetpodd(char *optarg, size_t *var, char *lo, char so, char* spack,
 	error_at_line(EXIT_FAILURE, 0, filename, lineno,
 		      FIXEDFORFILE" should be >0 and odd.", lo, optarg);
       else
-	error(EXIT_FAILURE, 0, FIXEDFOROPTION" should be >0 and odd.",
+	error(EXIT_FAILURE, 0, FIXEDFOROPTION" should be >0 and odd",
 	      lo, so, optarg);
     }
 }
@@ -299,8 +339,8 @@ sizetpodd(char *optarg, size_t *var, char *lo, char so, char* spack,
 
 
 void
-floatl0(char *optarg, float *var, char *lo, char so, char* spack,
-	char *filename, size_t lineno)
+gal_checkset_float_l_0(char *optarg, float *var, char *lo, char so, char* spack,
+                       char *filename, size_t lineno)
 {
   float tmp;
   char *tailptr;
@@ -323,8 +363,8 @@ floatl0(char *optarg, float *var, char *lo, char so, char* spack,
 
 
 void
-floatl0s1(char *optarg, float *var, char *lo, char so, char* spack,
-	  char *filename, size_t lineno)
+gal_checkset_float_l_0_s_1(char *optarg, float *var, char *lo, char so,
+                         char* spack, char *filename, size_t lineno)
 {
   float tmp;
   char *tailptr;
@@ -346,8 +386,8 @@ floatl0s1(char *optarg, float *var, char *lo, char so, char* spack,
 
 
 void
-anyfloat(char *optarg, float *var, char *lo, char so, char *spack,
-	 char *filename, size_t lineno)
+gal_checkset_any_float(char *optarg, float *var, char *lo, char so, char *spack,
+                       char *filename, size_t lineno)
 {
   char *tailptr;
   *var=strtof(optarg, &tailptr);
@@ -359,8 +399,8 @@ anyfloat(char *optarg, float *var, char *lo, char so, char *spack,
 
 
 void
-doublel0(char *optarg, double *var, char *lo, char so, char* spack,
-	 char *filename, size_t lineno)
+gal_checkset_double_l_0(char *optarg, double *var, char *lo, char so,
+                        char* spack, char *filename, size_t lineno)
 {
   double tmp;
   char *tailptr;
@@ -383,8 +423,8 @@ doublel0(char *optarg, double *var, char *lo, char so, char* spack,
 
 
 void
-doublele0(char *optarg, double *var, char *lo, char so, char* spack,
-          char *filename, size_t lineno)
+gal_checkset_double_el_0(char *optarg, double *var, char *lo, char so,
+                         char* spack, char *filename, size_t lineno)
 {
   double tmp;
   char *tailptr;
@@ -407,8 +447,9 @@ doublele0(char *optarg, double *var, char *lo, char so, char* spack,
 
 
 void
-doublelvalue(char *optarg, double *var, char *lo, char so, char* spack,
-             double value, char *filename, size_t lineno)
+gal_checkset_double_l_value(char *optarg, double *var, char *lo, char so,
+                            char* spack, double value, char *filename,
+                            size_t lineno)
 {
   float tmp;
   char *tailptr;
@@ -431,8 +472,8 @@ doublelvalue(char *optarg, double *var, char *lo, char so, char* spack,
 
 
 void
-anydouble(char *optarg, double *var, char *lo, char so, char *spack,
-	  char *filename, size_t lineno)
+gal_checkset_any_double(char *optarg, double *var, char *lo, char so,
+                        char *spack, char *filename, size_t lineno)
 {
   char *tailptr;
   *var=strtod(optarg, &tailptr);
@@ -463,7 +504,7 @@ anydouble(char *optarg, double *var, char *lo, char so, char *spack,
 /**********          My String functions:          ************/
 /**************************************************************/
 int
-stringhasspace(char *in)
+gal_checkset_string_has_space(char *in)
 {
   do
     switch(*in)
@@ -480,7 +521,7 @@ stringhasspace(char *in)
 
 
 char *
-malloccat(char *inname, char *toappend)
+gal_checkset_malloc_cat(char *inname, char *toappend)
 {
   char *out;
   size_t inl, apl;
@@ -491,8 +532,8 @@ malloccat(char *inname, char *toappend)
   errno=0;
   out=malloc(inl+apl+1);
   if(out==NULL)
-    error(EXIT_FAILURE, errno, "Allocating %lu bytes in malloccat",
-	  inl+apl+1);
+    error(EXIT_FAILURE, errno,
+          "allocating %lu bytes in gal_checkset_malloc_cat", inl+apl+1);
 
   strcpy(out, inname);
   strcat(out, toappend);
@@ -502,13 +543,29 @@ malloccat(char *inname, char *toappend)
 
 
 
+/* Copy the input string to the output (and also allocate the
+   output. */
+void
+gal_checkset_allocate_copy(char *arg, char **copy)
+{
+  /* Allocate the necessary space: */
+  errno=0;
+  *copy=malloc(strlen(arg)+1);
+  if(*copy==NULL)
+    error(EXIT_FAILURE, errno, "%lu bytes to copy %s",
+          strlen(arg)+1, arg);
+  strcpy(*copy, arg);
+}
+
+
+
 
 /* This function is mainly for reading in the arguments (from the
    command line or configuration files) that need to be copied. The
    set argument is for making sure that it has not already been set
    before, see the main.h files of any program. */
 void
-allocatecopyset(char *arg, char **copy, int *set)
+gal_checkset_allocate_copy_set(char *arg, char **copy, int *set)
 {
   /* Incase *set==1, then you shouldn't do anything, just return. */
   if(*set) return;
@@ -547,7 +604,7 @@ allocatecopyset(char *arg, char **copy, int *set)
 /**************************************************************/
 /* Check if a file exists and report if it doesn't: */
 void
-checkfile(char *filename)
+gal_checkset_check_file(char *filename)
 {
   FILE *tmpfile;
   errno=0;
@@ -567,7 +624,7 @@ checkfile(char *filename)
 
 /* Check if a file exists. If so, remove it. */
 void
-checkremovefile(char *filename, int dontdelete)
+gal_checkset_check_remove_file(char *filename, int dontdelete)
 {
   FILE *tmpfile;
 
@@ -586,7 +643,7 @@ checkremovefile(char *filename, int dontdelete)
       if(dontdelete)
 	error(EXIT_FAILURE, 0, "%s already exists and you have "
 	      "asked to not remove it with the `--dontdelete` "
-	      "(`-D`) option.", filename);
+	      "(`-D`) option", filename);
 
       /* Delete the file: */
       errno=0;
@@ -609,16 +666,16 @@ checkremovefile(char *filename, int dontdelete)
    a directory) it will return 0. Finally, if it exists but cannot be
    deleted, report an error and abort. */
 int
-dir0file1(char *name, int dontdelete)
+gal_checkset_dir_0_file_1(char *name, int dontdelete)
 {
   FILE *tmpfile;
   struct stat nameinfo;
 
   if(name==NULL)
-    error(EXIT_FAILURE, 0, "A bug! The input to the dir0file1 function in "
-          "checkset.c should not be NULL. Please contact us at "
+    error(EXIT_FAILURE, 0, "a bug! The input to the gal_checkset_dir_0_file_1 "
+          "function in checkset.c should not be NULL. Please contact us at "
           PACKAGE_BUGREPORT" so we can see what went wrong and fix it in "
-          "future updates.");
+          "future updates");
 
   errno=0;
   if(stat(name, &nameinfo)!=0)
@@ -649,16 +706,16 @@ dir0file1(char *name, int dontdelete)
     return 0;
   else if (S_ISREG(nameinfo.st_mode)) /* It is a file, GOOD. */
     {
-      checkremovefile(name, dontdelete);
+      gal_checkset_check_remove_file(name, dontdelete);
       return 1;
     }
   else 				/* Not a file or a dir, ABORT */
-    error(EXIT_FAILURE, 0, "%s not a file or a directory.", name);
+    error(EXIT_FAILURE, 0, "%s not a file or a directory", name);
 
-  error(EXIT_FAILURE, 0, "A bug! In dir0file1, (in checkset.c) "
+  error(EXIT_FAILURE, 0, "a bug! In gal_checkset_dir_0_file_1, (in checkset.c) "
 	"The process should not reach the end of the function! Please "
         "contact us at "PACKAGE_BUGREPORT" so we can see what went wrong "
-        "and fix it in future updates.");
+        "and fix it in future updates");
   return 0;
 }
 
@@ -670,23 +727,20 @@ dir0file1(char *name, int dontdelete)
    input name (inname). The suffix of the input name (if present) will
    be removed and the given suffix will be put in the end. */
 void
-automaticoutput(char *inname, char *suffix, int removedirinfo,
+gal_checkset_automatic_output(char *inname, char *suffix, int removedirinfo,
 		int dontdelete, char **outname)
 {
   char *out;
   size_t i, l, offset=0;
 
-  /* Check if the input file is actually a readable file or not! */
-  checkfile(inname);
-
   /* Note that we are just using malloccat here to allocate the right
      space! The contents of the allocated space will be changed after
      this.*/
-  out=malloccat(inname, suffix);
+  out=gal_checkset_malloc_cat(inname, suffix);
 
-  /* Put the input in the space and remove all elements including and
-     after '.'. Note that if there is no '.' in the name, malloccat
-     has already appended inname and suffix.*/
+  /* Put the input in the space and remove all elements including and after '.'.
+     Note that if there is no '.' in the name, gal_checkset_malloc_cat has
+     already appended inname and suffix.*/
   l=strlen(inname);
   strcpy(out, inname);
   for(i=l;i!=0;--i)
@@ -719,7 +773,7 @@ automaticoutput(char *inname, char *suffix, int removedirinfo,
     }
 
   /* Remove the created filename if it already exits. */
-  checkremovefile(out, dontdelete);
+  gal_checkset_check_remove_file(out, dontdelete);
 
   /* Free the outname if it was already allocated before. */
   free(*outname);
@@ -730,32 +784,105 @@ automaticoutput(char *inname, char *suffix, int removedirinfo,
 
 
 
+/* Given a filename, this function will separate its directory name
+   part. */
+char *
+gal_checkset_dir_part(char *input)
+{
+  char *out;
+  size_t i, l;
+
+  /* Find the first slash. */
+  l=strlen(input);
+  for(i=l;i!=0;--i)
+    if(input[i]=='/')
+      break;
+
+  /* If there was no slash, then the current directory should be
+     given: */
+  if(i==0)
+    {
+      errno=0;
+      out=malloc(3*sizeof *out);
+      if(out==NULL)
+        error(EXIT_FAILURE, errno, "%lu bytes for current directory "
+              "gal_checkset_dir_part", 3*sizeof *out);
+      strcpy(out, "./");
+    }
+  else
+    {
+      errno=0;
+      out=malloc((l+1)*sizeof *out);
+      if(out==NULL)
+        error(EXIT_FAILURE, errno, "%lu bytes for gal_checkset_dir_part",
+              (l+1)*sizeof *out);
+      strcpy(out, input);
+      out[i+1]='\0';
+    }
+
+  return out;
+}
+
+
+
+
+
+/* Given a file name, keep the non-directory part. Note that if there
+   is no forward slash in the input name, the full input name is
+   considered to be the notdir output.*/
+char *
+gal_checkset_not_dir_part(char *input)
+{
+  size_t i, l;
+  char *out, *tmp=input;
+
+  /* Find the first `/' to identify the directory */
+  l=strlen(input);
+  for(i=l;i!=0;--i)
+    if(input[i]=='/')
+      { tmp=&input[i+1]; break; }
+
+  /* Get the length of the notdir name: */
+  l=strlen(tmp);
+  errno=0;
+  out=malloc(l*sizeof *out);
+  if(out==NULL)
+    error(EXIT_FAILURE, errno, "%lu bytes for notdir", l*sizeof *out);
+
+  strcpy(out, tmp);
+  return out;
+}
+
+
+
+
+
 /* Check if dirname is actually a real directory and that we can
    actually write inside of it. To insure all conditions an actual
    file will be made */
 void
-checkdirwriteaddslash(char **dirname)
+gal_checkset_check_dir_write_add_slash(char **dirname)
 {
   int file_d;
   char *tmpname, *indir=*dirname/*, buf[]="A test"*/;
 
   /* Set the template for the temporary file: */
   if(indir[strlen(indir)-1]=='/')
-    tmpname=malloccat(indir, "gnuastroXXXXXX");
+    tmpname=gal_checkset_malloc_cat(indir, "gnuastroXXXXXX");
   else
-    tmpname=malloccat(indir, "/gnuastroXXXXXX");
+    tmpname=gal_checkset_malloc_cat(indir, "/gnuastroXXXXXX");
 
   /* Make a temporary file name and try openning it. */
   errno=0;
   file_d=mkstemp(tmpname);
   if(file_d==-1)
-    error(EXIT_FAILURE, errno, "Cannot write output in the directory %s",
+    error(EXIT_FAILURE, errno, "cannot write output in the directory %s",
 	  indir);
   /*
   errno=0;
   printf("\n\n%s\n\n", tmpname);
   if( write(file_d, buf, strlen(buf)) == -1 )
-    error(EXIT_FAILURE, errno, "%s: Writing to this temporary file to "
+    error(EXIT_FAILURE, errno, "%s: writing to this temporary file to "
 	  "check the given `%s` directory", tmpname, indir);
   */
   errno=0;
@@ -766,7 +893,7 @@ checkdirwriteaddslash(char **dirname)
   /* Delete the temporary file: */
   errno=0;
   if(unlink(tmpname)==-1)
-    error(EXIT_FAILURE, errno, "%s: Removing this temporary file made "
+    error(EXIT_FAILURE, errno, "%s: removing this temporary file made "
 	  "to check the given `%s directory`", tmpname, indir);
 
   /* Remove the extra characters that were added for the random name. */
