@@ -18,7 +18,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with gnuastro. If not, see <http://www.gnu.org/licenses/>.
+along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 **********************************************************************/
 #include <config.h>
 
@@ -30,11 +30,13 @@ along with gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <float.h>
 #include <stdlib.h>
 
-#include "timing.h"	/* Includes time.h and sys/time.h   */
-#include "txtarrayvv.h"
-#include "linkedlist.h"
-#include "configfiles.h"
-#include "fitsarrayvv.h"
+#include <nproc.h>              /* From Gnulib.                     */
+
+#include <gnuastro/timing.h>    /* Includes time.h and sys/time.h   */
+#include <gnuastro/txtarrayvv.h>
+#include <gnuastro/linkedlist.h>
+#include <gnuastro/configfiles.h>
+#include <gnuastro/fitsarrayvv.h>
 
 #include "main.h"
 
@@ -66,7 +68,7 @@ readconfig(char *filename, struct converttparams *p)
   size_t lineno=0, len=200;
   char *line, *name, *value;
   struct uiparams *up=&p->up;
-  struct commonparams *cp=&p->cp;
+  struct gal_commonparams *cp=&p->cp;
   char key='a';	/* Not used, just a place holder. */
 
   /* When the file doesn't exist or can't be opened, it is ignored. It
@@ -90,43 +92,44 @@ readconfig(char *filename, struct converttparams *p)
   while(getline(&line, &len, fp) != -1)
     {
       /* Prepare the "name" and "value" strings, also set lineno. */
-      STARTREADINGLINE;
+      GAL_CONFIGFILES_START_READING_LINE;
 
       /* Inputs: */
       if(strcmp(name, "hdu")==0)
-        allocatecopyset(value, &cp->hdu, &cp->hduset);
+        gal_checkset_allocate_copy_set(value, &cp->hdu, &cp->hduset);
       else if(strcmp(name, "hdu2")==0)
-        allocatecopyset(value, &up->hdu2, &up->hdu2set);
+        gal_checkset_allocate_copy_set(value, &up->hdu2, &up->hdu2set);
       else if(strcmp(name, "hdu3")==0)
-        allocatecopyset(value, &up->hdu3, &up->hdu3set);
+        gal_checkset_allocate_copy_set(value, &up->hdu3, &up->hdu3set);
       else if(strcmp(name, "hdu4")==0)
-        allocatecopyset(value, &up->hdu4, &up->hdu4set);
+        gal_checkset_allocate_copy_set(value, &up->hdu4, &up->hdu4set);
 
 
       /* Outputs: */
       else if(strcmp(name, "output")==0)
-        allocatecopyset(value, &cp->output, &cp->outputset);
+        gal_checkset_allocate_copy_set(value, &cp->output, &cp->outputset);
 
       else if(strcmp(name, "quality")==0)
 	{
 	  if(up->qualityset) continue;
-          intsmallerequalto(value, &p->quality, name, key,
-                            p->cp.spack, filename, lineno, 100);
+          gal_checkset_int_smaller_equal_to(value, &p->quality, name, key,
+                                            p->cp.spack, filename, lineno, 100);
           if(p->quality<0)
-            error(EXIT_FAILURE, 0, "The quality option should be positive.");
+            error(EXIT_FAILURE, 0, "the quality option should be positive");
 	  up->qualityset=1;
 	}
       else if(strcmp(name, "widthincm")==0)
 	{
 	  if(up->widthincmset) continue;
-          floatl0(value, &p->widthincm, name, key, SPACK, filename, lineno);
+          gal_checkset_float_l_0(value, &p->widthincm, name, key, SPACK,
+                                 filename, lineno);
 	  up->widthincmset=1;
 	}
       else if(strcmp(name, "borderwidth")==0)
 	{
 	  if(up->borderwidthset) continue;
-          intelzero(value, &p->borderwidth, name, key, SPACK,
-                    filename, lineno);
+          gal_checkset_int_el_zero(value, &p->borderwidth, name, key, SPACK,
+                                   filename, lineno);
 	  up->borderwidthset=1;
 	}
 
@@ -138,32 +141,33 @@ readconfig(char *filename, struct converttparams *p)
       else if(strcmp(name, "fluxlow")==0)
 	{
 	  if(up->fluxlowset) continue;
-          anydouble(value, &p->fluxlow, name, key, p->cp.spack,
-                   filename, lineno);
+          gal_checkset_any_double(value, &p->fluxlow, name, key, p->cp.spack,
+                                  filename, lineno);
           up->fluxlowset=1;
 	}
       else if(strcmp(name, "fluxhigh")==0)
 	{
 	  if(up->fluxhighset) continue;
-          anydouble(value, &p->fluxhigh, name, key, p->cp.spack,
-                   filename, lineno);
+          gal_checkset_any_double(value, &p->fluxhigh, name, key, p->cp.spack,
+                                  filename, lineno);
           up->fluxhighset=1;
 	}
       else if(strcmp(name, "maxbyte")==0)
 	{
 	  if(up->maxbyteset) continue;
-          intsmallerequalto(value, &tmp, "maxbyte", key,
-                            p->cp.spack, NULL, 0, UINT8_MAX);
+          gal_checkset_int_smaller_equal_to(value, &tmp, "maxbyte", key,
+                                            p->cp.spack, NULL, 0, UINT8_MAX);
           if(tmp<0)
-            error(EXIT_FAILURE, 0, "--maxbyte (-m) should be positive.");
+            error(EXIT_FAILURE, 0, "--maxbyte (-m) should be positive");
           p->maxbyte=tmp;
           p->up.maxbyteset=1;
 	}
 
 
 
+      /* Operating modes: */
       /* Read options common to all programs */
-      READ_COMMONOPTIONS_FROM_CONF
+      GAL_CONFIGFILES_READ_COMMONOPTIONS_FROM_CONF
 
 
       else
@@ -183,19 +187,19 @@ void
 printvalues(FILE *fp, struct converttparams *p)
 {
   struct uiparams *up=&p->up;
-  struct commonparams *cp=&p->cp;
+  struct gal_commonparams *cp=&p->cp;
 
   /* Print all the options that are set. Separate each group with a
      commented line explaining the options in that group. */
   fprintf(fp, "\n# Input image:\n");
   if(cp->hduset)
-    PRINTSTINGMAYBEWITHSPACE("hdu", cp->hdu);
+    GAL_CHECKSET_PRINT_STRING_MAYBE_WITH_SPACE("hdu", cp->hdu);
   if(up->hdu2set)
-    PRINTSTINGMAYBEWITHSPACE("hdu2", up->hdu2);
+    GAL_CHECKSET_PRINT_STRING_MAYBE_WITH_SPACE("hdu2", up->hdu2);
   if(up->hdu3set)
-    PRINTSTINGMAYBEWITHSPACE("hdu3", up->hdu3);
+    GAL_CHECKSET_PRINT_STRING_MAYBE_WITH_SPACE("hdu3", up->hdu3);
   if(up->hdu4set)
-    PRINTSTINGMAYBEWITHSPACE("hdu4", up->hdu4);
+    GAL_CHECKSET_PRINT_STRING_MAYBE_WITH_SPACE("hdu4", up->hdu4);
 
 
   fprintf(fp, "\n# Output parameters:\n");
@@ -219,8 +223,9 @@ printvalues(FILE *fp, struct converttparams *p)
 
   /* For the operating mode, first put the macro to print the common
      options, then the (possible options particular to this
-     program) */
-  PRINT_COMMONOPTIONS;
+     program). */
+  fprintf(fp, "\n# Operating modes:\n");
+  GAL_CONFIGFILES_PRINT_COMMONOPTIONS;
 }
 
 
@@ -233,31 +238,31 @@ void
 checkifset(struct converttparams *p)
 {
   struct uiparams *up=&p->up;
-  struct commonparams *cp=&p->cp;
+  struct gal_commonparams *cp=&p->cp;
 
   int intro=0;
   if(cp->hduset==0)
-    REPORT_NOTSET("hdu");
+    GAL_CONFIGFILES_REPORT_NOTSET("hdu");
   if(up->hdu2set==0)
-    REPORT_NOTSET("hdu2");
+    GAL_CONFIGFILES_REPORT_NOTSET("hdu2");
   if(up->hdu3set==0)
-    REPORT_NOTSET("hdu3");
+    GAL_CONFIGFILES_REPORT_NOTSET("hdu3");
   if(up->hdu4set==0)
-    REPORT_NOTSET("hdu4");
+    GAL_CONFIGFILES_REPORT_NOTSET("hdu4");
   if(up->qualityset==0)
-    REPORT_NOTSET("quality");
+    GAL_CONFIGFILES_REPORT_NOTSET("quality");
   if(up->widthincmset==0)
-    REPORT_NOTSET("widthincm");
+    GAL_CONFIGFILES_REPORT_NOTSET("widthincm");
   if(up->borderwidthset==0)
-    REPORT_NOTSET("borderwidth");
+    GAL_CONFIGFILES_REPORT_NOTSET("borderwidth");
   if(up->fluxlowset==0)
-    REPORT_NOTSET("fluxlow");
+    GAL_CONFIGFILES_REPORT_NOTSET("fluxlow");
   if(up->fluxhighset==0)
-    REPORT_NOTSET("fluxhigh");
+    GAL_CONFIGFILES_REPORT_NOTSET("fluxhigh");
   if(up->maxbyteset==0)
-    REPORT_NOTSET("maxbyte");
+    GAL_CONFIGFILES_REPORT_NOTSET("maxbyte");
 
-  END_OF_NOTSET_REPORT;
+  GAL_CONFIGFILES_END_OF_NOTSET_REPORT;
 }
 
 
@@ -356,7 +361,7 @@ void
 adddotautomaticoutput(struct converttparams *p)
 {
   size_t i;
-  struct commonparams *cp=&p->cp;
+  struct gal_commonparams *cp=&p->cp;
   char *tmp, *basename="output.txt";
 
   /* Find the first file name in the input(s). */
@@ -373,7 +378,7 @@ adddotautomaticoutput(struct converttparams *p)
       errno=0;
       tmp=malloc(strlen(cp->output)+10*sizeof *tmp);
       if(tmp==NULL)
-        error(EXIT_FAILURE, errno, "%lu bytes for suffix name.",
+        error(EXIT_FAILURE, errno, "%lu bytes for suffix name",
               strlen(cp->output)+10*sizeof *tmp);
       sprintf(tmp, ".%s", cp->output);
       free(cp->output);
@@ -381,10 +386,10 @@ adddotautomaticoutput(struct converttparams *p)
     }
 
   /* Set the automatic output and make sure we have write access. */
-  automaticoutput(basename, cp->output, cp->removedirinfo,
+  gal_checkset_automatic_output(basename, cp->output, cp->removedirinfo,
                   cp->dontdelete, &cp->output);
-  if( dir0file1(cp->output, cp->dontdelete)==0 )
-    error(EXIT_FAILURE, 0, "%s is a directory.", cp->output);
+  if( gal_checkset_dir_0_file_1(cp->output, cp->dontdelete)==0 )
+    error(EXIT_FAILURE, 0, "%s is a directory", cp->output);
 }
 
 
@@ -395,21 +400,21 @@ void
 sanitycheck(struct converttparams *p)
 {
   size_t i, j;
-  struct commonparams *cp=&p->cp;
+  struct gal_commonparams *cp=&p->cp;
 
   /* The flux range: */
   if(p->fluxlow>p->fluxhigh)
-    error(EXIT_FAILURE, 0, "The value of `--fluxlow` (`-L`, %.3f) is "
-          "larger than `--fluxhigh` (`-H`, %.3f).", p->fluxlow, p->fluxhigh);
+    error(EXIT_FAILURE, 0, "the value of `--fluxlow` (`-L`, %.3f) is "
+          "larger than `--fluxhigh` (`-H`, %.3f)", p->fluxlow, p->fluxhigh);
 
   /* Make sure there are 1 (for grayscale), 3 (for RGB) or 4 (for
      CMYK) color channels. */
   if(p->numch!=1 && p->numch!=3 && p->numch!=4)
-    error(EXIT_FAILURE, 0, "The number of input color channels has to be "
+    error(EXIT_FAILURE, 0, "the number of input color channels has to be "
           "1 (for non image data, grayscale or only K channel in CMYK), "
           "3 (for RGB) and 4 (for CMYK). You have given %lu color channels. "
           "Note that some file formats (for example JPEG) can contain more "
-          "than one color channel.", p->numch);
+          "than one color channel", p->numch);
 
   /* Make sure that there is atleast one input file (not only blank)
      and set the sizes of the blank channels to the first non-blank
@@ -417,7 +422,7 @@ sanitycheck(struct converttparams *p)
   for(i=0;i<p->numch;++i)
     if(p->isblank[i]==0) break;
   if(i==p->numch)
-    error(EXIT_FAILURE, 0, "All the input(s) are of type blank!");
+    error(EXIT_FAILURE, 0, "all the input(s) are of type blank");
   for(j=0;j<p->numch;++j)
     if(p->isblank[j])
       {
@@ -436,8 +441,8 @@ sanitycheck(struct converttparams *p)
           for(i=0;i<p->numch;++i)
             fprintf(stderr, "Channel %lu is %lu x %lu pixels.\n", i,
                     p->s1[i], p->s0[i]);
-          error(EXIT_FAILURE, 0, "The input color channels have different "
-                "sizes.");
+          error(EXIT_FAILURE, 0, "the input color channels have different "
+                "sizes");
         }
     }
 
@@ -448,26 +453,26 @@ sanitycheck(struct converttparams *p)
         errno=0;
         p->ch[i]=calloc(p->s0[0]*p->s1[0], sizeof *p->ch[i]);
         if(p->ch[i]==NULL)
-          error(EXIT_FAILURE, errno, "Allocating %lu bytes for the blank "
+          error(EXIT_FAILURE, errno, "allocating %lu bytes for the blank "
                 "channel %lu", p->s0[0]*p->s1[0]*sizeof *p->ch[i], i);
       }
 
   /* The output file name. First find the first non-blank file name: */
   if(cp->outputset)
     {
-      if(nameisfits(cp->output))
+      if(gal_fitsarray_name_is_fits(cp->output))
         {
           p->outputtype=FITSFORMAT;
-          if( nameisfitssuffix(cp->output) )
+          if( gal_fitsarray_name_is_fits_suffix(cp->output) )
             adddotautomaticoutput(p);
         }
       else if(nameisjpeg(cp->output))
         {
 #ifndef HAS_LIBJPEG
-          error(EXIT_FAILURE, 0, "You have asked for a JPEG output, however, "
+          error(EXIT_FAILURE, 0, "you have asked for a JPEG output, however, "
                 "when %s was configured libjpeg was not available. To write "
                 "to JPEG files, libjpeg is required. Please install it and "
-                "configure, make and install %s again.", PACKAGE_STRING,
+                "configure, make and install %s again", PACKAGE_STRING,
                 PACKAGE_STRING);
 #else
           p->outputtype=JPEGFORMAT;
@@ -492,10 +497,10 @@ sanitycheck(struct converttparams *p)
           /* If the length of the name is shorter than 4 characters, it is
              most probably a mis-spelled extension, warn the user. */
           if(strlen(cp->output)<=5)
-            error(EXIT_FAILURE, 0, ": (Warning) Your output file name is "
+            error(EXIT_FAILURE, 0, "your output file name is "
                     "`%s`, based on its length, it might be a mis-spelled "
                     "extension. Your input is converted to a plain text "
-                    "format file with That name.",
+                    "format file with That name",
                     cp->output);
 
           p->outputtype=TXTFORMAT;
@@ -503,23 +508,23 @@ sanitycheck(struct converttparams *p)
           /* If output type is not an image, there should only be one color
              channel: */
           if(p->numch>1)
-            error(EXIT_FAILURE, 0, "Text output (`--output=%s`) can only be "
+            error(EXIT_FAILURE, 0, "text output (`--output=%s`) can only be "
                   "completed with one input color channel. You have given "
                   "%lu. Note that some formats (for example JPEG) can have "
                   "more than one color channel in each file. You can first "
                   "convert the file to FITS, then convert the desired "
-                  "channel to text by specifying the HDU.",
+                  "channel to text by specifying the HDU",
                   cp->output, p->numch);
         }
     }
   else
-    error(EXIT_FAILURE, 0, "No ouput file name or extension is specified, "
+    error(EXIT_FAILURE, 0, "no ouput file name or extension is specified, "
           "Please run "SPACK" again with the `--output' or `-o' option.\n\n"
           "The value to this option doesn't have to be a file name, you "
           "can also only give an extension (in which case your input file"
           "name will be used for the base name of the output. For example "
           "running `"SPACK" filename.fits -ojpg') will produce the output "
-          "file `filename.jpg').");
+          "file `filename.jpg')");
 }
 
 
@@ -550,7 +555,7 @@ preparearrays(struct converttparams *p)
   size_t i;
   void *array;
   double *d, *df;
-  struct stll *tmp;
+  struct gal_linkedlist_stll *tmp;
   char *hdu=NULL, **names=p->names;
 
   /* Put the names in the correct order. */
@@ -563,29 +568,34 @@ preparearrays(struct converttparams *p)
     {
       /* Check if p->numch has not exceeded 4. */
       if(p->numch>=4)
-        error(EXIT_FAILURE, 0, "The number of input color channels (not "
+        error(EXIT_FAILURE, 0, "the number of input color channels (not "
               "files) has exceeded 4! Note that one file can contain more "
-              "than one color channel.");
+              "than one color channel");
+
+      /* Make sure this input file exists (if it isn't blank). */
+      if(strcmp(names[i], "blank")) gal_checkset_check_file(names[i]);
 
       /* FITS: */
-      if( nameisfits(names[i]) )
+      if( gal_fitsarray_name_is_fits(names[i]) )
         {
           switch(p->numch) /* Get the HDU value for this channel. */
             {
             case 0: hdu=p->cp.hdu; break;   case 1: hdu=p->up.hdu2; break;
             case 2: hdu=p->up.hdu3; break;  case 3: hdu=p->up.hdu4; break;
-            default: error(EXIT_FAILURE, 0, "A bug! In parsing the input "
+            default: error(EXIT_FAILURE, 0, "a bug! In parsing the input "
                            "FITS files, it has gone beyond four! Please "
                            "contact us so we can see what caused this "
-                           "problem and fix it.");
+                           "problem and fix it");
             }
-          p->numnul[p->numch]=fitsimgtoarray(names[i], hdu,
-                                             &p->bitpixs[p->numch], &array,
-                                             &p->s0[p->numch],
-                                             &p->s1[p->numch]);
-          changetype(array, p->bitpixs[p->numch],
-                     p->s0[p->numch]*p->s1[p->numch], p->numnul[p->numch],
-                     (void **)(&p->ch[p->numch]), DOUBLE_IMG);
+          p->numnul[p->numch]=
+            gal_fitsarray_fits_img_to_array(names[i], hdu,
+                                            &p->bitpixs[p->numch], &array,
+                                            &p->s0[p->numch],
+                                            &p->s1[p->numch]);
+          gal_fitsarray_change_type(array, p->bitpixs[p->numch],
+                                    p->s0[p->numch]*p->s1[p->numch],
+                                    p->numnul[p->numch],
+                                    (void **)(&p->ch[p->numch]), DOUBLE_IMG);
           free(array);
           ++p->numch;
         }
@@ -596,10 +606,10 @@ preparearrays(struct converttparams *p)
       else if ( nameisjpeg(names[i]) )
         {
 #ifndef HAS_LIBJPEG
-          error(EXIT_FAILURE, 0, "You are giving a JPEG input, however, "
+          error(EXIT_FAILURE, 0, "you are giving a JPEG input, however, "
                 "when %s was configured libjpeg was not available. To read "
                 "from JPEG files, libjpeg is required. Please install it and "
-                "configure, make and install %s again.", PACKAGE_STRING,
+                "configure, make and install %s again", PACKAGE_STRING,
                 PACKAGE_STRING);
 #else
           preparejpeg(p, names[i]);
@@ -622,7 +632,7 @@ preparearrays(struct converttparams *p)
       else if ( nameiseps(names[i]) )
         error(EXIT_FAILURE, 0, "EPS files cannot be used as input. Since "
               "EPS files are not raster graphics, they are only used as "
-              "output.");
+              "output");
 
 
 
@@ -630,21 +640,21 @@ preparearrays(struct converttparams *p)
       else if ( nameispdf(names[i]) )
         error(EXIT_FAILURE, 0, "PDF files cannot be used as input. Since "
               "PDF files are not raster graphics, they are only used as "
-              "output.");
+              "output");
 
 
       /* Text: */
       else
         {
-          txttoarray(names[i], &p->ch[p->numch],
-                     &p->s0[p->numch], &p->s1[p->numch]);
+          gal_txtarray_txt_to_array(names[i], &p->ch[p->numch],
+                                    &p->s0[p->numch], &p->s1[p->numch]);
           df = (d=p->ch[p->numch]) + p->s0[p->numch]*p->s1[p->numch];
           do if(isnan(*d++)) break; while(d<df);
           if(d==df)
-            checkremovefile(TXTARRAYVVLOG, 0);
+            gal_checkset_check_remove_file(GAL_TXTARRAY_LOG, 0);
           else
-            error(EXIT_FAILURE, 0, "%s contains non-numeric data, see %s.",
-                  names[i], TXTARRAYVVLOG);
+            error(EXIT_FAILURE, 0, "%s contains non-numeric data, see %s",
+                  names[i], GAL_TXTARRAY_LOG);
           p->bitpixs[p->numch]=DOUBLE_IMG;
           ++p->numch;
         }
@@ -674,30 +684,30 @@ preparearrays(struct converttparams *p)
 void
 setparams(int argc, char *argv[], struct converttparams *p)
 {
-  struct commonparams *cp=&p->cp;
+  struct gal_commonparams *cp=&p->cp;
 
   /* Set the non-zero initial values, the structure was initialized to
      have a zero value for all elements. */
   cp->spack         = SPACK;
   cp->verb          = 1;
-  cp->numthreads    = DP_NUMTHREADS;
+  cp->numthreads    = num_processors(NPROC_CURRENT);
   cp->removedirinfo = 1;
   p->invert         = 1;
 
   /* Read the arguments. */
   errno=0;
   if(argp_parse(&thisargp, argc, argv, 0, 0, p))
-    error(EXIT_FAILURE, errno, "Parsing arguments");
+    error(EXIT_FAILURE, errno, "parsing arguments");
 
   /* Add the user default values and save them if asked. */
-  CHECKSETCONFIG;
+  GAL_CONFIGFILES_CHECK_SET_CONFIG;
 
   /* Check if all the required parameters are set. */
   checkifset(p);
 
   /* Print the values for each parameter. */
   if(cp->printparams)
-    REPORT_PARAMETERS_SET;
+    GAL_CONFIGFILES_REPORT_PARAMETERS_SET;
 
   /* Prepare the arrays: */
   preparearrays(p);
@@ -732,7 +742,7 @@ void
 freeandreport(struct converttparams *p)
 {
   size_t i;
-  struct stll *tmp, *ttmp;
+  struct gal_linkedlist_stll *tmp, *ttmp;
 
   free(p->cp.hdu);
   free(p->up.hdu2);
