@@ -18,7 +18,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with gnuastro. If not, see <http://www.gnu.org/licenses/>.
+along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 **********************************************************************/
 #include <config.h>
 
@@ -29,10 +29,10 @@ along with gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <stdlib.h>
 #include <gsl/gsl_errno.h>
 
-#include "timing.h"
-#include "astrthreads.h"
-#include "fitsarrayvv.h"
-#include "spatialconvolve.h"
+#include <gnuastro/timing.h>
+#include <gnuastro/threads.h>
+#include <gnuastro/fitsarrayvv.h>
+#include <gnuastro/spatialconvolve.h>
 
 #include "main.h"
 #include "convolve.h"
@@ -73,9 +73,9 @@ complextoreal(double *c, size_t size, int action, double **output)
       do { *o++ = *c;                            c+=2; } while(o<of);
       break;
     default:
-      error(EXIT_FAILURE, 0, "A bug! Please contact us at %s so we can "
+      error(EXIT_FAILURE, 0, "a bug! Please contact us at %s so we can "
             "correct it. For some reason, the action code for complextoreal "
-            "in convolve.c (%d) is not recognized.", PACKAGE_BUGREPORT,
+            "in convolve.c (%d) is not recognized", PACKAGE_BUGREPORT,
             action);
     }
 }
@@ -87,6 +87,15 @@ complextoreal(double *c, size_t size, int action, double **output)
 /* Multily two complex arrays and save the result in the first:
 
    (a+ib)*(c+id)=ac+iad+ibc-bd=(ac-bd)+i(ad-bc)
+
+   The loop is easy to understand: we want to replace two
+   variables. But changing one, will affect the other. So what we do,
+   is to store the final value of one, then replace the second, then
+   finally replace the first one.
+
+   Here, we first get the real component but don't put it in the
+   output. Then we find and replace the imaginary component, finally,
+   we put the new real component in the image.
  */
 void
 complexarraymultiply(double *a, double *b, size_t size)
@@ -98,7 +107,7 @@ complexarraymultiply(double *a, double *b, size_t size)
     {
       r      = (*a * *b) - (*(a+1) * *(b+1));
       *(a+1) = (*(a+1) * *b) + (*a * *(b+1));
-      *a++=r;     /* Go onto the imaginary part of a. */
+      *a++=r;            /* Go onto (set) the imaginary part of a. */
       b+=2;
     }
   while(++a<af);  /* Go onto the next complex number. */
@@ -108,6 +117,52 @@ complexarraymultiply(double *a, double *b, size_t size)
 
 
 
+/* Divide the elements of the first array by the elements of the
+   second array and put the result into the elements of the first
+   array.
+
+   (a+ib)/(c+id)=[(a+ib)*(c-id)]/[(c+id)*(c-id)]
+                =(ac-iad+ibc+bd)/(c^2+d^2)
+                =[(ac+bd)+i(bc-ad)]/(c^2+d^2)
+
+   See the explanations above complexarraymultiply for an explanation
+   on the loop.
+ */
+void
+complexarraydivide(double *a, double *b, size_t size)
+{
+  double r, *af;
+
+  af=a+2*size;
+  do
+    {
+      if (sqrt(*b**b + *(b+1)**(b+1))>MINGOODDIVSPEC)
+        {
+          r      = ( ( (*a * *b) + (*(a+1) * *(b+1)) )
+                     / ( *b * *b + *(b+1) * *(b+1) ) );
+          *(a+1) = ( ( (*(a+1) * *b) - (*a * *(b+1)) )
+                     / ( *b * *b + *(b+1) * *(b+1) ) );
+          *a=r;
+
+          if(sqrt(*a**a + *(a+1)**(a+1))>1.00001f)
+            *a=*(a+1)=0.0f;
+        }
+      else
+        {
+          *a=0;
+          *(a+1)=0;
+        }
+
+      a+=2;
+      b+=2;
+    }
+  while(a<af);  /* Go onto the next complex number. */
+}
+
+
+
+
+
 
 
 
@@ -137,13 +192,19 @@ makepaddedcomplex(struct convolveparams *p)
   /* Find the sizes of the padded image, note that since the kernel
      sizes are always odd, the extra padding on the input image is
      always going to be an even number (clearly divisable). */
-  ps0=p->ps0=p->is0+p->ks0-1;   /* Find the padded image size. */
-  ps1=p->ps1=p->is1+p->ks1-1;
+  ps0=p->ps0 = p->makekernel ? p->is0 : p->is0+p->ks0-1;
+  ps1=p->ps1 = p->makekernel ? p->is1 : p->is1+p->ks1-1;
 
-  errno=0;                      /* Pad the input image.        */
+  /* The Discrete Fourier transforms operate faster on even-sized
+     arrays. So if the padded sides are not even, make them so: */
+  if(ps0%2) ps0=p->ps0=ps0+1;
+  if(ps1%2) ps1=p->ps1=ps1+1;
+
+  /* Allocate the space for the padded input image and fill it. */
+  errno=0;
   pimg=p->pimg=malloc(2*ps0*ps1*sizeof *pimg);
   if(pimg==NULL)
-    error(EXIT_FAILURE, errno, "%lu bytes for pimg.", ps0*ps1*sizeof *pimg);
+    error(EXIT_FAILURE, errno, "%lu bytes for pimg", ps0*ps1*sizeof *pimg);
   for(i=0;i<ps0;++i)
     {
       op=(o=pimg+i*2*ps1)+2*ps1; /* pimg is complex.            */
@@ -156,10 +217,11 @@ makepaddedcomplex(struct convolveparams *p)
     }
 
 
-  errno=0;                      /* Pad the Kernel.             */
+  /* Allocate the space for the padded Kernel and fill it. */
+  errno=0;
   pker=p->pker=malloc(2*ps0*ps1*sizeof *pker);
   if(pker==NULL)
-    error(EXIT_FAILURE, errno, "%lu bytes for pimg.", ps0*ps1*sizeof *pker);
+    error(EXIT_FAILURE, errno, "%lu bytes for pker", ps0*ps1*sizeof *pker);
   for(i=0;i<ps0;++i)
     {
       op=(o=pker+i*2*ps1)+2*ps1; /* pker is complex.            */
@@ -188,11 +250,28 @@ removepaddingcorrectroundoff(struct convolveparams *p)
 {
   size_t ps1=p->ps1;
   float *o, *input=p->input;
+  size_t i, hi0, hi1, is0, is1;
   double *d, *df, *start, *pimg=p->pimg;
-  size_t i, hi0, hi1, is0=p->is0, is1=p->is1;
 
-  hi0=(p->ks0-1)/2;
-  hi1=(p->ks1-1)/2;
+  /* Set all the necessary parameters to crop the desired region. hi0
+     and hi1 are the coordinates of the first pixel in the output
+     image. In the case of deconvolution, if the maximum radius is
+     larger smaller than the input image, we will also only be using
+     region that contains non-zero rows and columns.*/
+  if(p->makekernel)
+    {
+      hi0 = 2*p->makekernel-1<p->is0 ? p->ps0/2-p->makekernel : 0;
+      hi1 = 2*p->makekernel-1<p->is1 ? p->ps1/2-p->makekernel : 0;
+      p->is0 = 2*p->makekernel-1<p->is0 ? 2*p->makekernel-1 : p->is0;
+      p->is1 = 2*p->makekernel-1<p->is1 ? 2*p->makekernel-1 : p->is1;
+    }
+  else
+    {
+      hi0=(p->ks0-1)/2;
+      hi1=(p->ks1-1)/2;
+    }
+  is0=p->is0;
+  is1=p->is1;
 
   /* To start with, `start' points to the first pixel in the final
      image: */
@@ -209,6 +288,10 @@ removepaddingcorrectroundoff(struct convolveparams *p)
     }
 }
 
+
+
+
+
 /* Allocate the necessary arrays, note that we put everything in the
    first element of the fftonthreadparams structure array. All the
    other elements will point to this one later. This structure will be
@@ -265,6 +348,77 @@ freefp(struct fftonthreadparams *fp)
 
 
 
+/* Unfortunately I don't understand why the division operation in
+   deconvolution (makekernel) does not produce a centered image, the
+   image is translated by half the input size in both dimensions. So I
+   am correcting this in the spatial domain here. */
+void
+correctdeconvolve(struct convolveparams *p, double **spatial)
+{
+  double r, *s, *n, *d, *df, sum=0.0f;
+  size_t i, j, ps0=p->ps0, ps1=p->ps1;
+  int ii, jj, ci=p->ps0/2-1, cj=p->ps1/2-1;
+
+  /* Check if the image has even sides. */
+  if(ps0%2 || ps1%2)
+    error(EXIT_FAILURE, 0, "a bug! Please contact us at %s. In "
+          "correctdeconvolve, the padded image sides are not an "
+          "even number", PACKAGE_BUGREPORT);
+
+  /* First convert the complex image to a real image: */
+  complextoreal(p->pimg, ps0*ps1, COMPLEXTOREALSPEC, &s);
+
+  /* Allocate the array to keep the new values */
+  errno=0;
+  n=malloc(ps0*ps1*sizeof *n);
+  if(n==NULL)
+    error(EXIT_FAILURE, errno, "%lu bytes for n in correctdeconvolve "
+          "(convolve.c). ", ps0*ps1*sizeof *n);
+
+
+  /* Put the elements in their proper place: For example in one
+     dimention where the values are actually the true distances:
+
+        s[0]=0, s[1]=1, s[2]=2, s[3]=3, s[4]=4, s[5]=5
+
+     We want the value 0 to be in the `center'. Note that `s' is
+     periodic, for example the next 6 elements have distances:
+
+        s[6]=0, s[7]=1, s[8]=2, s[9]=3, s[10]=4, s[11]=5
+
+     So a `center'ed array would be like:
+
+        s[0]=4, s[1]=5, s[2]=0, s[3]=1, s[4]=2, s[5]=3
+
+     The relations between the old (i and j) and new (ii and jj) come
+     from something like the above line.
+   */
+  for(i=0;i<ps0;++i)
+    {
+      ii= i>ps0/2 ? i-(ps0/2+1) : i+ps0/2-1;
+      for(j=0;j<ps1;++j)
+        {
+          jj = j>ps1/2 ? j-(ps1/2+1) : j+ps1/2-1;
+
+          r=sqrt( (ii-ci)*(ii-ci) + (jj-cj)*(jj-cj) );
+          sum += n[ii*ps1+jj] = r < p->makekernel ? s[i*ps1+j] : 0;
+
+          /*printf("(%lu, %lu) --> (%lu, %lu)\n", i, j, ii, jj);*/
+        }
+    }
+
+
+  /* Divide all elements by the sum so the kernel is normalized: */
+  df=(d=n)+ps0*ps1; do *d++/=sum; while(d<df);
+
+
+  /* Clean up: */
+  free(s);
+  *spatial=n;
+}
+
+
+
 
 
 
@@ -327,11 +481,11 @@ onedimensionfft(void *inparam)
      by each other) is stored in p->pimg. So the check below works
      both in the forward and the backward transformation.
   */
-  for(i=0;indexs[i]!=NONTHRDINDEX;++i)
+  for(i=0;indexs[i]!=GAL_THREADS_NON_THRD_INDEX;++i)
     {
       data = ( indexs[i]<maxindex
                ? &pimg[ 2*indexs[i]*indmultip ]   /* *2 because complex. */
-               : &pker[ 2*(indexs[i]-maxindex)*indmultip ]);
+               : &pker[ 2*(indexs[i]-maxindex)*indmultip ] );
 
       gsl_fft_complex_transform(data, stride, size, wavetable, work,
                                 forward1backwardn1);
@@ -375,16 +529,16 @@ twodimensionfft(struct convolveparams *p, struct fftonthreadparams *fp,
   if(forward1backwardn1==1)       multiple=2;
   else if(forward1backwardn1==-1) multiple=1;
   else
-    error(EXIT_FAILURE, 0, "A bug! In forward2dfft, the value of the "
+    error(EXIT_FAILURE, 0, "a bug! In forward2dfft, the value of the "
           "variable forward1backwardn1 is somehow not 1 or 2, but %d. "
           "Please contact us at "PACKAGE_BUGREPORT" so we can find the "
-          "cause of the problem and fix it.", forward1backwardn1);
+          "cause of the problem and fix it", forward1backwardn1);
 
 
   /* ==================== */
   /* 1D FFT on each row. */
   /* ==================== */
-  distinthreads(multiple*p->ps0, nt, &indexs, &thrdcols);
+  gal_threads_dist_in_threads(multiple*p->ps0, nt, &indexs, &thrdcols);
   if(nt==1)
     {
       fp[0].stride=1;
@@ -400,11 +554,11 @@ twodimensionfft(struct convolveparams *p, struct fftonthreadparams *fp,
 	 threads spinned off. */
       if( multiple*p->ps0 < nt ) nb=multiple*p->ps0+1;
       else nb=nt+1;
-      attrbarrierinit(&attr, &b, nb);
+      gal_threads_attr_barrier_init(&attr, &b, nb);
 
       /* Spin off the threads: */
       for(i=0;i<nt;++i)
-        if(indexs[i*thrdcols]!=NONTHRDINDEX)
+        if(indexs[i*thrdcols]!=GAL_THREADS_NON_THRD_INDEX)
           {
             fp[i].id=i;
             fp[i].b=&b;
@@ -413,7 +567,7 @@ twodimensionfft(struct convolveparams *p, struct fftonthreadparams *fp,
             fp[i].forward1backwardn1=forward1backwardn1;
 	    err=pthread_create(&t, &attr, onedimensionfft, &fp[i]);
 	    if(err)
-	      error(EXIT_FAILURE, 0, "Can't create thread %lu for rows.",
+	      error(EXIT_FAILURE, 0, "can't create thread %lu for rows",
                     i);
           }
 
@@ -430,7 +584,7 @@ twodimensionfft(struct convolveparams *p, struct fftonthreadparams *fp,
   /* 1D FFT on each column. */
   /* ====================== */
   /* No comments, exact duplicate, except the p->ps1s! */
-  distinthreads(multiple*p->ps1, nt, &indexs, &thrdcols);
+  gal_threads_dist_in_threads(multiple*p->ps1, nt, &indexs, &thrdcols);
   if(nt==1)
     {
       fp[0].stride=p->ps1;
@@ -442,9 +596,9 @@ twodimensionfft(struct convolveparams *p, struct fftonthreadparams *fp,
     {
       if( multiple*p->ps1 < nt ) nb=multiple*p->ps1+1;
       else nb=nt+1;
-      attrbarrierinit(&attr, &b, nb);
+      gal_threads_attr_barrier_init(&attr, &b, nb);
       for(i=0;i<nt;++i)
-        if(indexs[i*thrdcols]!=NONTHRDINDEX)
+        if(indexs[i*thrdcols]!=GAL_THREADS_NON_THRD_INDEX)
           {
             fp[i].b=&b;
             fp[i].stride=p->ps1; /* On each column, stride is p->ps1 */
@@ -452,7 +606,7 @@ twodimensionfft(struct convolveparams *p, struct fftonthreadparams *fp,
             fp[i].forward1backwardn1=forward1backwardn1;
 	    err=pthread_create(&t, &attr, onedimensionfft, &fp[i]);
 	    if(err)
-	      error(EXIT_FAILURE, 0, "Can't create thread %lu for columns.",
+	      error(EXIT_FAILURE, 0, "can't create thread %lu for columns",
                     i);
           }
       pthread_barrier_wait(&b);
@@ -473,20 +627,24 @@ frequencyconvolve(struct convolveparams *p)
   struct timeval t1;
   int verb=p->cp.verb;
   struct fftonthreadparams *fp;
+  char *operation = p->makekernel ? "Divided" : "Multiplied";
+
 
   /* Make the padded arrays. */
   if(verb) gettimeofday(&t1, NULL);
   makepaddedcomplex(p);
-  if(verb) reporttiming(&t1, "Input and Kernel images padded.", 1);
+  if(verb) gal_timing_report(&t1, "Input and Kernel images padded.", 1);
   if(p->viewfreqsteps)
     {
       complextoreal(p->pimg, p->ps0*p->ps1, COMPLEXTOREALREAL, &tmp);
-      arraytofitsimg(p->up.freqstepsname, "Input transform", DOUBLE_IMG,
-                     tmp, p->ps0, p->ps1, 0, NULL, NULL, SPACK_STRING);
+      gal_fitsarray_array_to_fits_img(p->up.freqstepsname, "Input padded",
+                                      DOUBLE_IMG, tmp, p->ps0, p->ps1, 0,
+                                      NULL, NULL, SPACK_STRING);
       free(tmp);
       complextoreal(p->pker, p->ps0*p->ps1, COMPLEXTOREALREAL, &tmp);
-      arraytofitsimg(p->up.freqstepsname, "Kernel transform", DOUBLE_IMG,
-                     tmp, p->ps0, p->ps1, 0, NULL, NULL, SPACK_STRING);
+      gal_fitsarray_array_to_fits_img(p->up.freqstepsname, "Kernel padded",
+                                      DOUBLE_IMG, tmp, p->ps0, p->ps1, 0,
+                                      NULL, NULL, SPACK_STRING);
       free(tmp);
     }
 
@@ -498,61 +656,67 @@ frequencyconvolve(struct convolveparams *p)
   /* Forward 2D FFT on each image. */
   if(verb) gettimeofday(&t1, NULL);
   twodimensionfft(p, fp, 1);
-  if(verb) reporttiming(&t1, "Images converted to frequency domain.", 1);
+  if(verb) gal_timing_report(&t1, "Images converted to frequency domain.", 1);
   if(p->viewfreqsteps)
     {
       complextoreal(p->pimg, p->ps0*p->ps1, COMPLEXTOREALSPEC, &tmp);
-      arraytofitsimg(p->up.freqstepsname, "Input transform", DOUBLE_IMG,
-                     tmp, p->ps0, p->ps1, 0, NULL, NULL, SPACK_STRING);
+      gal_fitsarray_array_to_fits_img(p->up.freqstepsname, "Input transform",
+                                      DOUBLE_IMG, tmp, p->ps0, p->ps1, 0,
+                                      NULL, NULL, SPACK_STRING);
       free(tmp);
       complextoreal(p->pker, p->ps0*p->ps1, COMPLEXTOREALSPEC, &tmp);
-      arraytofitsimg(p->up.freqstepsname, "Kernel transform", DOUBLE_IMG,
-                     tmp, p->ps0, p->ps1, 0, NULL, NULL, SPACK_STRING);
+      gal_fitsarray_array_to_fits_img(p->up.freqstepsname, "Kernel transform",
+                                      DOUBLE_IMG, tmp, p->ps0, p->ps1, 0,
+                                      NULL, NULL, SPACK_STRING);
       free(tmp);
     }
 
-
-  /* Multiply the two arrays and save them in the output.*/
+  /* Multiply or divide the two arrays and save them in the output.*/
   if(verb) gettimeofday(&t1, NULL);
-  complexarraymultiply(p->pimg, p->pker, p->ps0*p->ps1);
-  if(verb) reporttiming(&t1, "Multiplied in the frequency domain.", 1);
+  if(p->makekernel)
+    {
+      complexarraydivide(p->pimg, p->pker, p->ps0*p->ps1);
+      if(verb) gal_timing_report(&t1, "Divided in the frequency domain.", 1);
+    }
+  else
+    {
+      complexarraymultiply(p->pimg, p->pker, p->ps0*p->ps1);
+      if(verb) gal_timing_report(&t1, "Multiplied in the frequency domain.", 1);
+    }
   if(p->viewfreqsteps)
     {
       complextoreal(p->pimg, p->ps0*p->ps1, COMPLEXTOREALSPEC, &tmp);
-      arraytofitsimg(p->up.freqstepsname, "Multiplied", DOUBLE_IMG,
-                     tmp, p->ps0, p->ps1, 0, NULL, NULL, SPACK_STRING);
+      gal_fitsarray_array_to_fits_img(p->up.freqstepsname, operation,
+                                      DOUBLE_IMG, tmp, p->ps0, p->ps1, 0,
+                                      NULL, NULL, SPACK_STRING);
       free(tmp);
     }
 
-
-
   /* Forward (in practice inverse) 2D FFT on each image. */
   if(verb) gettimeofday(&t1, NULL);
   twodimensionfft(p, fp, -1);
-  complextoreal(p->pimg, p->ps0*p->ps1, COMPLEXTOREALREAL, &tmp);
-  if(verb) reporttiming(&t1, "Converted back to the spatial domain.", 1);
+  if(p->makekernel) correctdeconvolve(p, &tmp);
+  else complextoreal(p->pimg, p->ps0*p->ps1, COMPLEXTOREALREAL, &tmp);
+  if(verb) gal_timing_report(&t1, "Converted back to the spatial domain.", 1);
   if(p->viewfreqsteps)
     {
-      arraytofitsimg(p->up.freqstepsname, "Spatial", DOUBLE_IMG,
-                     tmp, p->ps0, p->ps1, 0, NULL, NULL, SPACK_STRING);
+      gal_fitsarray_array_to_fits_img(p->up.freqstepsname, "Spatial",
+                                      DOUBLE_IMG, tmp, p->ps0, p->ps1, 0,
+                                      NULL, NULL, SPACK_STRING);
       free(tmp);
     }
 
-
   /* Free the padded arrays (they are no longer needed) and put the
      converted array (that is real, not complex) in p->pimg. */
   free(p->pimg);
   free(p->pker);
   p->pimg=tmp;
 
-
   /* Crop out the center, numbers smaller than 10^{-17} are errors,
      remove them. */
   if(verb) gettimeofday(&t1, NULL);
   removepaddingcorrectroundoff(p);
-  if(verb) reporttiming(&t1, "Padded parts removed.", 1);
-
-
+  if(verb) gal_timing_report(&t1, "Padded parts removed.", 1);
 
 
   /* Free all the allocated space. */
@@ -577,7 +741,6 @@ frequencyconvolve(struct convolveparams *p)
 
 
 
-
 /******************************************************************/
 /*************          Outside function          *****************/
 /******************************************************************/
@@ -586,7 +749,7 @@ convolve(struct convolveparams *p)
 {
   float *convolved;
   long *meshindexs;
-  struct meshparams *mp=&p->mp;
+  struct gal_mesh_params *mp=&p->mp;
 
   /* Do the convolution. */
   if(p->spatial)
@@ -595,20 +758,22 @@ convolve(struct convolveparams *p)
       mp->img=p->input;      mp->s0=p->is0;      mp->s1=p->is1;
       mp->kernel=p->kernel;  mp->ks0=p->ks0;     mp->ks1=p->ks1;
       mp->numthreads=p->cp.numthreads;
-      makemesh(mp);
+      gal_mesh_make_mesh(mp);
       if(p->meshname)
         {
-          checkmeshid(mp, &meshindexs);
-          arraytofitsimg(p->meshname, "Input", FLOAT_IMG, p->mp.img,
-                         mp->s0, mp->s1, p->numblank, p->wcs, NULL,
-                         SPACK_STRING);
-          arraytofitsimg(p->meshname, "MeshIndexs", LONG_IMG, meshindexs,
-                         mp->s0, mp->s1, 0, p->wcs, NULL, SPACK_STRING);
+          gal_check_mesh_id(mp, &meshindexs);
+          gal_fitsarray_array_to_fits_img(p->meshname, "Input", FLOAT_IMG,
+                                          p->mp.img, mp->s0, mp->s1,
+                                          p->anyblank, p->wcs, NULL,
+                                          SPACK_STRING);
+          gal_fitsarray_array_to_fits_img(p->meshname, "MeshIndexs", LONG_IMG,
+                                          meshindexs, mp->s0, mp->s1, 0, p->wcs,
+                                          NULL, SPACK_STRING);
           free(meshindexs);
         }
 
       /* Do the spatial convolution on the mesh: */
-      spatialconvolveonmesh(mp, &convolved);
+      gal_mesh_spatial_convolve_on_mesh(mp, &convolved);
 
       /* Replace the input image array with the convolved array: */
       free(p->input);
@@ -619,6 +784,7 @@ convolve(struct convolveparams *p)
 
   /* Save the output (which is in p->input) array. Note that p->input
      will be freed in ui.c. */
-  arraytofitsimg(p->cp.output, "Convolved", FLOAT_IMG, p->input,
-                 p->is0, p->is1, p->numblank, p->wcs, NULL, SPACK_STRING);
+  gal_fitsarray_array_to_fits_img(p->cp.output, "Convolved", FLOAT_IMG,
+                                  p->input, p->is0, p->is1, p->anyblank, p->wcs,
+                                  NULL, SPACK_STRING);
 }
