@@ -18,7 +18,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with gnuastro. If not, see <http://www.gnu.org/licenses/>.
+along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 **********************************************************************/
 #ifndef CITE_H
 #define CITE_H
@@ -30,8 +30,8 @@ along with gnuastro. If not, see <http://www.gnu.org/licenses/>.
 	   "Citations are vital for the continued work on %s.\n"	\
 	   "Thank you for citing it in your research paper.\n"		\
 	   "\nPlease cite as \"%s\":\n\n%s\n\n%s",			\
-	   SPACK_NAME, SPACK_NAME, SPACK_STRING, GNUASTROBIBTEX,	\
-	   IMGCROPBIBTEX);						\
+	   SPACK_NAME, SPACK_NAME, SPACK_STRING,                        \
+           GAL_STRINGS_MAIN_BIBTEX, IMGCROPBIBTEX);		        \
     exit(EXIT_SUCCESS);							\
 }
 
