@@ -18,26 +18,29 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with gnuastro. If not, see <http://www.gnu.org/licenses/>.
+along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 **********************************************************************/
 #include <config.h>
 
 #include <math.h>
+#include <ctype.h>
 #include <stdio.h>
 #include <errno.h>
 #include <error.h>
+#include <float.h>
 #include <string.h>
 #include <stdlib.h>
 
-#include "box.h"
-#include "timing.h"
-#include "checkset.h"
-#include "fitsarrayvv.h"
+#include <gnuastro/box.h>
+#include <gnuastro/timing.h>
+#include <gnuastro/polygon.h>
+#include <gnuastro/checkset.h>
+#include <gnuastro/fitsarrayvv.h>
 
 #include "main.h"
 
 #include "crop.h"
-
+#include "wcsmode.h"
 
 
 
@@ -90,7 +93,7 @@ sectionparser(char *section, long *naxes, long *fpixel, long *lpixel)
 	case ',':
 	  ++dim;
 	  if(dim==2)
-	    error(EXIT_FAILURE, 0, "Extra `,` in `%s`.", section);
+	    error(EXIT_FAILURE, 0, "Extra `,` in `%s`", section);
 	  forl='f';
 	  ++pt;
 	  break;
@@ -99,9 +102,9 @@ sectionparser(char *section, long *naxes, long *fpixel, long *lpixel)
 	  ++pt;
 	  break;
 	case '.':
-	  error(EXIT_FAILURE, 0, "The numbers in the argument to "
+	  error(EXIT_FAILURE, 0, "the numbers in the argument to "
 		"`--section` (`-s') have to be integers. You input "
-		"includes a float number: %s.",
+		"includes a float number: %s",
 		section);
 	  break;
 	case ' ': case '\t':
@@ -148,10 +151,10 @@ sectionparser(char *section, long *naxes, long *fpixel, long *lpixel)
     }
 
   if(fpixel[0]>=lpixel[0] || fpixel[1]>=lpixel[1])
-    error(EXIT_FAILURE, 0, "The bottom left corner coordinates "
+    error(EXIT_FAILURE, 0, "the bottom left corner coordinates "
 	  "cannot be larger or equal to the top right's! Your section "
 	  "string (%s) has been read as: bottom left coordinate "
-	  "(%ld, %ld) to top right coordinate (%ld, %ld).",
+	  "(%ld, %ld) to top right coordinate (%ld, %ld)",
 	  section, fpixel[0], fpixel[1], lpixel[0], lpixel[1]);
 
   /*
@@ -166,6 +169,232 @@ sectionparser(char *section, long *naxes, long *fpixel, long *lpixel)
 
 
 
+void
+polygonparser(struct imgcropparams *p)
+{
+  size_t dim=0;
+  char *tailptr;
+  double read[2], *array;
+  struct gal_linkedlist_tdll *gal_linkedlist_tdll=NULL;
+  char *pt=p->up.polygon;
+
+  /* Parse the string. */
+  while(*pt!='\0')
+    {
+      switch(*pt)
+	{
+	case ',':
+	  ++dim;
+	  if(dim==2)
+	    error(EXIT_FAILURE, 0, "Extra `,` in `%s`", p->up.polygon);
+	  ++pt;
+	  break;
+	case ':':
+          if(dim==0)
+            error(EXIT_FAILURE, 0, "not enough coordinates for at least "
+                  "one polygon vertex (in %s)", p->up.polygon);
+          dim=0;
+	  ++pt;
+	  break;
+	default:
+          break;
+        }
+
+      /* strtod will skip white spaces if they are before a number,
+         but not when they are before a : or ,. So we need to remove
+         all white spaces. White spaces are usually put beside each
+         other, so if one is encountered, go along the string until
+         the white space characters finish.  */
+      if(isspace(*pt))
+        ++pt;
+      else
+        {
+          /* Read the number: */
+          read[dim]=strtod(pt, &tailptr);
+
+          /* Check if there actually was a number.
+          printf("\n\n------\n%lu: %f (%s)\n", dim, read[dim], tailptr);
+          */
+
+          /* Make sure if a number was read at all? */
+          if(tailptr==pt)	        /* No number was read!             */
+            error(EXIT_FAILURE, 0, "%s could not be parsed as a floating "
+                  "point number", tailptr);
+
+          /* If this was the second dimension, then put the values
+             into the linked list: */
+          if(dim==1)
+            gal_linkedlist_add_to_tdll(&gal_linkedlist_tdll, read[0], read[1]);
+
+          /* The job here is done, start from tailptr */
+          pt=tailptr;
+        }
+    }
+
+  /* Convert the linked list to an array: */
+  gal_linkedlist_tdll_to_array_inv(gal_linkedlist_tdll, &array, &p->nvertices);
+  if(p->imgmode) { p->ipolygon=array; p->wpolygon=NULL;  }
+  else           { p->ipolygon=NULL;  p->wpolygon=array; }
+
+  /* Put them in the proper order in WCS mode: */
+
+  /* For a check:
+  {
+    size_t i;
+    double *polygon=p->imgmode?p->ipolygon:p->wpolygon;
+    for(i=0;i<p->nvertices;++i)
+      printf("(%f, %f)\n", polygon[i*2], polygon[i*2+1]);
+  }
+  */
+
+  /* Clean up: */
+  gal_linkedlist_free_tdll(gal_linkedlist_tdll);
+}
+
+
+
+
+
+void
+imgpolygonflpixel(double *ipolygon, size_t nvertices, long *fpixel,
+                  long *lpixel)
+{
+  size_t i;
+  double minx=FLT_MAX, miny=FLT_MAX;
+  double maxx=-FLT_MAX, maxy=-FLT_MAX;
+
+  /* Find their minimum and maximum values. */
+  for(i=0;i<nvertices;++i)
+    {
+      if(ipolygon[i*2]>maxx) maxx=ipolygon[i*2];
+      if(ipolygon[i*2]<minx) minx=ipolygon[i*2];
+      if(ipolygon[i*2+1]>maxy) maxy=ipolygon[i*2+1];
+      if(ipolygon[i*2+1]<miny) miny=ipolygon[i*2+1];
+    }
+
+  /* Set the first and last pixel. */
+  fpixel[0] = minx - (int)minx >=0.5 ? (int)minx + 1 : (int)minx;
+  fpixel[1] = miny - (int)miny >=0.5 ? (int)miny + 1 : (int)miny;
+  lpixel[0] = maxx - (int)maxx >=0.5 ? (int)maxx + 1 : (int)maxx;
+  lpixel[1] = maxy - (int)maxy >=0.5 ? (int)maxy + 1 : (int)maxy;
+}
+
+
+
+
+
+void
+polygonmask(struct cropparams *crp, void *array, long *fpixel_i,
+            size_t s0, size_t s1)
+{
+  long *lb, *la=array;
+  short *sb, *sa=array;
+  float *fb, *fa=array;
+  LONGLONG *Lb, *La=array;
+  unsigned char *bb, *ba=array;
+  double *db, *ipolygon, point[2], *da=array;
+  int outpolygon=crp->p->outpolygon, bitpix=crp->p->bitpix;
+  size_t i, *ordinds, size=s0*s1, nvertices=crp->p->nvertices;
+
+
+  /* First of all, allocate enough space to put a copy of the input
+     coordinates (we will be using that after sorting in an
+     anti-clickwise manner.) */
+  errno=0; ipolygon=malloc(2*nvertices*sizeof *ipolygon);
+  if(ipolygon==NULL)
+    error(EXIT_FAILURE, errno, "%lu bytes for ipolygon in polygonmask "
+          "(crop.c)", 2*nvertices*sizeof *ipolygon);
+  errno=0; ordinds=malloc(nvertices*sizeof *ordinds);
+  if(ordinds==NULL)
+    error(EXIT_FAILURE, errno, "%lu bytes for ordinds in polygonmask "
+          "(crop.c)", nvertices*sizeof *ordinds);
+
+
+  /* Find the order of the polygons and put the elements in the proper
+     order. Also subtract the fpixel_i coordinates from all the
+     vertices to bring them into the crop image coordinates.*/
+  gal_polygon_ordered_corners(crp->ipolygon, crp->p->nvertices, ordinds);
+  for(i=0;i<crp->p->nvertices;++i)
+    {
+      ipolygon[i*2  ] = crp->ipolygon[ordinds[i]*2]   - fpixel_i[0];
+      ipolygon[i*2+1] = crp->ipolygon[ordinds[i]*2+1] - fpixel_i[1];
+    }
+
+
+  /* Go over all the pixels in the image and if they are within the
+     polygon keep them if the user has asked for it.*/
+  switch(bitpix)
+    {
+    case BYTE_IMG:
+      bb=gal_fitsarray_bitpix_blank(bitpix);
+      for(i=0;i<size;++i)
+        {
+          point[0]=i%s1+1; point[1]=i/s1+1;
+          if(gal_polygon_pin(ipolygon, point, nvertices)==outpolygon) ba[i]=*bb;
+        }
+      free(bb);
+      break;
+    case SHORT_IMG:
+      sb=gal_fitsarray_bitpix_blank(bitpix);
+      for(i=0;i<size;++i)
+        {
+          point[0]=i%s1+1; point[1]=i/s1+1;
+          if(gal_polygon_pin(ipolygon, point, nvertices)==outpolygon) sa[i]=*sb;
+        }
+      free(sb);
+      break;
+    case LONG_IMG:
+      lb=gal_fitsarray_bitpix_blank(bitpix);
+      for(i=0;i<size;++i)
+        {
+          point[0]=i%s1+1; point[1]=i/s1+1;
+          if(gal_polygon_pin(ipolygon, point, nvertices)==outpolygon) la[i]=*lb;
+        }
+      free(lb);
+      break;
+    case LONGLONG_IMG:
+      Lb=gal_fitsarray_bitpix_blank(bitpix);
+      for(i=0;i<size;++i)
+        {
+          point[0]=i%s1+1; point[1]=i/s1+1;
+          if(gal_polygon_pin(ipolygon, point, nvertices)==outpolygon) La[i]=*Lb;
+        }
+      free(Lb);
+      break;
+    case FLOAT_IMG:
+      fb=gal_fitsarray_bitpix_blank(bitpix);
+      for(i=0;i<size;++i)
+        {
+          point[0]=i%s1+1; point[1]=i/s1+1;
+          if(gal_polygon_pin(ipolygon, point, nvertices)==outpolygon) fa[i]=*fb;
+        }
+      free(fb);
+      break;
+    case DOUBLE_IMG:
+      db=gal_fitsarray_bitpix_blank(bitpix);
+      for(i=0;i<size;++i)
+        {
+          point[0]=i%s1+1; point[1]=i/s1+1;
+          if(gal_polygon_pin(ipolygon, point, nvertices)==outpolygon) da[i]=*db;
+        }
+      free(db);
+      break;
+    default:
+      error(EXIT_FAILURE, 0, "a bug! Please contact us at %s, so we "
+            "can fix the problem. For some reason, an unrecognized "
+            "bitpix value (%d) has been seen in polygonmask (crop.c)",
+            PACKAGE_BUGREPORT, bitpix);
+    }
+
+  /* Clean up: */
+  free(ordinds);
+  free(ipolygon);
+}
+
+
+
+
+
 
 
 
@@ -206,9 +435,9 @@ changezerotonan(void *array, size_t size, int bitpix)
       break;
 
     default:
-      error(EXIT_FAILURE, 0, "In changezerotonan, bitpix is not "
+      error(EXIT_FAILURE, 0, "in changezerotonan, bitpix is not "
 	    "recognized! This is out of the users control and is a bug, "
-	    "please report it to us so we see how it was caused and fix it.");
+	    "please report it to us so we see how it was caused and fix it");
     }
 }
 
@@ -222,7 +451,7 @@ void
 cropname(struct cropparams *crp)
 {
   struct imgcropparams *p=crp->p;
-  struct commonparams *cp=&p->cp;
+  struct gal_commonparams *cp=&p->cp;
   struct imgcroplog *log=&crp->p->log[crp->outindex];
 
   /* Set the output name and crop sides: */
@@ -235,7 +464,7 @@ cropname(struct cropparams *crp)
 	      "imgcroponthreads", crp->outlen);
       sprintf(log->name, "%s%lu%s", cp->output, crp->outindex+1,
 	      p->suffix);
-      checkremovefile(log->name, cp->dontdelete);
+      gal_checkset_check_remove_file(log->name, cp->dontdelete);
     }
   else
     {
@@ -243,10 +472,10 @@ cropname(struct cropparams *crp)
       if(p->outnameisfile)            /* An output file was specified. */
 	{
 	  log->name=cp->output;
-	  checkremovefile(log->name, cp->dontdelete);
+	  gal_checkset_check_remove_file(log->name, cp->dontdelete);
 	}
       else	  /* The output was a directory, use automatic output. */
-	automaticoutput(p->imgs[crp->imgindex].name, p->suffix,
+	gal_checkset_automatic_output(p->imgs[crp->imgindex].name, p->suffix,
 			cp->removedirinfo, cp->dontdelete,
 			&log->name);
     }
@@ -270,36 +499,62 @@ cropflpixel(struct cropparams *crp)
   if(p->imgmode)
     {
       if(p->up.catset)
-	borderfromcenter(p->cat[crp->outindex*p->cs1+p->xcol],
-			 p->cat[crp->outindex*p->cs1+p->ycol],
-			 p->iwidth, fpixel, lpixel);
+        gal_box_border_from_center(p->cat[crp->outindex*p->cs1+p->xcol],
+                                   p->cat[crp->outindex*p->cs1+p->ycol],
+                                   p->iwidth, fpixel, lpixel);
       else if(p->up.xcset)
-	borderfromcenter(p->xc, p->yc, p->iwidth, fpixel, lpixel);
+        gal_box_border_from_center(p->xc, p->yc, p->iwidth, fpixel, lpixel);
       else if(p->up.sectionset)
 	sectionparser(p->section, naxes, fpixel, lpixel);
+      else if(p->up.polygonset)
+        {
+          if(p->outpolygon==0)
+            imgpolygonflpixel(p->ipolygon, p->nvertices, fpixel, lpixel);
+        }
       else
-	error(EXIT_FAILURE, 0, "A bug! In image mode, neither of the "
-	      "following has been set: a catalog, a central pixel or "
-	      "a section of the image. Please contact us to see how it "
-	      "got to this impossible place!");
+	error(EXIT_FAILURE, 0, "a bug! In image mode, neither of the "
+	      "following has been set: a catalog, a central pixel, "
+	      "a section or a polygon in the image. Please contact us "
+              "to see how it got to this impossible place! You should "
+              "have been warned of this condition long before ImageCrop "
+              "reaches this point");
     }
   else if(p->wcsmode) /* In wcsmode, crp->world is already filled.       */
     {		      /* Note that p->iwidth was set based on p->wwidth. */
-      status=0;
-      if(wcss2p(p->imgs[crp->imgindex].wcs, ncoord, nelem, crp->world,
-		phi, theta, imgcrd, pixcrd, &status) )
-	error(EXIT_FAILURE, 0, "wcss2p error %d: %s", status,
-	      wcs_errmsg[status]);
-      borderfromcenter(pixcrd[0], pixcrd[1], p->iwidth, fpixel, lpixel);
-      /*
-      printf("\n(%f, %f): (%ld, %ld) -- (%ld, %ld)\n\n", pixcrd[0],
-	     pixcrd[1], fpixel[0], fpixel[1], lpixel[0], lpixel[1]);
-      */
+      if(p->up.polygonset)
+        { /* Fill crp->ipolygon in wcspolygonpixel, then set flpixel*/
+          fillcrpipolygon(crp);
+          if(p->outpolygon==0)
+            imgpolygonflpixel(crp->ipolygon, p->nvertices, fpixel, lpixel);
+        }
+      else
+        {
+          status=0;
+          if(wcss2p(p->imgs[crp->imgindex].wcs, ncoord, nelem, crp->world,
+                    phi, theta, imgcrd, pixcrd, &status) )
+            error(EXIT_FAILURE, 0, "wcss2p error %d: %s", status,
+                  wcs_errmsg[status]);
+          gal_box_border_from_center(pixcrd[0], pixcrd[1], p->iwidth, fpixel,
+                                     lpixel);
+          /*
+            printf("\n(%f, %f): (%ld, %ld) -- (%ld, %ld)\n\n", pixcrd[0],
+                   pixcrd[1], fpixel[0], fpixel[1], lpixel[0], lpixel[1]);
+          */
+        }
     }
   else
-    error(EXIT_FAILURE, 0, "A bug! in cropflpixel (crop.c), "
+    error(EXIT_FAILURE, 0, "a bug! in cropflpixel (crop.c), "
 	  "neither imgmode or wcsmode are set. Please contact us so "
-	  "we can see how it got to this impossible place.");
+	  "we can see how it got to this impossible place");
+
+  /* If the user only wants regions outside to the polygon, then set
+     the fpixel and lpixel to cover the full input image. */
+  if(p->up.polygonset && p->outpolygon)
+    {
+      crp->lpixel[0]=naxes[0];
+      crp->lpixel[1]=naxes[1];
+      crp->fpixel[0]=crp->fpixel[1]=1;
+    }
 }
 
 
@@ -356,17 +611,17 @@ firstcropmakearray(struct cropparams *crp, long *fpixel_i,
   /* Create the FITS image extension and array and fill it with null
      values. */
   if(fits_create_file(&crp->outfits, outname, &status))
-    fitsioerror(status, "Creating file.");
+    gal_fitsarray_io_error(status, "creating file");
   ofp=crp->outfits;
   if(fits_create_img(ofp, bitpix, naxis, naxes, &status))
-    fitsioerror(status, "Creating image.");
+    gal_fitsarray_io_error(status, "creating image");
   if(bitpix==BYTE_IMG || bitpix==SHORT_IMG
      || bitpix==LONG_IMG || bitpix==LONGLONG_IMG)
     if(fits_write_key(ofp, crp->p->datatype, "BLANK",
-		      crp->p->bitnul, "Pixels with no data.", &status) )
-      fitsioerror(status, "Adding Blank.");
+		      crp->p->bitnul, "pixels with no data", &status) )
+      gal_fitsarray_io_error(status, "adding Blank");
   if(fits_write_null_img(ofp, 1, naxes[0]*naxes[1], &status))
-    fitsioerror(status, "Writing null array.");
+    gal_fitsarray_io_error(status, "writing null array");
 
 
   /* Write the WCS header keywords in the output FITS image, then
@@ -376,7 +631,7 @@ firstcropmakearray(struct cropparams *crp, long *fpixel_i,
       crpix0 = img->wcs->crpix[0] - (fpixel_i[0]-1) + (fpixel_c[0]-1);
       crpix1 = img->wcs->crpix[1] - (fpixel_i[1]-1) + (fpixel_c[1]-1);
       if(fits_write_record(ofp, blankrec, &status))
-        fitsioerror(status, NULL);
+        gal_fitsarray_io_error(status, NULL);
       sprintf(titlerec, "%sWCS information", startblank);
       for(i=strlen(titlerec);i<79;++i)
         titlerec[i]=' ';
@@ -385,18 +640,18 @@ firstcropmakearray(struct cropparams *crp, long *fpixel_i,
         fits_write_record(ofp, &img->wcstxt[i*80], &status);
       fits_update_key(ofp, TDOUBLE, "CRPIX1", &crpix0, NULL, &status);
       fits_update_key(ofp, TDOUBLE, "CRPIX2", &crpix1, NULL, &status);
-      fitsioerror(status, NULL);
+      gal_fitsarray_io_error(status, NULL);
     }
 
 
   /* Add the Crop information. */
   if(fits_write_record(ofp, blankrec, &status))
-    fitsioerror(status, NULL);
+    gal_fitsarray_io_error(status, NULL);
   sprintf(titlerec, "%sCrop information", startblank);
   for(i=strlen(titlerec);i<79;++i)
     titlerec[i]=' ';
   if(fits_write_record(ofp, titlerec, &status))
-    fitsioerror(status, NULL);
+    gal_fitsarray_io_error(status, NULL);
 }
 
 
@@ -421,21 +676,22 @@ onecrop(struct cropparams *crp)
   char basename[FLEN_KEYWORD];
   long fpixel_i[2] , lpixel_i[2];
   fitsfile *ifp=crp->infits, *ofp;
-  struct fitsheaderll *headers=NULL;
+  struct gal_fitsarray_header_ll *headers=NULL;
   int status=0, anynul=0, bitpix=p->bitpix;
   long fpixel_o[2], lpixel_o[2], inc[2]={1,1};
   char region[FLEN_VALUE], regionkey[FLEN_KEYWORD];
 
 
   /* Find the first and last pixel of this crop box from this input
-     image. */
+     image. If the outer polygon region is to be kept, then set the
+     sides to the image sides.*/
   cropflpixel(crp);
   fpixel_i[0]=crp->fpixel[0];      fpixel_i[1]=crp->fpixel[1];
   lpixel_i[0]=crp->lpixel[0];      lpixel_i[1]=crp->lpixel[1];
 
 
   /* Find the overlap and apply it if there is any overlap. */
-  if( overlap(img->naxes, fpixel_i, lpixel_i, fpixel_o, lpixel_o) )
+  if( gal_box_overlap(img->naxes, fpixel_i, lpixel_i, fpixel_o, lpixel_o) )
     {
       /* Make the output FITS image and initialize it with an array of
 	 NaN or BLANK values. Note that for FLOAT_IMG and DOUBLE_IMG,
@@ -448,11 +704,11 @@ onecrop(struct cropparams *crp)
       /* Read the desired part of the image, then write it into this
 	 array. */
       cropsize=(lpixel_i[0]-fpixel_i[0]+1)*(lpixel_i[1]-fpixel_i[1]+1);
-      array=bitpixalloc(cropsize, bitpix);
+      array=gal_fitsarray_bitpix_alloc(cropsize, bitpix);
       status=0;
       if(fits_read_subset(ifp, p->datatype, fpixel_i, lpixel_i, inc,
 			  p->bitnul, array, &anynul, &status))
-	fitsioerror(status, NULL);
+	gal_fitsarray_io_error(status, NULL);
 
 
       /* If we have a floating point or double image, pixels with zero
@@ -463,30 +719,48 @@ onecrop(struct cropparams *crp)
 	changezerotonan(array, cropsize, bitpix);
 
 
+      /* If a polygon is given, remove all the pixels within or
+         outside of it.*/
+      if(p->up.polygonset)
+        {
+          /* In WCS mode, crp->ipolygon was allocated and filled in
+             wcspolygonflpixel (wcsmode.c). */
+          if(p->imgmode) crp->ipolygon=p->ipolygon;
+          polygonmask(crp, array, fpixel_i, lpixel_i[1]-fpixel_i[1]+1,
+                      lpixel_i[0]-fpixel_i[0]+1);
+          if(p->wcsmode) free(crp->ipolygon);
+        }
+
+
       /* Write the array into the image. */
       status=0;
       if( fits_write_subset(ofp, p->datatype, fpixel_o, lpixel_o,
 			    array, &status) )
-	fitsioerror(status, NULL);
+	gal_fitsarray_io_error(status, NULL);
 
 
       /* A section has been added to the cropped image from this input
 	 image, so increment crp->imgcount and save the information of
 	 this image. */
       sprintf(basename, "ICF%lu", ++p->log[crp->outindex].numimg);
-      filenameinkeywords(basename, img->name, &headers);
+      gal_fitsarray_file_name_in_keywords(basename, img->name, &headers);
       sprintf(regionkey, "%sPIX", basename);
       sprintf(region, "%ld:%ld,%ld:%ld", fpixel_i[0], lpixel_i[0]+1,
 	      fpixel_i[1], lpixel_i[1]+1);
-      add_to_fitsheaderllend(&headers, TSTRING, regionkey, 0, region, 0,
-			     "Range of pixels used for this output.",
-			     0, NULL);
-      updatekeys(ofp, &headers);
+      gal_fitsarray_add_to_fits_header_ll_end(&headers, TSTRING, regionkey, 0,
+                                              region, 0, "Range of pixels "
+                                              "used for this output.", 0, NULL);
+      gal_fitsarray_update_keys(ofp, &headers);
 
 
       /* Free the allocated array. */
       free(array);
     }
+  else
+    if(p->up.polygonset && p->outpolygon==0 && p->wcsmode)
+      free(crp->ipolygon);
+
+
   return;
 }
 
@@ -534,7 +808,7 @@ iscenterfilled(struct cropparams *crp)
 
   /* Get the final size of the output image. */
   if( fits_get_img_size(ofp, maxdim, naxes, &status) )
-    fitsioerror(status, NULL);
+    gal_fitsarray_io_error(status, NULL);
 
   /* Get the size and range of the central region to check. The +1 is
      because in FITS, counting begins from 1, not zero. */
@@ -545,10 +819,10 @@ iscenterfilled(struct cropparams *crp)
 
   /* Allocate the array and read in the pixels. */
   size=checkcenter*checkcenter;
-  array=bitpixalloc(size, bitpix);
+  array=gal_fitsarray_bitpix_alloc(size, bitpix);
   if( fits_read_subset(ofp, p->datatype, fpixel, lpixel, inc,
 		       p->bitnul, array, &anynul, &status) )
-    fitsioerror(status, NULL);
+    gal_fitsarray_io_error(status, NULL);
 
   /* Depending on bitpix, check the central pixels of the image. */
   nulcount=0;
@@ -589,9 +863,9 @@ iscenterfilled(struct cropparams *crp)
       break;
 
     default:
-      error(EXIT_FAILURE, 0, "In iscenterfilled, the bitbix is not "
+      error(EXIT_FAILURE, 0, "in iscenterfilled, the bitbix is not "
 	    "recognized! This is not possible by the user, so it is a "
-	    "a bug. Please contact us so we can correct it.");
+	    "a bug. Please contact us so we can correct it");
     }
   free(array);
 
@@ -627,7 +901,7 @@ printlog(struct imgcropparams *p)
 {
   size_t i;
   FILE *logfile;
-  char msg[VERBMSGLENGTH_V];
+  char msg[GAL_TIMING_VERB_MSG_LENGTH_V];
   struct imgcroplog *log=p->log;
   size_t numfiles=0, numcentfilled=0, numstitched=0;
 
@@ -647,7 +921,7 @@ printlog(struct imgcropparams *p)
 	}
 
   /* Check to see if the file exists and remove if if it is ok. */
-  checkremovefile(LOGFILENAME, p->cp.dontdelete);
+  gal_checkset_check_remove_file(LOGFILENAME, p->cp.dontdelete);
 
   /* Make the file and print the top comments. If the file can't be
      opened for write mode, there is no problem, this is a log file,
@@ -675,22 +949,22 @@ printlog(struct imgcropparams *p)
 
       /* Then print each output's information. */
       for(i=0;log[i].name;++i)
-	fprintf(logfile, "%s     %-8lu%-2d\n", log[i].name,
-		log[i].numimg, log[i].centerfilled);
+        fprintf(logfile, "%s     %-8lu%-2d\n", log[i].name,
+                log[i].numimg, log[i].centerfilled);
 
       /* Report Summary: */
       if(p->cp.verb && p->up.catset)
 	{
 	  sprintf(msg, "%lu images created.", numfiles);
-	  reporttiming(NULL, msg, 1);
+	  gal_timing_report(NULL, msg, 1);
 	  sprintf(msg, "%lu filled in the center.",
 		  numcentfilled);
-	  reporttiming(NULL, msg, 1);
+	  gal_timing_report(NULL, msg, 1);
 	  if(numstitched)
 	    {
 	      sprintf(msg, "%lu used more than one input.",
 		      numstitched);
-	      reporttiming(NULL, msg, 1);
+	      gal_timing_report(NULL, msg, 1);
 	    }
 	}
 
