@@ -18,7 +18,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with gnuastro. If not, see <http://www.gnu.org/licenses/>.
+along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 **********************************************************************/
 #include <config.h>
 
@@ -29,12 +29,14 @@ along with gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <string.h>
 #include <fitsio.h>
 
-#include "timing.h"	/* Includes time.h and sys/time.h   */
-#include "checkset.h"
-#include "txtarrayvv.h"
-#include "commonargs.h"
-#include "configfiles.h"
-#include "fitsarrayvv.h"
+#include <nproc.h>              /* From Gnulib.                     */
+
+#include <gnuastro/timing.h>    /* Includes time.h and sys/time.h   */
+#include <gnuastro/checkset.h>
+#include <gnuastro/txtarrayvv.h>
+#include <gnuastro/commonargs.h>
+#include <gnuastro/configfiles.h>
+#include <gnuastro/fitsarrayvv.h>
 
 #include "main.h"
 #include "crop.h"
@@ -71,7 +73,7 @@ readconfig(char *filename, struct imgcropparams *p)
   char *line, *name, *value;
   struct uiparams *up=&p->up;
   size_t lineno=0, len=200, tmp;
-  struct commonparams *cp=&p->cp;
+  struct gal_commonparams *cp=&p->cp;
   char key='a';	/* Not used, just a place holder. */
   int imgmodeset=0, wcsmodeset=0; /* For unambiguous default file checking. */
 
@@ -96,12 +98,12 @@ readconfig(char *filename, struct imgcropparams *p)
   while(getline(&line, &len, fp) != -1)
     {
       /* Prepare the "name" and "value" strings, also set lineno. */
-      STARTREADINGLINE;
+      GAL_CONFIGFILES_START_READING_LINE;
 
       /* Operating modes: */
       if(strcmp(name, "imgmode")==0)
 	{
-	  intzeroorone(value, &zeroorone, name, key, SPACK,
+	  gal_checkset_int_zero_or_one(value, &zeroorone, name, key, SPACK,
 		       filename, lineno);
 	  if(zeroorone)
 	    {
@@ -120,7 +122,7 @@ readconfig(char *filename, struct imgcropparams *p)
 	}
       else if(strcmp(name, "wcsmode")==0)
 	{
-	  intzeroorone(value, &zeroorone, name, key, SPACK,
+	  gal_checkset_int_zero_or_one(value, &zeroorone, name, key, SPACK,
 		       filename, lineno);
 	  if(zeroorone)
 	    {
@@ -144,40 +146,40 @@ readconfig(char *filename, struct imgcropparams *p)
 
       /* Inputs: */
       else if(strcmp(name, "hdu")==0)
-        allocatecopyset(value, &cp->hdu, &cp->hduset);
+        gal_checkset_allocate_copy_set(value, &cp->hdu, &cp->hduset);
 
       else if(strcmp(name, "racol")==0)
 	{
 	  if(up->racolset) continue;
-	  sizetelzero(value, &p->racol, name, key, SPACK,
+	  gal_checkset_sizet_el_zero(value, &p->racol, name, key, SPACK,
 		      filename, lineno);
 	  up->racolset=1;
 	}
       else if(strcmp(name, "deccol")==0)
 	{
 	  if(up->deccolset) continue;
-	  sizetelzero(value, &p->deccol, name, key, SPACK,
+	  gal_checkset_sizet_el_zero(value, &p->deccol, name, key, SPACK,
 		      filename, lineno);
 	  up->deccolset=1;
 	}
       else if(strcmp(name, "xcol")==0)
 	{
 	  if(up->xcolset) continue;
-	  sizetelzero(value, &p->xcol, name, key, SPACK,
+	  gal_checkset_sizet_el_zero(value, &p->xcol, name, key, SPACK,
 		      filename, lineno);
 	  up->xcolset=1;
 	}
       else if(strcmp(name, "ycol")==0)
 	{
 	  if(up->ycolset) continue;
-	  sizetelzero(value, &p->ycol, name, key, SPACK,
+	  gal_checkset_sizet_el_zero(value, &p->ycol, name, key, SPACK,
 		      filename, lineno);
 	  up->ycolset=1;
 	}
       else if(strcmp(name, "iwidth")==0)
 	{
 	  if(up->iwidthset) continue;
-	  sizetlzero(value, &tmp, name, key, SPACK,
+	  gal_checkset_sizet_l_zero(value, &tmp, name, key, SPACK,
 		     filename, lineno);
 	  p->iwidth[0]=p->iwidth[1]=tmp;
 	  up->iwidthset=1;
@@ -185,12 +187,24 @@ readconfig(char *filename, struct imgcropparams *p)
       else if(strcmp(name, "wwidth")==0)
 	{
 	  if(up->wwidthset) continue;
-	  doublel0(value, &p->wwidth, name, key, SPACK,
+	  gal_checkset_double_l_0(value, &p->wwidth, name, key, SPACK,
 		   filename, lineno);
 	  up->wwidthset=1;
 	}
-
-
+      else if(strcmp(name, "hstartwcs")==0)
+	{
+	  if(up->hstartwcsset) continue;
+	  gal_checkset_sizet_el_zero(value, &p->hstartwcs, name, key, SPACK,
+                      filename, lineno);
+	  up->hstartwcsset=1;
+	}
+      else if(strcmp(name, "hendwcs")==0)
+	{
+	  if(up->hendwcsset) continue;
+	  gal_checkset_sizet_el_zero(value, &p->hendwcs, name, key, SPACK,
+                      filename, lineno);
+	  up->hendwcsset=1;
+	}
 
 
 
@@ -198,7 +212,7 @@ readconfig(char *filename, struct imgcropparams *p)
       else if(strcmp(name, "checkcenter")==0)
 	{
 	  if(up->checkcenterset) continue;
-	  sizetelzero(value, &p->checkcenter, name, key, SPACK,
+	  gal_checkset_sizet_el_zero(value, &p->checkcenter, name, key, SPACK,
 		      filename, lineno);
 	  up->checkcenterset=1;
 	}
@@ -208,7 +222,7 @@ readconfig(char *filename, struct imgcropparams *p)
 	  errno=0;
 	  cp->output=malloc(strlen(value)+1);
 	  if(cp->output==NULL)
-	    error(EXIT_FAILURE, errno, "Space for output");
+	    error(EXIT_FAILURE, errno, "space for output");
 	  strcpy(cp->output, value);
 	  cp->outputset=1;
 	}
@@ -218,15 +232,13 @@ readconfig(char *filename, struct imgcropparams *p)
 	  errno=0;
 	  p->suffix=malloc(strlen(value)+1);
 	  if(p->suffix==NULL)
-	    error(EXIT_FAILURE, errno, "Space for prefix.");
+	    error(EXIT_FAILURE, errno, "space for prefix");
 	  strcpy(p->suffix, value);
 	  up->suffixset=1;
 	}
 
-
       /* Read options common to all programs */
-      READ_COMMONOPTIONS_FROM_CONF
-
+      GAL_CONFIGFILES_READ_COMMONOPTIONS_FROM_CONF
 
       else
 	error_at_line(EXIT_FAILURE, 0, filename, lineno,
@@ -245,13 +257,13 @@ void
 printvalues(FILE *fp, struct imgcropparams *p)
 {
   struct uiparams *up=&p->up;
-  struct commonparams *cp=&p->cp;
+  struct gal_commonparams *cp=&p->cp;
 
   /* Print all the options that are set. Separate each group with a
      commented line explaining the options in that group. */
   fprintf(fp, "\n# Input image:\n");
   if(cp->hduset)
-    PRINTSTINGMAYBEWITHSPACE("hdu", cp->hdu);
+    GAL_CHECKSET_PRINT_STRING_MAYBE_WITH_SPACE("hdu", cp->hdu);
 
 
   fprintf(fp, "\n# Output parameters:\n");
@@ -276,12 +288,17 @@ printvalues(FILE *fp, struct imgcropparams *p)
     fprintf(fp, CONF_SHOWFMT"%lu\n", "deccol", p->deccol);
   if(up->wwidthset)
     fprintf(fp, CONF_SHOWFMT"%.3f\n", "wwidth", p->wwidth);
+  if(up->hstartwcsset)
+    fprintf(fp, CONF_SHOWFMT"%lu\n", "hstartwcs", p->hstartwcs);
+  if(up->hendwcsset)
+    fprintf(fp, CONF_SHOWFMT"%lu\n", "hendwcs", p->hendwcs);
 
 
   /* For the operating mode, first put the macro to print the common
      options, then the (possible options particular to this
-     program) */
-  PRINT_COMMONOPTIONS;
+     program). */
+  fprintf(fp, "\n# Operating mode:\n");
+  GAL_CONFIGFILES_PRINT_COMMONOPTIONS;
   if(up->imgmodeset)
     fprintf(fp, CONF_SHOWFMT"%d\n", "imgmode", p->imgmode);
   if(up->wcsmodeset)
@@ -299,30 +316,31 @@ void
 checkifset(struct imgcropparams *p)
 {
   struct uiparams *up=&p->up;
-  struct commonparams *cp=&p->cp;
+  struct gal_commonparams *cp=&p->cp;
 
   int intro=0;
   if(up->imgmodeset==0 && up->wcsmodeset==0)
-    REPORT_NOTSET("imgmode or wcsmode");
+    GAL_CONFIGFILES_REPORT_NOTSET("imgmode or wcsmode");
   if(cp->hduset==0)
-    REPORT_NOTSET("hdu");
+    GAL_CONFIGFILES_REPORT_NOTSET("hdu");
   if(up->xcolset==0)
-    REPORT_NOTSET("xcol");
+    GAL_CONFIGFILES_REPORT_NOTSET("xcol");
   if(up->ycolset==0)
-    REPORT_NOTSET("ycol");
+    GAL_CONFIGFILES_REPORT_NOTSET("ycol");
   if(up->iwidthset==0)
-    REPORT_NOTSET("iwidth");
+    GAL_CONFIGFILES_REPORT_NOTSET("iwidth");
   if(up->racolset==0)
-    REPORT_NOTSET("racol");
+    GAL_CONFIGFILES_REPORT_NOTSET("racol");
   if(up->deccolset==0)
-    REPORT_NOTSET("deccol");
+    GAL_CONFIGFILES_REPORT_NOTSET("deccol");
   if(up->wwidthset==0)
-    REPORT_NOTSET("wwidth");
+    GAL_CONFIGFILES_REPORT_NOTSET("wwidth");
   if(up->suffixset==0)
-    REPORT_NOTSET("suffix");
+    GAL_CONFIGFILES_REPORT_NOTSET("suffix");
   if(up->checkcenterset==0)
-    REPORT_NOTSET("checkcenter");
-  END_OF_NOTSET_REPORT;
+    GAL_CONFIGFILES_REPORT_NOTSET("checkcenter");
+
+  GAL_CONFIGFILES_END_OF_NOTSET_REPORT;
 }
 
 
@@ -352,9 +370,9 @@ void
 sanitycheck(struct imgcropparams *p)
 {
   int checksum;
+  char forerr[100];
   struct uiparams *up=&p->up;
-  struct commonparams *cp=&p->cp;
-  char *t1=NULL, *t2=NULL, *t3=NULL, *boxparam;
+  struct gal_commonparams *cp=&p->cp;
 
 
 
@@ -366,12 +384,12 @@ sanitycheck(struct imgcropparams *p)
 
   /* Width and checkcenter are odd */
   if(p->iwidth[0]<3)
-    error(EXIT_FAILURE, 0, "--iwidth has to be >3 pixels.");
+    error(EXIT_FAILURE, 0, "--iwidth has to be >3 pixels");
   else if(p->iwidth[0]%2==0)
       p->iwidth[0]+=1;
   p->iwidth[1]=p->iwidth[0];
   if(p->checkcenter<3)
-    error(EXIT_FAILURE, 0, "--checkcenter has to be >3 pixels.");
+    error(EXIT_FAILURE, 0, "--checkcenter has to be >3 pixels");
   else if(p->checkcenter%2==0)
     p->checkcenter+=1;
 
@@ -379,11 +397,11 @@ sanitycheck(struct imgcropparams *p)
 
   /* deccol!=racol and xcol!=ycol */
   if(p->racol==p->deccol)
-    error(EXIT_FAILURE, 0, "The columns for RA and Dec must be "
-	  "different.");
+    error(EXIT_FAILURE, 0, "the columns for RA and Dec must be "
+	  "different");
   if(p->xcol==p->ycol)
-    error(EXIT_FAILURE, 0, "The columns for x and y must be "
-	  "different.");
+    error(EXIT_FAILURE, 0, "the columns for x and y must be "
+	  "different");
 
 
 
@@ -392,8 +410,8 @@ sanitycheck(struct imgcropparams *p)
   checksum=up->raset+up->decset;
   if(checksum==2) {p->imgmode=0; p->wcsmode=1;}
   else if(checksum)/* is not zero */
-    error(EXIT_FAILURE, 0, "The options `--ra` and `--dec` should be "
-	  "called together.");
+    error(EXIT_FAILURE, 0, "the options `--ra` and `--dec` should be "
+	  "called together");
 
 
 
@@ -402,8 +420,8 @@ sanitycheck(struct imgcropparams *p)
   checksum=up->xcset+up->ycset;
   if(checksum==2) {p->imgmode=1; p->wcsmode=0;}
   else if(checksum)/* is not zero */
-    error(EXIT_FAILURE, 0, "The options `--xc` and `--yc` should be "
-	  "called together.");
+    error(EXIT_FAILURE, 0, "the options `--xc` and `--yc` should be "
+	  "called together");
 
 
 
@@ -414,22 +432,29 @@ sanitycheck(struct imgcropparams *p)
 
   /* Make sure that the multiple one crop box options have not been
      called together. */
-  checksum=up->raset+up->xcset+up->sectionset;
+  checksum=up->raset+up->xcset+up->sectionset+up->polygonset;
   if(checksum)
     {
-      if(up->raset) t1="(`--ra` and `--dec`) ";
-      if(up->xcset) t2="(`--xc` and `--yc`) ";
-      if(up->sectionset) t3="(`--section) ";
-
       /* Only one of the three should be called. */
       if(checksum!=1)
-	error(EXIT_FAILURE, 0, "There are several ways to specify a crop "
-	      "box on the command line, see `--help`. But they should not "
-	      "be called together. You have asked for "
-	      "%s%s%s simultaneously!", t1?t1:"", t2?t2:"", t3?t3:"");
+        {
+          forerr[0]='\0';
+          if(up->raset)
+            strcat(forerr, "(`--ra' and `--dec'), ");
+          if(up->xcset)
+            strcat(forerr, "(`--xc' and `--yc'), ");
+          if(up->sectionset)
+            strcat(forerr, "(`--section'), ");
+          if(up->polygonset)
+            strcat(forerr, "(`--polygon'), ");
+          error(EXIT_FAILURE, 0, "there are several ways to specify a crop "
+                "box on the command line, see `--help`. But they should not "
+                "be called together. You have asked for %s simultaneously",
+                forerr);
+        }
 
       /* Check if the value for --output is a file or a directory? */
-      p->outnameisfile=dir0file1(cp->output, cp->dontdelete);
+      p->outnameisfile=gal_checkset_dir_0_file_1(cp->output, cp->dontdelete);
 
       /* When there is only one output, only one thread is needed. */
       cp->numthreads=1;
@@ -437,10 +462,13 @@ sanitycheck(struct imgcropparams *p)
       /* Not with a catalog. */
       if(up->catname)
 	{
-	  if(t1) boxparam=t1; else if(t2) boxparam=t2; else boxparam=t3;
-	  error(EXIT_FAILURE, 0, "A catalog name (%s) and one box parameters "
-		"%s""cannot be given together on the command line. ",
-		up->catname, boxparam);
+          if(up->sectionset) strcpy(forerr, "`--section'");
+          if(up->polygonset) strcpy(forerr, "`--polygon'");
+          if(up->xcset) strcpy(forerr, "`--xc' and `--yc'");
+	  if(up->raset) strcpy(forerr, "`--ra' and `--dec'");
+	  error(EXIT_FAILURE, 0, "a catalog name (%s) and command line crop "
+		"parameters (%s) cannot be given together", up->catname,
+                forerr);
 	}
     }
   else
@@ -449,38 +477,38 @@ sanitycheck(struct imgcropparams *p)
 	 command line, in the steps above, we set the image mode or
 	 wcs mode.*/
       if(p->imgmode && p->wcsmode)
-	error(EXIT_FAILURE, 0, "Only one of imgmode or wcsmode "
-	      "must be called. They cannot operate together.");
+	error(EXIT_FAILURE, 0, "only one of imgmode or wcsmode "
+	      "must be called. They cannot operate together");
       else if(p->imgmode==0 && p->wcsmode==0)
-	error(EXIT_FAILURE, 0, "At least one of imgmode or "
-	      "wcsmode must be called.");
+	error(EXIT_FAILURE, 0, "at least one of imgmode or "
+	      "wcsmode must be called");
 
       /* Make sure a catalog is set. */
       if(up->catset)
 	{
 	  if(p->numimg>1 && p->imgmode)
-	    error(EXIT_FAILURE, 0, "In image mode, when a catalog is "
-		  "specified, only one image may be provided.");
+	    error(EXIT_FAILURE, 0, "in image mode, when a catalog is "
+		  "specified, only one image may be provided");
 	}
       else
-	error(EXIT_FAILURE, 0, "No catalog. When no crop coordinates "
+	error(EXIT_FAILURE, 0, "no catalog. When no crop coordinates "
 	      "are specified on the command line, a catalog must be "
-	      "provided.");
+	      "provided");
 
       /* Make sure the given output is a directory. */
-      checkdirwriteaddslash(&cp->output);
+      gal_checkset_check_dir_write_add_slash(&cp->output);
 
       /* Make sure the columns of data are within the catalog range of
 	 columns: */
       if(p->imgmode)
 	{
-	  CHECKCOLINCAT(p->xcol, "xcol");
-	  CHECKCOLINCAT(p->ycol, "ycol");
+	  GAL_CHECKSET_CHECK_COL_IN_CAT(p->xcol, "xcol");
+	  GAL_CHECKSET_CHECK_COL_IN_CAT(p->ycol, "ycol");
 	}
       else
 	{
-	  CHECKCOLINCAT(p->racol, "racol");
-	  CHECKCOLINCAT(p->deccol, "deccol");
+	  GAL_CHECKSET_CHECK_COL_IN_CAT(p->racol, "racol");
+	  GAL_CHECKSET_CHECK_COL_IN_CAT(p->deccol, "deccol");
 	}
     }
 
@@ -488,8 +516,8 @@ sanitycheck(struct imgcropparams *p)
 
   /* If in image mode, there should only be one input image. */
   if(p->imgmode && p->numimg>1)
-    error(EXIT_FAILURE, 0, "In image mode, only one input image may be "
-	  "specified.");
+    error(EXIT_FAILURE, 0, "in image mode, only one input image may be "
+	  "specified");
 
 
 
@@ -497,7 +525,27 @@ sanitycheck(struct imgcropparams *p)
   /* If we are in WCS mode, noblanks must be off */
   if(p->wcsmode && p->noblank)
     error(EXIT_FAILURE, 0, "`--noblanks` (`-b`) is only for image mode. "
-	  "You have called it with WCS mode.");
+	  "You have called it with WCS mode");
+
+
+
+  /* Parse the polygon vertices if they are given to make sure that
+     there is no error: */
+  if(p->up.polygonset)
+    {
+      polygonparser(p);
+      if(p->nvertices<3)
+        error(EXIT_FAILURE, 0, "a polygon has to have 3 or more vertices, "
+              "you have only given %lu (%s)", p->nvertices, p->up.polygon);
+      if(p->outpolygon && p->numimg>1)
+        error(EXIT_FAILURE, 0, "currently in WCS mode, outpolygon can only "
+              "be set to zero when there is one image, you have given %lu "
+              "images. For multiple images the region will be very large. "
+              "It is best if you first crop out the larger region you want "
+              "into one image, then mask the polygon", p->numimg);
+    }
+  else
+    p->wpolygon=p->ipolygon=NULL;
 
 
 
@@ -512,7 +560,7 @@ sanitycheck(struct imgcropparams *p)
 	  "again with this flag to use multiple threads, run `info %s "
 	  "CFITSIO` for more information. Alternatively you can set "
 	  "the number of threads to 1 by adding the `--numthreads=1` "
-	  "or `-N1` options.", cp->numthreads, PACKAGE);
+	  "or `-N1` options", cp->numthreads, PACKAGE);
 }
 
 
@@ -546,7 +594,7 @@ preparearrays(struct imgcropparams *p)
   fitsfile *tmpfits;
   struct timeval t1;
   struct inputimgs *img;
-  char msg[VERBMSGLENGTH_V];
+  char msg[GAL_TIMING_VERB_MSG_LENGTH_V];
   int i, status, firstbitpix=0;
 
   if(p->cp.verb) gettimeofday(&t1, NULL);
@@ -571,39 +619,40 @@ preparearrays(struct imgcropparams *p)
       /* Get the image properties. */
       status=0;
       img=&p->imgs[i];
-      pop_from_stll(&p->up.stll, &img->name);
-      readfitshdu(img->name, p->cp.hdu, IMAGE_HDU, &tmpfits);
-      imgbitpixsize(tmpfits, &p->bitpix, img->naxes);
-      readwcs(tmpfits, &img->nwcs, &img->wcs);
+      gal_linkedlist_pop_from_stll(&p->up.gal_linkedlist_stll, &img->name);
+      gal_fitsarray_read_fits_hdu(img->name, p->cp.hdu, IMAGE_HDU, &tmpfits);
+      gal_fitsarray_img_bitpix_size(tmpfits, &p->bitpix, img->naxes);
+      gal_fitsarray_read_wcs(tmpfits, &img->nwcs, &img->wcs, p->hstartwcs,
+                             p->hendwcs);
       if(img->wcs)
         {
-          status=wcshdo(WCSHDO_safe, img->wcs, &img->nwcskeys, &img->wcstxt);
+          status=wcshdo(0, img->wcs, &img->nwcskeys, &img->wcstxt);
           if(status)
-            error(EXIT_FAILURE, 0, "wcshdo ERROR %d: %s.", status,
+            error(EXIT_FAILURE, 0, "wcshdo ERROR %d: %s", status,
                   wcs_errmsg[status]);
         }
       else
         if(p->wcsmode)
-          error(EXIT_FAILURE, 0, "The WCS structure of %s (hdu: %s) "
+          error(EXIT_FAILURE, 0, "the WCS structure of %s (hdu: %s) "
                 "image is not recognized. So RA and Dec cannot be used "
                 "as input. You can try with pixel coordinates in the "
                 "Image Mode (note that the crops will lack WCS "
-                "header information).", img->name, p->cp.hdu);
+                "header information)", img->name, p->cp.hdu);
       fits_close_file(tmpfits, &status);
-      fitsioerror(status, NULL);
+      gal_fitsarray_io_error(status, NULL);
 
       /* Make sure all the images have the same BITPIX and set the
 	 basic BITPIX related parameters. */
       if(firstbitpix==0)
 	{
 	  firstbitpix=p->bitpix;
-	  p->datatype=bitpixtodtype(p->bitpix);
-	  p->bitnul=bitpixblank(p->bitpix);
+	  p->datatype=gal_fitsarray_bitpix_to_dtype(p->bitpix);
+	  p->bitnul=gal_fitsarray_bitpix_blank(p->bitpix);
 	}
       else if(firstbitpix!=p->bitpix)
 	error(EXIT_FAILURE, 0, "%s: BITPIX=%d. Previous images had a "
 	      "BITPIX value of %d, For "SPACK_NAME" to work, all images "
-	      "must have the same pixel data type.",
+	      "must have the same pixel data type",
 	      img->name, p->bitpix, firstbitpix);
 
       /* In WCS mode, Check resolution and get the first pixel
@@ -615,7 +664,7 @@ preparearrays(struct imgcropparams *p)
      numbers so that we can put a NULL character in the name section
      of it to sign its end (something like a string). This is done so
      we don't have to worry about the length calculation any more! */
-  if(p->up.xcset || p->up.sectionset || p->up.raset)
+  if(p->up.xcset || p->up.sectionset || p->up.raset || p->up.polygonset)
     num=1;
   else
     num=p->cs0;
@@ -629,7 +678,7 @@ preparearrays(struct imgcropparams *p)
   if(p->cp.verb)
     {
       sprintf(msg, "Read metadata of %lu images.", p->numimg);
-      reporttiming(&t1, msg, 1);
+      gal_timing_report(&t1, msg, 1);
     }
 }
 
@@ -657,33 +706,33 @@ preparearrays(struct imgcropparams *p)
 void
 setparams(int argc, char *argv[], struct imgcropparams *p)
 {
-  struct commonparams *cp=&p->cp;
+  struct gal_commonparams *cp=&p->cp;
 
   /* Set the non-zero initial values, the structure was initialized to
      have a zero value for all elements. */
   cp->spack         = SPACK;
   cp->verb          = 1;
-  cp->numthreads    = DP_NUMTHREADS;
+  cp->numthreads    = num_processors(NPROC_CURRENT);
   cp->removedirinfo = 1;
 
   /* Read the arguments. */
   errno=0;
   if(argp_parse(&thisargp, argc, argv, 0, 0, p))
-    error(EXIT_FAILURE, errno, "Parsing arguments");
+    error(EXIT_FAILURE, errno, "parsing arguments");
 
   /* Add the user default values and save them if asked. */
-  CHECKSETCONFIG;
+  GAL_CONFIGFILES_CHECK_SET_CONFIG;
 
   /* Check if all the required parameters are set. */
   checkifset(p);
 
   /* Print the values for each parameter. */
   if(cp->printparams)
-    REPORT_PARAMETERS_SET;
+    GAL_CONFIGFILES_REPORT_PARAMETERS_SET;
 
   /* Read catalog if given. */
   if(p->up.catname)
-    txttoarray(p->up.catname, &p->cat, &p->cs0, &p->cs1);
+    gal_txtarray_txt_to_array(p->up.catname, &p->cat, &p->cs0, &p->cs1);
 
   /* If cp->output was not specified on the command line or in any of
      the configuration files, then automatic output should be used, in
@@ -692,14 +741,14 @@ setparams(int argc, char *argv[], struct imgcropparams *p)
     {
       p->cp.output=malloc(2+1); /* 2 is length of "./" */
       if(p->cp.output==NULL)
-        error(EXIT_FAILURE, errno, "Space for output");
+        error(EXIT_FAILURE, errno, "space for output");
       strcpy(p->cp.output, "./");
       p->cp.outputset=1;
     }
 
   /* Do a sanity check. */
   sanitycheck(p);
-  checkremovefile(TXTARRAYVVLOG, 0);
+  gal_checkset_check_remove_file(GAL_TXTARRAY_LOG, 0);
 
   /* Everything is ready, notify the user of the program starting. */
   if(cp->verb)
@@ -742,6 +791,8 @@ freeandreport(struct imgcropparams *p, struct timeval *t1)
   free(p->cp.hdu);
   free(p->bitnul);
   free(p->suffix);
+  free(p->wpolygon);
+  free(p->ipolygon);
 
   /* If these two pointers point to the same place,, that plce will be
      freed below. */
@@ -755,7 +806,7 @@ freeandreport(struct imgcropparams *p, struct timeval *t1)
         free(p->imgs[i].wcstxt);
         status=wcsvfree(&p->imgs[i].nwcs, &p->imgs[i].wcs);
         if(status)
-          error(EXIT_FAILURE, 0, "wcsvfree ERROR %d: %s.", status,
+          error(EXIT_FAILURE, 0, "wcsvfree ERROR %d: %s", status,
                 wcs_errmsg[status]);
       }
   free(p->imgs);
@@ -767,5 +818,5 @@ freeandreport(struct imgcropparams *p, struct timeval *t1)
 
   /* Print the final message. */
   if(p->cp.verb)
-    reporttiming(t1, SPACK_NAME" finished in: ", 0);
+    gal_timing_report(t1, SPACK_NAME" finished in: ", 0);
 }
