@@ -18,7 +18,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with gnuastro. If not, see <http://www.gnu.org/licenses/>.
+along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 **********************************************************************/
 #include <config.h>
 
@@ -30,16 +30,18 @@ along with gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <string.h>
 #include <fitsio.h>
 
-#include "timing.h"	/* Includes time.h and sys/time.h   */
-#include "checkset.h"
-#include "forqsort.h"
-#include "arraymanip.h"
-#include "statistics.h"
-#include "txtarrayvv.h"
-#include "commonargs.h"
-#include "configfiles.h"
-#include "fitsarrayvv.h"
-#include "fixedstringmacros.h"
+#include <nproc.h>              /* From Gnulib.                     */
+
+#include <gnuastro/qsort.h>
+#include <gnuastro/timing.h>	/* Includes time.h and sys/time.h   */
+#include <gnuastro/checkset.h>
+#include <gnuastro/arraymanip.h>
+#include <gnuastro/statistics.h>
+#include <gnuastro/txtarrayvv.h>
+#include <gnuastro/commonargs.h>
+#include <gnuastro/configfiles.h>
+#include <gnuastro/fitsarrayvv.h>
+#include <gnuastro/fixedstringmacros.h>
 
 #include "main.h"
 
@@ -73,7 +75,7 @@ readconfig(char *filename, struct imgstatparams *p)
   size_t lineno=0, len=200;
   char *line, *name, *value;
   struct uiparams *up=&p->up;
-  struct commonparams *cp=&p->cp;
+  struct gal_commonparams *cp=&p->cp;
   char key='a';	/* Not used, just a place holder. */
 
   /* When the file doesn't exist or can't be opened, it is ignored. It
@@ -97,45 +99,46 @@ readconfig(char *filename, struct imgstatparams *p)
   while(getline(&line, &len, fp) != -1)
     {
       /* Prepare the "name" and "value" strings, also set lineno. */
-      STARTREADINGLINE;
+      GAL_CONFIGFILES_START_READING_LINE;
 
 
 
 
       /* Inputs: */
       if(strcmp(name, "hdu")==0)
-        allocatecopyset(value, &cp->hdu, &cp->hduset);
+        gal_checkset_allocate_copy_set(value, &cp->hdu, &cp->hduset);
 
       else if (strcmp(name, "mask")==0)
-        allocatecopyset(value, &up->maskname, &up->masknameset);
+        gal_checkset_allocate_copy_set(value, &up->maskname, &up->masknameset);
 
       else if (strcmp(name, "mhdu")==0)
-        allocatecopyset(value, &up->mhdu, &up->mhduset);
+        gal_checkset_allocate_copy_set(value, &up->mhdu, &up->mhduset);
       else if(strcmp(name, "mirrordist")==0)
 	{
 	  if(up->mirrordistset) continue;
-          floatl0(value, &p->mirrordist, name, key, SPACK,
-                     filename, lineno);
+          gal_checkset_float_l_0(value, &p->mirrordist, name, key, SPACK,
+                                 filename, lineno);
           up->mirrordistset=1;
 	}
 
 
+
       /* Outputs */
       else if(strcmp(name, "output")==0)
-        allocatecopyset(value, &cp->output, &cp->outputset);
+        gal_checkset_allocate_copy_set(value, &cp->output, &cp->outputset);
 
       else if(strcmp(name, "mirrorplotdist")==0)
 	{
 	  if(up->mirrorplotdistset) continue;
-          floatl0(value, &p->mirrorplotdist, name, key, SPACK,
-                    filename, lineno);
+          gal_checkset_float_l_0(value, &p->mirrorplotdist, name, key, SPACK,
+                                 filename, lineno);
           up->mirrorplotdistset=1;
 	}
       else if(strcmp(name, "onebinvalue")==0)
 	{
 	  if(up->onebinvalueset) continue;
-          anyfloat(value, &p->onebinvalue, name, key, SPACK,
-                     filename, lineno);
+          gal_checkset_any_float(value, &p->onebinvalue, name, key, SPACK,
+                                 filename, lineno);
           up->onebinvalueset=1;
 	}
 
@@ -144,29 +147,29 @@ readconfig(char *filename, struct imgstatparams *p)
       else if(strcmp(name, "histnumbins")==0)
 	{
 	  if(up->histnumbinsset) continue;
-          sizetlzero(value, &p->histnumbins, name, key, SPACK,
-                     filename, lineno);
+          gal_checkset_sizet_l_zero(value, &p->histnumbins, name, key, SPACK,
+                                    filename, lineno);
           up->histnumbinsset=1;
 	}
       else if(strcmp(name, "histmin")==0)
 	{
 	  if(up->histminset) continue;
-          anyfloat(value, &p->histmin, name, key, SPACK,
-                     filename, lineno);
+          gal_checkset_any_float(value, &p->histmin, name, key, SPACK,
+                                 filename, lineno);
           up->histminset=1;
 	}
       else if(strcmp(name, "histmax")==0)
 	{
 	  if(up->histmaxset) continue;
-          anyfloat(value, &p->histmax, name, key, SPACK,
-                   filename, lineno);
+          gal_checkset_any_float(value, &p->histmax, name, key, SPACK,
+                                 filename, lineno);
           up->histmaxset=1;
 	}
       else if(strcmp(name, "histquant")==0)
 	{
 	  if(up->histquantset) continue;
-          floatl0s1(value, &p->histquant, name, key, SPACK,
-                    filename, lineno);
+          gal_checkset_float_l_0_s_1(value, &p->histquant, name, key, SPACK,
+                                     filename, lineno);
           up->histquantset=1;
 	}
 
@@ -175,29 +178,29 @@ readconfig(char *filename, struct imgstatparams *p)
       else if(strcmp(name, "cfpnum")==0)
 	{
 	  if(up->cfpnumset) continue;
-          sizetlzero(value, &p->cfpnum, name, key, SPACK,
-                     filename, lineno);
+          gal_checkset_sizet_l_zero(value, &p->cfpnum, name, key, SPACK,
+                                    filename, lineno);
           up->cfpnumset=1;
 	}
       else if(strcmp(name, "cfpmin")==0)
 	{
 	  if(up->cfpminset) continue;
-          anyfloat(value, &p->cfpmin, name, key, SPACK,
-                     filename, lineno);
+          gal_checkset_any_float(value, &p->cfpmin, name, key, SPACK,
+                                 filename, lineno);
           up->cfpminset=1;
 	}
       else if(strcmp(name, "cfpmax")==0)
 	{
 	  if(up->cfpmaxset) continue;
-          anyfloat(value, &p->cfpmax, name, key, SPACK,
-                     filename, lineno);
+          gal_checkset_any_float(value, &p->cfpmax, name, key, SPACK,
+                                 filename, lineno);
           up->cfpmaxset=1;
 	}
       else if(strcmp(name, "cfpquant")==0)
 	{
 	  if(up->cfpquantset) continue;
-          floatl0s1(value, &p->cfpquant, name, key, SPACK,
-                    filename, lineno);
+          gal_checkset_float_l_0_s_1(value, &p->cfpquant, name, key, SPACK,
+                                     filename, lineno);
           up->cfpquantset=1;
 	}
 
@@ -205,28 +208,29 @@ readconfig(char *filename, struct imgstatparams *p)
       else if(strcmp(name, "sigclipmultip")==0)
 	{
 	  if(up->sigclipmultipset) continue;
-          floatl0(value, &p->sigclipmultip, name, key, SPACK,
-                    filename, lineno);
+          gal_checkset_float_l_0(value, &p->sigclipmultip, name, key, SPACK,
+                                 filename, lineno);
           up->sigclipmultipset=1;
 	}
       else if(strcmp(name, "sigcliptolerance")==0)
 	{
 	  if(up->sigcliptoleranceset) continue;
-          floatl0(value, &p->sigcliptolerance, name, key, SPACK,
-                    filename, lineno);
+          gal_checkset_float_l_0(value, &p->sigcliptolerance, name, key, SPACK,
+                                 filename, lineno);
           up->sigcliptoleranceset=1;
 	}
       else if(strcmp(name, "sigclipnum")==0)
 	{
 	  if(up->sigclipnumset) continue;
-          sizetlzero(value, &p->sigclipnum, name, key, SPACK,
-                     filename, lineno);
+          gal_checkset_sizet_l_zero(value, &p->sigclipnum, name, key, SPACK,
+                                    filename, lineno);
           up->sigclipnumset=1;
 	}
 
 
+      /* Operating modes: */
       /* Read options common to all programs */
-      READ_COMMONOPTIONS_FROM_CONF
+      GAL_CONFIGFILES_READ_COMMONOPTIONS_FROM_CONF
 
 
       else
@@ -246,17 +250,17 @@ void
 printvalues(FILE *fp, struct imgstatparams *p)
 {
   struct uiparams *up=&p->up;
-  struct commonparams *cp=&p->cp;
+  struct gal_commonparams *cp=&p->cp;
 
   /* Print all the options that are set. Separate each group with a
      commented line explaining the options in that group. */
   fprintf(fp, "\n# Input image:\n");
   if(cp->hduset)
-    PRINTSTINGMAYBEWITHSPACE("hdu", cp->hdu);
+    GAL_CHECKSET_PRINT_STRING_MAYBE_WITH_SPACE("hdu", cp->hdu);
   if(up->masknameset)
-    PRINTSTINGMAYBEWITHSPACE("mask", up->maskname);
+    GAL_CHECKSET_PRINT_STRING_MAYBE_WITH_SPACE("mask", up->maskname);
   if(up->mhdu)
-    PRINTSTINGMAYBEWITHSPACE("mhdu", up->mhdu);
+    GAL_CHECKSET_PRINT_STRING_MAYBE_WITH_SPACE("mhdu", up->mhdu);
   if(up->mirrordistset)
     fprintf(fp, CONF_SHOWFMT"%.2f\n", "mirrordist", p->mirrordist);
 
@@ -300,10 +304,12 @@ printvalues(FILE *fp, struct imgstatparams *p)
   if(up->sigclipnumset)
     fprintf(fp, CONF_SHOWFMT"%lu\n", "sigclipnum", p->sigclipnum);
 
+
   /* For the operating mode, first put the macro to print the common
      options, then the (possible options particular to this
-     program) */
-  PRINT_COMMONOPTIONS;
+     program). */
+  fprintf(fp, "\n# Operating mode:\n");
+  GAL_CONFIGFILES_PRINT_COMMONOPTIONS;
 }
 
 
@@ -318,30 +324,30 @@ void
 checkifset(struct imgstatparams *p)
 {
   struct uiparams *up=&p->up;
-  struct commonparams *cp=&p->cp;
+  struct gal_commonparams *cp=&p->cp;
 
   int intro=0;
   if(cp->hduset==0)
-    REPORT_NOTSET("hdu");
+    GAL_CONFIGFILES_REPORT_NOTSET("hdu");
   if(up->mirrordistset==0)
-    REPORT_NOTSET("mirrordist");
+    GAL_CONFIGFILES_REPORT_NOTSET("mirrordist");
   if(up->mirrorplotdistset==0)
-    REPORT_NOTSET("mirrorplotdist");
+    GAL_CONFIGFILES_REPORT_NOTSET("mirrorplotdist");
   if(up->onebinvalueset==0)
-    REPORT_NOTSET("onebinvalue");
+    GAL_CONFIGFILES_REPORT_NOTSET("onebinvalue");
   if(up->histnumbinsset==0)
-    REPORT_NOTSET("histnumbins");
+    GAL_CONFIGFILES_REPORT_NOTSET("histnumbins");
   if(up->cfpnumset==0)
-    REPORT_NOTSET("cfpnum");
+    GAL_CONFIGFILES_REPORT_NOTSET("cfpnum");
   if(up->sigclipmultipset==0)
-    REPORT_NOTSET("sigclipmultip");
+    GAL_CONFIGFILES_REPORT_NOTSET("sigclipmultip");
   if(up->sigcliptoleranceset==0)
-    REPORT_NOTSET("sigcliptolerance");
+    GAL_CONFIGFILES_REPORT_NOTSET("sigcliptolerance");
   if(up->sigclipnumset==0)
-    REPORT_NOTSET("sigclipnum");
+    GAL_CONFIGFILES_REPORT_NOTSET("sigclipnum");
 
 
-  END_OF_NOTSET_REPORT;
+  GAL_CONFIGFILES_END_OF_NOTSET_REPORT;
 }
 
 
@@ -371,9 +377,13 @@ sanitycheck(struct imgstatparams *p)
 {
   char *basename;
 
+  /* Make sure the input file exists. */
+  gal_checkset_check_file(p->up.inputname);
+
   /* Set the p->up.maskname accordingly: */
-  fileorextname(p->up.inputname, p->cp.hdu, p->up.masknameset,
-                &p->up.maskname, p->up.mhdu, p->up.mhduset, "mask");
+  gal_fitsarray_file_or_ext_name(p->up.inputname, p->cp.hdu, p->up.masknameset,
+                                 &p->up.maskname, p->up.mhdu,
+                                 p->up.mhduset, "mask");
 
   /* Set the names of the output files: */
   if(p->cp.outputset) basename=p->cp.output;
@@ -381,30 +391,34 @@ sanitycheck(struct imgstatparams *p)
   if(p->histname)
     {
       p->histname=NULL;         /* It wasn't allocated. */
-      automaticoutput(basename, "_hist.txt", p->cp.removedirinfo,
-                      p->cp.dontdelete, &p->histname);
+      gal_checkset_automatic_output(basename, "_hist.txt", p->cp.removedirinfo,
+                                    p->cp.dontdelete, &p->histname);
     }
   if(p->cfpname)
     {
       p->cfpname=NULL;         /* It wasn't allocated. */
-      automaticoutput(basename, "_cfp.txt", p->cp.removedirinfo,
-                      p->cp.dontdelete, &p->cfpname);
+      gal_checkset_automatic_output(basename, "_cfp.txt", p->cp.removedirinfo,
+                                    p->cp.dontdelete, &p->cfpname);
     }
   if(p->mhistname)              /* The mode mirror distribution will need */
     {                           /* both a histogram and cfp.              */
       p->mcfpname=p->mhistname=NULL;
-      automaticoutput(basename, "_modehist.txt", p->cp.removedirinfo,
-                      p->cp.dontdelete, &p->mhistname);
-      automaticoutput(basename, "_modecfp.txt", p->cp.removedirinfo,
-                      p->cp.dontdelete, &p->mcfpname);
+      gal_checkset_automatic_output(basename, "_modehist.txt",
+                                    p->cp.removedirinfo, p->cp.dontdelete,
+                                    &p->mhistname);
+      gal_checkset_automatic_output(basename, "_modecfp.txt",
+                                    p->cp.removedirinfo, p->cp.dontdelete,
+                                    &p->mcfpname);
     }
   if(isnan(p->mirror)==0)
     {
       p->mirrorhist=p->mirrorcfp=NULL;
-      automaticoutput(basename, "_mirrorhist.txt", p->cp.removedirinfo,
-                      p->cp.dontdelete, &p->mirrorhist);
-      automaticoutput(basename, "_mirrorcfp.txt", p->cp.removedirinfo,
-                      p->cp.dontdelete, &p->mirrorcfp);
+      gal_checkset_automatic_output(basename, "_mirrorhist.txt",
+                                    p->cp.removedirinfo, p->cp.dontdelete,
+                                    &p->mirrorhist);
+      gal_checkset_automatic_output(basename, "_mirrorcfp.txt",
+                                    p->cp.removedirinfo, p->cp.dontdelete,
+                                    &p->mirrorcfp);
     }
 
 
@@ -413,23 +427,23 @@ sanitycheck(struct imgstatparams *p)
   if(p->cfpname && p->histname==NULL)
     {
       if(p->cfpsimhist)
-	error(EXIT_FAILURE, 0, "Without a histogram, `--cfpsimhist` is "
-              "meaningless.");
+	error(EXIT_FAILURE, 0, "without a histogram, `--cfpsimhist` is "
+              "meaningless");
       if (p->maxcfpeqmaxhist)
-	error(EXIT_FAILURE, 0, "Without a histogram, `--maxcfpeqmaxhist` "
-		"is meaningless.\n");
+	error(EXIT_FAILURE, 0, "without a histogram, `--maxcfpeqmaxhist` "
+		"is meaningless");
     }
 
   /* Check if `--maxcfpeqmaxhist` and `--normcfp` are not called
      together: */
   if(p->normcfp && p->maxcfpeqmaxhist)
     error(EXIT_FAILURE, 0, "`--normcfp` and `--maxcfpeqmaxhist` "
-          "cannot be called together.\n");
+          "cannot be called together\n");
 
   /* Check if `normhist` and `maxhistone` are not called together: */
   if(p->normhist && p->maxhistone)
     error(EXIT_FAILURE, 0, "`--normhist` and `--histnumbins` cannot be "
-	      "called together.\n");
+	      "called together\n");
 }
 
 
@@ -457,28 +471,28 @@ void
 preparearrays(struct imgstatparams *p)
 {
   float min;
-  int bitpix;
-  size_t numblank, s0, s1;
+  size_t s0, s1;
+  int bitpix, anyblank;
   struct uiparams *up=&p->up;
 
   /* Read the input and mask arrays: */
-  filetofloat(up->inputname, up->maskname, p->cp.hdu, up->mhdu,
-              &p->img, &bitpix, &numblank, &s0, &s1);
+  gal_fitsarray_file_to_float(up->inputname, up->maskname, p->cp.hdu, up->mhdu,
+                              &p->img, &bitpix, &anyblank, &s0, &s1);
   p->size=s0*s1;
 
   /* If the minimum value is to be used as a mask then do it: */
   if(p->ignoremin)
     {
-      floatmin(p->img, p->size, &min);
-      freplacevalue(p->img, p->size, min, NAN);
+      gal_statistics_float_min(p->img, p->size, &min);
+      gal_arraymanip_freplace_value(p->img, p->size, min, NAN);
     }
 
   /* Move all the non-nan elements to the start of the array: */
-  nonans(p->img, &p->size);
+  gal_arraymanip_no_nans(p->img, &p->size);
 
   /* Make a sorted array for most of the jobs: */
-  floatcopy(p->img, p->size, &p->sorted);
-  qsort(p->sorted, p->size, sizeof *p->sorted, floatincreasing);
+  gal_arraymanip_float_copy(p->img, p->size, &p->sorted);
+  qsort(p->sorted, p->size, sizeof *p->sorted, gal_qsort_float_increasing);
 
   /* Check the given range: */
   if(p->histname || p->asciihist || p->mhistname || p->mirrorhist)
@@ -486,11 +500,15 @@ preparearrays(struct imgstatparams *p)
       if(up->histquantset)
         {
           if(p->histquant>=0.5)
-            error(EXIT_FAILURE, 0, "The value to `--histquant' (-Q) must "
+            error(EXIT_FAILURE, 0, "the value to `--histquant' (-Q) must "
                   "Be smaller than 0.5, because it sets the lower limit of "
-                  "the value range. The higher limit will be 1-Q.");
-          p->histmin=p->sorted[indexfromquantile(p->size, p->histquant)];
-          p->histmax=p->sorted[indexfromquantile(p->size, 1 - p->histquant)];
+                  "the value range. The higher limit will be 1-Q");
+          p->histmin=
+            p->sorted[gal_statistics_index_from_quantile(p->size,
+                                                         p->histquant)];
+          p->histmax=
+            p->sorted[gal_statistics_index_from_quantile(p->size,
+                                                         1 - p->histquant)];
         }
       else
         {
@@ -501,29 +519,29 @@ preparearrays(struct imgstatparams *p)
               p->histmax=p->sorted[p->size-1];
               break;
             case 1:
-              error(EXIT_FAILURE, 0, "The options `--histmin' (-i) and "
+              error(EXIT_FAILURE, 0, "the options `--histmin' (-i) and "
                     "`--histmax' (-x) should both be specified. You have "
-                    "only given the %s."HOWTOCHECKVALUES,
+                    "only given the %s"GAL_STRINGS_HOW_TO_CHECK_VALUES,
                     up->histminset==1 ? "former" : "latter");
               break;
             case 2:
               if(p->histmin>=p->histmax)
-                error(EXIT_FAILURE, 0, "The value to `--histmin' (-i) (%f) "
-                      "is larger or equal to that of `--histmax' (-x) (%f)."
-                      HOWTOCHECKVALUES, p->histmin, p->histmax);
+                error(EXIT_FAILURE, 0, "the value to `--histmin' (-i) (%f) "
+                      "is larger or equal to that of `--histmax' (-x) (%f)"
+                      GAL_STRINGS_HOW_TO_CHECK_VALUES, p->histmin, p->histmax);
               if(p->histmin>p->sorted[p->size-1] || p->histmax<p->sorted[0])
-                error(EXIT_FAILURE, 0, "The range of data is %.5f to %.5f. "
+                error(EXIT_FAILURE, 0, "the range of data is %.5f to %.5f. "
                       "However, you have set `--histmin' (-i) and "
                       "`--histmax' (-x) to %.5f and %.5f respectively. "
-                      "They do not overlap!"HOWTOCHECKVALUES,
+                      "They do not overlap"GAL_STRINGS_HOW_TO_CHECK_VALUES,
                       p->sorted[0], p->sorted[p->size-1], p->histmin,
                       p->histmax);
               break;
             default:
-              error(EXIT_FAILURE, 0, "A bug! Please contact us at "
+              error(EXIT_FAILURE, 0, "a bug! Please contact us at "
                     PACKAGE_BUGREPORT" So we can solve the problem. the "
                     "value of up->histminset+up->histmaxset is not 0, 1 or "
-                    "2.");
+                    "2");
             }
         }
     }
@@ -538,11 +556,14 @@ preparearrays(struct imgstatparams *p)
       if(up->cfpquantset)
         {
           if(p->cfpquant>=0.5)
-            error(EXIT_FAILURE, 0, "The value to `--cfpquant' (-U) must "
+            error(EXIT_FAILURE, 0, "the value to `--cfpquant' (-U) must "
                   "Be smaller than 0.5, because it sets the lower limit of "
-                  "the value range. The higher limit will be 1-U.");
-          p->cfpmin=p->sorted[indexfromquantile(p->size, p->cfpquant)];
-          p->cfpmax=p->sorted[indexfromquantile(p->size, 1 - p->cfpquant)];
+                  "the value range. The higher limit will be 1-U");
+          p->cfpmin=p->sorted[gal_statistics_index_from_quantile(p->size,
+                                                                 p->cfpquant)];
+          p->cfpmax=
+            p->sorted[gal_statistics_index_from_quantile(p->size,
+                                                         1 - p->cfpquant)];
         }
       else
         {
@@ -553,29 +574,29 @@ preparearrays(struct imgstatparams *p)
               p->cfpmax=p->sorted[p->size-1];
               break;
             case 1:
-              error(EXIT_FAILURE, 0, "The options `--cfpmin' (-a) and "
+              error(EXIT_FAILURE, 0, "the options `--cfpmin' (-a) and "
                     "`--cfpmax' (-b) should both be specified. You have "
-                    "only given the %s."HOWTOCHECKVALUES,
+                    "only given the %s"GAL_STRINGS_HOW_TO_CHECK_VALUES,
                     up->cfpminset==1 ? "former" : "latter");
               break;
             case 2:
               if(p->cfpmin>p->cfpmax)
-                error(EXIT_FAILURE, 0, "The value to `--cfpmin' (-a) (%.f) "
-                      "is larger than that of `--cfpmax' (-b) (%f)."
-                      HOWTOCHECKVALUES, p->cfpmin, p->cfpmax);
+                error(EXIT_FAILURE, 0, "the value to `--cfpmin' (-a) (%.f) "
+                      "is larger than that of `--cfpmax' (-b) (%f)"
+                      GAL_STRINGS_HOW_TO_CHECK_VALUES, p->cfpmin, p->cfpmax);
               if(p->cfpmin>p->sorted[p->size-1] || p->cfpmax<p->sorted[0])
-                error(EXIT_FAILURE, 0, "The range of data is %.5f to %.5f. "
+                error(EXIT_FAILURE, 0, "the range of data is %.5f to %.5f. "
                       "However, you have set `--cfpmin' (-a) and "
                       "`--cfpmax' (-b) to %.5f and %.5f respectively. "
-                      "They do not overlap!"HOWTOCHECKVALUES,
+                      "They do not overlap"GAL_STRINGS_HOW_TO_CHECK_VALUES,
                       p->sorted[0], p->sorted[p->size-1], p->cfpmin,
                       p->cfpmax);
               break;
             default:
-              error(EXIT_FAILURE, 0, "A bug! Please contact us at "
+              error(EXIT_FAILURE, 0, "a bug! Please contact us at "
                     PACKAGE_BUGREPORT" So we can solve the problem. the "
                     "value of up->cfpminset+up->cfpmaxset is not 0, 1 or "
-                    "2.");
+                    "2");
             }
         }
     }
@@ -605,13 +626,13 @@ preparearrays(struct imgstatparams *p)
 void
 setparams(int argc, char *argv[], struct imgstatparams *p)
 {
-  struct commonparams *cp=&p->cp;
+  struct gal_commonparams *cp=&p->cp;
 
   /* Set the non-zero initial values, the structure was initialized to
      have a zero value for all elements. */
   cp->spack         = SPACK;
   cp->verb          = 1;
-  cp->numthreads    = DP_NUMTHREADS;
+  cp->numthreads    = num_processors(NPROC_CURRENT);
   cp->removedirinfo = 1;
 
   p->asciihist      = 1;
@@ -623,17 +644,17 @@ setparams(int argc, char *argv[], struct imgstatparams *p)
   /* Read the arguments. */
   errno=0;
   if(argp_parse(&thisargp, argc, argv, 0, 0, p))
-    error(EXIT_FAILURE, errno, "Parsing arguments");
+    error(EXIT_FAILURE, errno, "parsing arguments");
 
   /* Add the user default values and save them if asked. */
-  CHECKSETCONFIG;
+  GAL_CONFIGFILES_CHECK_SET_CONFIG;
 
   /* Check if all the required parameters are set. */
   checkifset(p);
 
   /* Print the values for each parameter. */
   if(cp->printparams)
-    REPORT_PARAMETERS_SET;
+    GAL_CONFIGFILES_REPORT_PARAMETERS_SET;
 
   /* Do a sanity check. */
   sanitycheck(p);
@@ -689,5 +710,5 @@ freeandreport(struct imgstatparams *p, struct timeval *t1)
 
   /* Print the final message. */
   if(p->cp.verb)
-    reporttiming(t1, SPACK_NAME" finished in: ", 0);
+    gal_timing_report(t1, SPACK_NAME" finished in: ", 0);
 }
