@@ -18,20 +18,20 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with gnuastro. If not, see <http://www.gnu.org/licenses/>.
+along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 **********************************************************************/
 #ifndef CITE_H
 #define CITE_H
 
-#define IMGCROPBIBTEX ""
+#define IMGWARPBIBTEX ""
 
 #define PRINTCITEABORT {						\
     printf("\nWe hope %s has been useful for your research.\n"		\
 	   "Citations are vital for the continued work on %s.\n"	\
 	   "Thank you for citing it in your research paper.\n"		\
 	   "\nPlease cite as \"%s\":\n\n%s\n\n%s",			\
-	   SPACK_NAME, SPACK_NAME, SPACK_STRING, GNUASTROBIBTEX,	\
-	   IMGCROPBIBTEX);						\
+	   SPACK_NAME, SPACK_NAME, SPACK_STRING,                        \
+           GAL_STRINGS_MAIN_BIBTEX, IMGWARPBIBTEX);		        \
     exit(EXIT_SUCCESS);							\
 }
 
