@@ -18,7 +18,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with gnuastro. If not, see <http://www.gnu.org/licenses/>.
+along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 **********************************************************************/
 #include <config.h>
 
@@ -29,14 +29,14 @@ along with gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <string.h>
 #include <stdlib.h>
 
-#include "box.h"
-#include "timing.h"
-#include "checkset.h"
-#include "statistics.h"
-#include "arraymanip.h"
-#include "txtarrayvv.h"
-#include "astrthreads.h"
-#include "fitsarrayvv.h"
+#include <gnuastro/box.h>
+#include <gnuastro/timing.h>
+#include <gnuastro/threads.h>
+#include <gnuastro/checkset.h>
+#include <gnuastro/statistics.h>
+#include <gnuastro/arraymanip.h>
+#include <gnuastro/txtarrayvv.h>
+#include <gnuastro/fitsarrayvv.h>
 
 #include "main.h"
 
@@ -72,7 +72,7 @@ preparewcs(struct mkprofparams *p)
   /* Initialize the structure (allocate all the arrays). */
   wcs.flag=-1;
   if( (status=wcsini(1, 2, &wcs)) )
-    error(EXIT_FAILURE, 0, "wcsinit error %d: %s.",
+    error(EXIT_FAILURE, 0, "wcsinit error %d: %s",
 	  status, wcs_errmsg[status]);
 
   /* Correct the CRPIX values. */
@@ -96,17 +96,17 @@ preparewcs(struct mkprofparams *p)
 
   /* Set up the wcs structure: */
   if( (status=wcsset(&wcs)) )
-    error(EXIT_FAILURE, 0, "wcsset error %d: %s.", status,
+    error(EXIT_FAILURE, 0, "wcsset error %d: %s", status,
 	  wcs_errmsg[status]);
 
   /* Write the WCS structure to a header string. */
   if( (status=wcshdo(WCSHDO_safe, &wcs, &p->wcsnkeyrec, &p->wcsheader)) )
-    error(EXIT_FAILURE, 0, "wcshdo error %d: %s.", status,
+    error(EXIT_FAILURE, 0, "wcshdo error %d: %s", status,
 	  wcs_errmsg[status]);
 
   /* Free the allocated spaces by wcsini/wcsset: */
   if( (status=wcsfree(&wcs)) )
-    error(EXIT_FAILURE, 0, "wcsfree error %d: %s.", status,
+    error(EXIT_FAILURE, 0, "wcsfree error %d: %s", status,
 	  wcs_errmsg[status]);
 }
 
@@ -142,7 +142,7 @@ builtqueue_addempty(struct builtqueue **bq)
   errno=0;
   tbq=malloc(sizeof *tbq);
   if(tbq==NULL)
-    error(EXIT_FAILURE, 0, "%lu byte element in builtqueue_addempty.",
+    error(EXIT_FAILURE, 0, "%lu byte element in builtqueue_addempty",
 	  sizeof *tbq);
 
   /* Initialize some of the values. */
@@ -185,23 +185,14 @@ saveindividual(struct mkonthread *mkp)
   struct mkprofparams *p=mkp->p;
 
   size_t len;
-  char *jobname;
   double crpix[2];
   long os=p->oversample;
   struct builtqueue *ibq=mkp->ibq;
-  char *outname, *outdir=p->cp.output;
-
-  /* Save the array to an image. */
-  if(p->dir0file1==0)
-    len=strlen(outdir)+NUMBERNAMESTRLEN;
-  else
-    {
-      outdir="";
-      len=NUMBERNAMESTRLEN;
-    }
+  char *outname, *jobname, *outdir=p->outdir;
 
   /* Allocate the space for the name of this file. */
   errno=0;
+  len=strlen(outdir)+NUMBERNAMESTRLEN+strlen(p->mergedimgname);
   outname=malloc(len*sizeof *outname);
   if(outname==NULL)
     error(EXIT_FAILURE, errno, "%lu bytes for name of object in "
@@ -213,26 +204,34 @@ saveindividual(struct mkonthread *mkp)
   crpix[1] = p->crpix[1] - os*(mkp->fpixel_i[1]-1);
 
   /* Write the name and save the FITS image: */
-  sprintf(outname, "%s%lu.fits", outdir, ibq->id);
-  checkremovefile(outname, p->cp.dontdelete);
+  sprintf(outname, "%s%lu_%s", outdir, ibq->id, p->basename);
+  gal_checkset_check_remove_file(outname, p->cp.dontdelete);
 
   /* Change NaN values to 0.0f: */
-  freplacevalue(ibq->img, mkp->width[1]*mkp->width[0], NAN, 0.0f);
+  gal_arraymanip_freplace_value(ibq->img, mkp->width[1]*mkp->width[0],
+                                NAN, 0.0f);
+  if(p->setconsttonan)
+    gal_arraymanip_freplace_value(ibq->img, mkp->width[1]*mkp->width[0],
+                                  CONSTFORNAN, NAN);
 
   /* Write the array to file (A separately built PSF doesn't need WCS
      coordinates): */
   if(ibq->ispsf && p->psfinimg==0)
-    arraytofitsimg(outname, "MockImg", FLOAT_IMG, ibq->img,
-		   mkp->width[1], mkp->width[0], 0, NULL, NULL,
-                   SPACK_STRING);
+    gal_fitsarray_array_to_fits_img(outname, "MockImg", FLOAT_IMG, ibq->img,
+                                    mkp->width[1], mkp->width[0], 0, NULL, NULL,
+                                    SPACK_STRING);
   else
-    atofcorrectwcs(outname, "MockImg", FLOAT_IMG, ibq->img,
-		   mkp->width[1], mkp->width[0], p->wcsheader,
-		   p->wcsnkeyrec, crpix, SPACK_STRING);
+    gal_fitsarray_atof_correct_wcs(outname, "MockImg", FLOAT_IMG, ibq->img,
+                                   mkp->width[1], mkp->width[0], p->wcsheader,
+                                   p->wcsnkeyrec, crpix, SPACK_STRING);
   ibq->indivcreated=1;
 
   /* Change 0.0f values to NAN: */
-  freplacevalue(ibq->img, mkp->width[1]*mkp->width[0], 0.0f, NAN);
+  if(p->setconsttonan)
+    gal_arraymanip_freplace_value(ibq->img, mkp->width[1]*mkp->width[0],
+                                  NAN, CONSTFORNAN);
+  gal_arraymanip_freplace_value(ibq->img, mkp->width[1]*mkp->width[0],
+                                0.0f, NAN);
 
   /* Report if in verbose mode. */
   if(p->cp.verb)
@@ -241,7 +240,7 @@ saveindividual(struct mkonthread *mkp)
       jobname=malloc((len+100)*sizeof *jobname);
       if(jobname==NULL)	error(EXIT_FAILURE, errno, "jobname in mkprof.c");
       sprintf(jobname, "%s created.", outname);
-      reporttiming(NULL, jobname, 2);
+      gal_timing_report(NULL, jobname, 2);
       free(jobname);
     }
   free(outname);
@@ -311,7 +310,7 @@ build(void *inparam)
   pthread_cond_t *qready=&p->qready;
 
   /* Make each profile that was specified for this thread. */
-  for(i=0;mkp->indexs[i]!=NONTHRDINDEX;++i)
+  for(i=0;mkp->indexs[i]!=GAL_THREADS_NON_THRD_INDEX;++i)
     {
       /* Create a new builtqueue element with all the information. fbq
 	 will be used when we want to add ibq to p->bq. It is defined
@@ -334,18 +333,18 @@ build(void *inparam)
       if((int)cat[p->fcol]==POINTCODE)
 	mkp->width[0]=mkp->width[1]=1;
       else
-	ellipseinbox(mkp->truncr, mkp->q*mkp->truncr,
-		     cat[p->pcol]*DEGREESTORADIANS, mkp->width);
+        gal_box_ellipse_in_box(mkp->truncr, mkp->q*mkp->truncr,
+                               cat[p->pcol]*DEGREESTORADIANS, mkp->width);
 
 
       /* Get the overlapping pixels using the starting points (NOT
 	 oversampled). */
-      borderfromcenter(cat[p->xcol], cat[p->ycol], mkp->width,
-		       ibq->fpixel_i, ibq->lpixel_i);
+      gal_box_border_from_center(cat[p->xcol], cat[p->ycol], mkp->width,
+                                 ibq->fpixel_i, ibq->lpixel_i);
       mkp->fpixel_i[0]=ibq->fpixel_i[0];
       mkp->fpixel_i[1]=ibq->fpixel_i[1];
-      ibq->overlaps = overlap(mkp->onaxes, ibq->fpixel_i, ibq->lpixel_i,
-			      ibq->fpixel_o, lpixel_o);
+      ibq->overlaps = gal_box_overlap(mkp->onaxes, ibq->fpixel_i, ibq->lpixel_i,
+                                      ibq->fpixel_o, lpixel_o);
 
 
       /* Build the profile if necessary, After this, the width is
@@ -359,7 +358,7 @@ build(void *inparam)
           /* Set the seed of the random number generator if the
              environment is not to be used. */
           if(mkp->p->envseed==0)
-            gsl_rng_set(mkp->rng, timebasedrngseed());
+            gsl_rng_set(mkp->rng, gal_timing_time_based_rng_seed());
 
           /* Make the profile */
 	  makeoneprofile(mkp);
@@ -409,7 +408,7 @@ build(void *inparam)
              this thread to add up its built profiles). So we have to
              lock the mutex to pass on this built structure to the
              builtqueue. */
-          else if (mkp->indexs[i+1]==NONTHRDINDEX)
+          else if (mkp->indexs[i+1]==GAL_THREADS_NON_THRD_INDEX)
             {
 	      pthread_mutex_lock(qlock);
 	      fbq->next=p->bq;
@@ -466,13 +465,14 @@ writelog(struct mkprofparams *p)
 	  "# Column 1: Overlap magnitude with final image "
 	  "(zeropoint: %.3f).\n"
 	  "# Column 2: Number of Monte Carlo integration pixels.\n"
-	  "# Column 3: Fraction of flux in Monte Carlo integration pixels.\n"
+	  "# Column 3: Fraction of brightness in Monte Carlo "
+          "integrated pixels.\n"
 	  "# Column 4: An individual image was created.\n",
 	  ctime(&p->rawtime), p->zeropoint);
 
 
-  arraytotxt(p->log, p->cs0, LOGNUMCOLS, comments, int_cols,
-	     accu_cols, space, prec, 'f', LOGFILENAME);
+  gal_txtarray_array_to_txt(p->log, p->cs0, LOGNUMCOLS, comments, int_cols,
+                            accu_cols, space, prec, 'f', LOGFILENAME);
 }
 
 
@@ -566,12 +566,13 @@ write(struct mkprofparams *p)
               colend=to+jw;
               do
                 {
-                if(!isnan(*from))
-                  {
-                    sum+=*from;
-                    *to = replace ? *from : *to+*from;
-                  }
-                ++from;
+                  if(!isnan(*from))
+                    {
+                      *from = p->setconsttonan ? NAN : *from;
+                      sum+=*from;
+                      *to = replace ? *from : *to+*from;
+                    }
+                  ++from;
                 }
               while(++to<colend);
               to+=w-jw; from+=ow-jw;	     /* Go to next row. */
@@ -595,9 +596,9 @@ write(struct mkprofparams *p)
 	  jobname=malloc(100*sizeof *jobname);
 	  if(jobname==NULL)
 	    error(EXIT_FAILURE, errno, "jobname in mkprof.c");
-	  sprintf(jobname, "Row %lu complete, %lu left to go.",
+	  sprintf(jobname, "row %lu complete, %lu left to go",
 		  ibq->id, cs0-complete);
-	  reporttiming(NULL, jobname, 2);
+	  gal_timing_report(NULL, jobname, 2);
 	  free(jobname);
 	}
 
@@ -618,17 +619,20 @@ write(struct mkprofparams *p)
       if(p->up.backname)
         {
           if(bitpix==FLOAT_IMG) array=out;
-          else changetype(p->out, FLOAT_IMG, p->naxes[1]*p->naxes[0],
-                          p->numblank, &array, bitpix);
-          arraytofitsimg(p->mergedimgname, "MockImg on back", bitpix,
-                         array, p->naxes[1], p->naxes[0], p->numblank,
-                         p->wcs, NULL, SPACK_STRING);
+          else gal_fitsarray_change_type(p->out, FLOAT_IMG,
+                                         p->naxes[1]*p->naxes[0],
+                                         p->anyblank, &array, bitpix);
+          gal_fitsarray_array_to_fits_img(p->mergedimgname, "MockImg on back",
+                                          bitpix, array, p->naxes[1],
+                                          p->naxes[0], p->anyblank, p->wcs,
+                                          NULL, SPACK_STRING);
           if(bitpix!=FLOAT_IMG) free(array);
         }
       else
-        atofcorrectwcs(p->mergedimgname, "MockImg", FLOAT_IMG, out,
-                       p->naxes[1], p->naxes[0], p->wcsheader,
-                       p->wcsnkeyrec, NULL, SPACK_STRING);
+        gal_fitsarray_atof_correct_wcs(p->mergedimgname, "MockImg", FLOAT_IMG,
+                                       out, p->naxes[1], p->naxes[0],
+                                       p->wcsheader, p->wcsnkeyrec,
+                                       NULL, SPACK_STRING);
       if(verb)
 	{
 	  errno=0;
@@ -636,7 +640,7 @@ write(struct mkprofparams *p)
 	  if(jobname==NULL)
 	    error(EXIT_FAILURE, errno, "final report in mkprof.c");
 	  sprintf(jobname, "%s created.", p->mergedimgname);
-	  reporttiming(&t1, jobname, 1);
+	  gal_timing_report(&t1, jobname, 1);
 	  free(jobname);
 	}
     }
@@ -695,7 +699,7 @@ mkprof(struct mkprofparams *p)
   /* Distribute the different profiles for different threads. Note
      that one thread is left out for writing, while nt-1 are left
      for building. */
-  distinthreads(p->cs0, nt, &indexs, &thrdcols);
+  gal_threads_dist_in_threads(p->cs0, nt, &indexs, &thrdcols);
 
   /* onaxes are sides of the image without over-sampling. */
   onaxes[0] = (p->naxes[0]-2*p->shift[0])/os + 2*p->shift[0]/os;
@@ -717,17 +721,17 @@ mkprof(struct mkprofparams *p)
 	 barrier stops. */
       if(p->cs0<nt) nb=p->cs0+1;
       else nb=nt+1;
-      attrbarrierinit(&attr, &b, nb);
+      gal_threads_attr_barrier_init(&attr, &b, nb);
 
       /* Initialize the condition variable and mutex. */
       err=pthread_mutex_init(&p->qlock, NULL);
-      if(err) error(EXIT_FAILURE, 0, "Mutex not initialized.");
+      if(err) error(EXIT_FAILURE, 0, "mutex not initialized");
       err=pthread_cond_init(&p->qready, NULL);
-      if(err) error(EXIT_FAILURE, 0, "Condition variable not initialized.");
+      if(err) error(EXIT_FAILURE, 0, "condition variable not initialized");
 
       /* Spin off the threads: */
       for(i=0;i<nt;++i)
-	if(indexs[i*thrdcols]!=NONTHRDINDEX)
+	if(indexs[i*thrdcols]!=GAL_THREADS_NON_THRD_INDEX)
 	  {
 	    mkp[i].p=p;
 	    mkp[i].b=&b;
@@ -737,13 +741,14 @@ mkprof(struct mkprofparams *p)
 	    mkp[i].indexs=&indexs[i*thrdcols];
 	    err=pthread_create(&t, &attr, build, &mkp[i]);
 	    if(err)
-	      error(EXIT_FAILURE, 0, "Can't create thread %lu.", i);
+	      error(EXIT_FAILURE, 0, "can't create thread %lu", i);
 	  }
     }
 
   /* Write the created arrays into the image. */
   write(p);
-  writelog(p);
+  if(p->cp.nolog==0)
+    writelog(p);
 
   /* If numthreads>1, then wait for all the jobs to finish and destroy
      the attribute and barrier. */
