@@ -18,15 +18,15 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with gnuastro. If not, see <http://www.gnu.org/licenses/>.
+along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 **********************************************************************/
 #ifndef ARGS_H
 #define ARGS_H
 
 #include <argp.h>
 
-#include "commonargs.h"
-#include "fixedstringmacros.h"
+#include <gnuastro/commonargs.h>
+#include <gnuastro/fixedstringmacros.h>
 
 
 
@@ -45,7 +45,7 @@ along with gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 
 /* Definition parameters for the argp: */
-const char *argp_program_version=SPACK_STRING"\n"COPYRIGHT
+const char *argp_program_version=SPACK_STRING"\n"GAL_STRINGS_COPYRIGHT
   "\n\nWritten by Mohammad Akhlaghi";
 const char *argp_program_bug_address=PACKAGE_BUGREPORT;
 static char args_doc[] = "ASTRdata";
@@ -56,12 +56,12 @@ static char args_doc[] = "ASTRdata";
 
 const char doc[] =
   /* Before the list of options: */
-  TOPHELPINFO
+  GAL_STRINGS_TOP_HELP_INFO
   SPACK_NAME" Detects and segments signal that is deeply burried in noise. "
   "It employs a noise-based detection and segmentation method enabling it "
   "to be very resilient to the rich diversity of shapes in astronomical "
   "targets.\n"
-  MOREHELPINFO
+  GAL_STRINGS_MORE_HELP_INFO
   /* After the list of options: */
   "\v"
   PACKAGE_NAME" home page: "PACKAGE_URL;
@@ -73,9 +73,9 @@ const char doc[] =
 /* Available letters for short options:
 
    f j w x z
-   A C J W X Y Z
+   A J K W X Y Z
 
-   Used numbers: <=519
+   Used numbers: <=518 (except 503, and 515)
 
    Options with keys (second structure element) larger than 500 do not
    have a short version.
@@ -160,11 +160,11 @@ static struct argp_option options[] =
       2
     },
     {
-      "saveskysubed",
-      518,
+      "grownclumps",
+      'C',
       0,
       0,
-      "Save the sky subtracted image, `_skysubed.fits'.",
+      "Save grownclumps instead of original clumps.",
       2
     },
 
@@ -380,7 +380,7 @@ static struct argp_option options[] =
       510,
       "INT",
       0,
-      "Detection S/N hist. N. bins `_XX_detsn.txt'.",
+      "Det. S/N histogram bins, suffix `_detsn.txt'.",
       4
     },
     {
@@ -424,14 +424,6 @@ static struct argp_option options[] =
       "Sky for false detections in file `_detsky.fits'.",
       4
     },
-    {
-      "checkdetectionsn",
-      503,
-      0,
-      0,
-      "Mesh detection S/N limit `_detsn.fits'.",
-      4
-    },
     {
       "checksky",
       512,
@@ -442,7 +434,7 @@ static struct argp_option options[] =
     },
     {
       "checkmaskdet",
-      519,
+      518,
       0,
       0,
       "Mask detections and sky, `_maskdet.fits'.",
@@ -467,17 +459,17 @@ static struct argp_option options[] =
     {
       "segquant",
       'g',
-      0,
+      "FLT",
       0,
       "Signal to noise ratio quantile for clumps.",
       5
     },
     {
-      "segsnhistnbins",
+      "clumpsnhistnbins",
       514,
       "INT",
       0,
-      "Segmentation S/N hist. N. bins `_XX_segsn.txt'.",
+      "Clump S/N histogram bins, suffix `_clumpsn.txt'.",
       5
     },
     {
@@ -491,7 +483,7 @@ static struct argp_option options[] =
     {
       "gthresh",
       'G',
-      0,
+      "FLT",
       0,
       "Threshold (STD multiple) to stop growing clumps.",
       5
@@ -521,14 +513,6 @@ static struct argp_option options[] =
       "Store segmentation steps in file `_seg.fits'.",
       5
     },
-    {
-      "checkclumpsn",
-      515,
-      0,
-      0,
-      "Mesh clump S/N limit `_clumpsn.fits'.",
-      4
-    },
 
 
     {
@@ -564,10 +548,10 @@ parse_opt(int key, char *arg, struct argp_state *state)
      check if the first character of arg is the equal sign, then the
      user is warned and the program is stopped: */
   if(arg && arg[0]=='=')
-    argp_error(state, "Incorrect use of the equal sign (`=`). For short "
+    argp_error(state, "incorrect use of the equal sign (`=`). For short "
 	       "options, `=` should not be used and for long options, "
 	       "there should be no space between the option, equal sign "
-	       "and value.");
+	       "and value");
 
   switch(key)
     {
@@ -575,27 +559,31 @@ parse_opt(int key, char *arg, struct argp_state *state)
 
     /* Input: */
     case 'M':
-      allocatecopyset(arg, &p->up.maskname, &p->up.masknameset);
+      gal_checkset_allocate_copy_set(arg, &p->up.maskname,
+                                     &p->up.masknameset);
       break;
     case 'H':
-      allocatecopyset(arg, &p->up.mhdu, &p->up.mhduset);
+      gal_checkset_allocate_copy_set(arg, &p->up.mhdu, &p->up.mhduset);
       break;
-    case 'K':
-      allocatecopyset(arg, &p->up.kernelname, &p->up.kernelnameset);
+    case 'k':
+      gal_checkset_allocate_copy_set(arg, &p->up.kernelname,
+                                     &p->up.kernelnameset);
       break;
     case 'U':
-      allocatecopyset(arg, &p->up.khdu, &p->up.khduset);
+      gal_checkset_allocate_copy_set(arg, &p->up.khdu, &p->up.khduset);
       break;
     case 'E':
       p->skysubtracted=1;
       p->up.skysubtractedset=1;
       break;
     case 'B':
-      floatl0s1(arg, &p->minbfrac, "minbfrac", key, SPACK, NULL, 0);
+      gal_checkset_float_l_0_s_1(arg, &p->minbfrac, "minbfrac", key,
+                                 SPACK, NULL, 0);
       p->up.minbfracset=1;
       break;
     case 'F':
-      sizetlzero(arg, &p->minnumfalse, "minnumfalse", key, SPACK, NULL, 0);
+      gal_checkset_sizet_l_zero(arg, &p->minnumfalse, "minnumfalse", key,
+                                SPACK, NULL, 0);
       p->up.minnumfalseset=1;
       break;
 
@@ -603,49 +591,58 @@ parse_opt(int key, char *arg, struct argp_state *state)
     case 517:
       p->detectonly=1;
       break;
-    case 518:
-      p->skysubedname="a";
+    case 'C':
+      p->grownclumps=1;
+      p->up.grownclumpsset=1;
       break;
 
     /* Mesh grid: */
     case 's':
-      sizetlzero(arg, &p->smp.meshsize, "smeshsize", key, SPACK, NULL, 0);
+      gal_checkset_sizet_l_zero(arg, &p->smp.meshsize, "smeshsize", key,
+                                SPACK, NULL, 0);
       p->up.smeshsizeset=1;
       break;
     case 'l':
-      sizetlzero(arg, &p->lmp.meshsize, "lmeshsize", key, SPACK, NULL, 0);
+      gal_checkset_sizet_l_zero(arg, &p->lmp.meshsize, "lmeshsize", key,
+                                SPACK, NULL, 0);
       p->up.lmeshsizeset=1;
       break;
     case 'a':
-      sizetlzero(arg, &p->smp.nch1, "nch1", key, SPACK, NULL, 0);
+      gal_checkset_sizet_l_zero(arg, &p->smp.nch1, "nch1", key, SPACK,
+                                NULL, 0);
       p->up.nch1set=1;
       break;
     case 'b':
-      sizetlzero(arg, &p->smp.nch2, "nch2", key, SPACK, NULL, 0);
+      gal_checkset_sizet_l_zero(arg, &p->smp.nch2, "nch2", key, SPACK,
+                                NULL, 0);
       p->up.nch2set=1;
       break;
     case 'L':
-      floatl0s1(arg, &p->smp.lastmeshfrac, "lastmeshfrac", key, SPACK,
-                NULL, 0);
+      gal_checkset_float_l_0_s_1(arg, &p->smp.lastmeshfrac, "lastmeshfrac",
+                                 key, SPACK, NULL, 0);
       p->up.lastmeshfracset=1;
       break;
     case 'd':
-      floatl0(arg, &p->smp.mirrordist, "mirrordist", key, SPACK, NULL, 0);
+      gal_checkset_float_l_0(arg, &p->smp.mirrordist, "mirrordist", key,
+                             SPACK, NULL, 0);
       p->up.mirrordistset=1;
       break;
     case 'Q':
-      floatl0s1(arg, &p->smp.minmodeq, "minmodeq", key, SPACK, NULL, 0);
+      gal_checkset_float_l_0_s_1(arg, &p->smp.minmodeq, "minmodeq", key,
+                                 SPACK, NULL, 0);
       p->up.minmodeqset=1;
       break;
     case 511:
       p->smp.interponlyblank=1;
       break;
     case 'n':
-      sizetlzero(arg, &p->smp.numnearest, "numnearest", key, SPACK, NULL, 0);
+      gal_checkset_sizet_l_zero(arg, &p->smp.numnearest, "numnearest", key,
+                                SPACK, NULL, 0);
       p->up.numnearestset=1;
       break;
     case 'T':
-      sizetpodd(arg, &p->smp.smoothwidth, "smoothwidth", key, SPACK, NULL, 0);
+      gal_checkset_sizet_p_odd(arg, &p->smp.smoothwidth, "smoothwidth", key,
+                               SPACK, NULL, 0);
       p->up.smoothwidthset=1;
       break;
     case 500:
@@ -670,54 +667,63 @@ parse_opt(int key, char *arg, struct argp_state *state)
 
     /* Detection */
     case 't':
-      floatl0s1(arg, &p->qthresh, "qthresh", key, SPACK, NULL, 0);
+      gal_checkset_float_l_0_s_1(arg, &p->qthresh, "qthresh", key, SPACK,
+                                 NULL, 0);
       p->up.qthreshset=1;
       break;
     case 'e':
-      sizetelzero(arg, &p->erode, "erode", key, SPACK, NULL, 0);
+      gal_checkset_sizet_el_zero(arg, &p->erode, "erode", key, SPACK,
+                                 NULL, 0);
       p->up.erodeset=1;
       break;
     case 506:
-      int4or8(arg, &p->erodengb, "erodengb", key, SPACK, NULL, 0);
+      gal_checkset_int_4_or_8(arg, &p->erodengb, "erodengb", key, SPACK,
+                              NULL, 0);
       p->up.erodengbset=1;
       break;
     case 'p':
-      sizetelzero(arg, &p->opening, "opening", key, SPACK, NULL, 0);
+      gal_checkset_sizet_el_zero(arg, &p->opening, "opening", key, SPACK,
+                                 NULL, 0);
       p->up.openingset=1;
       break;
     case 507:
-      int4or8(arg, &p->openingngb, "openingngb", key, SPACK, NULL, 0);
+      gal_checkset_int_4_or_8(arg, &p->openingngb, "openingngb", key, SPACK,
+                              NULL, 0);
       p->up.openingngbset=1;
       break;
     case 'u':
-      floatl0(arg, &p->sigclipmultip, "sigclipmultip", key, SPACK,
-              NULL, 0);
+      gal_checkset_float_l_0(arg, &p->sigclipmultip, "sigclipmultip", key,
+                             SPACK, NULL, 0);
       p->up.sigclipmultipset=1;
       break;
     case 'r':
-      floatl0s1(arg, &p->sigcliptolerance, "sigcliptolerance", key, SPACK,
-              NULL, 0);
+      gal_checkset_float_l_0_s_1(arg, &p->sigcliptolerance,
+                                 "sigcliptolerance", key, SPACK, NULL, 0);
       p->up.sigcliptoleranceset=1;
       break;
     case 'R':
-      anyfloat(arg, &p->dthresh, "dthresh", key, SPACK, NULL, 0);
+      gal_checkset_any_float(arg, &p->dthresh, "dthresh", key, SPACK,
+                             NULL, 0);
       p->up.dthreshset=1;
       break;
     case 'i':
-      sizetlzero(arg, &p->detsnminarea, "detsnminarea", key, SPACK, NULL, 0);
+      gal_checkset_sizet_l_zero(arg, &p->detsnminarea, "detsnminarea", key,
+                                SPACK, NULL, 0);
       p->up.detsnminareaset=1;
       break;
     case 510:
-      sizetelzero(arg, &p->detsnhistnbins, "detsnhistnbins", key,
-                  SPACK, NULL, 0);
+      gal_checkset_sizet_el_zero(arg, &p->detsnhistnbins, "detsnhistnbins",
+                                 key, SPACK, NULL, 0);
       p->up.detsnhistnbinsset=1;
       break;
     case 'c':
-      floatl0s1(arg, &p->detquant, "detquant", key, SPACK, NULL, 0);
+      gal_checkset_float_l_0_s_1(arg, &p->detquant, "detquant", key, SPACK,
+                                 NULL, 0);
       p->up.detquantset=1;
       break;
     case 'I':
-      sizetelzero(arg, &p->dilate, "dilate", key, SPACK, NULL, 0);
+      gal_checkset_sizet_el_zero(arg, &p->dilate, "dilate", key, SPACK,
+                                 NULL, 0);
       p->up.dilateset=1;
       break;
     case 505:
@@ -729,55 +735,51 @@ parse_opt(int key, char *arg, struct argp_state *state)
     case 509:
       p->detectionskyname="a";
       break;
-    case 503:
-      p->detectionsnname="a";
-      break;
     case 512:
       p->skyname="a";
       break;
-    case 519:
+    case 518:
       p->maskdetname="a";
       break;
 
 
     /* Segmentation: */
     case 'm':
-      sizetlzero(arg, &p->segsnminarea, "segsnminarea", key, SPACK, NULL, 0);
+      gal_checkset_sizet_l_zero(arg, &p->segsnminarea, "segsnminarea", key,
+                                SPACK, NULL, 0);
       p->up.segsnminareaset=1;
       break;
     case 'g':
-      floatl0s1(arg, &p->segquant, "segquant", key, SPACK, NULL, 0);
+      gal_checkset_float_l_0_s_1(arg, &p->segquant, "segquant", key, SPACK,
+                                 NULL, 0);
       p->up.segquantset=1;
       break;
     case 'v':
       p->keepmaxnearriver=1;
       break;
     case 'G':
-      anyfloat(arg, &p->gthresh, "gthresh", key, SPACK, NULL, 0);
+      gal_checkset_any_float(arg, &p->gthresh, "gthresh", key, SPACK, NULL, 0);
       p->up.gthreshset=1;
       break;
     case 'y':
-      sizetlzero(arg, &p->minriverlength, "minriverlength",
-                 key, SPACK, NULL, 0);
+      gal_checkset_sizet_l_zero(arg, &p->minriverlength, "minriverlength",
+                                key, SPACK, NULL, 0);
       p->up.minriverlengthset=1;
       break;
     case 'O':
-      floatl0(arg, &p->objbordersn, "objbordersn", key, SPACK,
-              NULL, 0);
+      gal_checkset_float_l_0(arg, &p->objbordersn, "objbordersn", key, SPACK,
+                             NULL, 0);
       p->up.objbordersnset=1;
       break;
 
     case 514:
-      sizetelzero(arg, &p->segsnhistnbins, "segsnhistnbins", key,
-                  SPACK, NULL, 0);
-      p->up.segsnhistnbinsset=1;
+      gal_checkset_sizet_el_zero(arg, &p->clumpsnhistnbins,
+                                 "clumpsnhistnbins", key, SPACK, NULL, 0);
+      p->up.clumpsnhistnbinsset=1;
       break;
     case 513:
       p->segmentationname="a";
       break;
-    case 515:
-      p->clumpsnname="a";
-      break;
 
 
     /* Operating modes: */
@@ -787,15 +789,15 @@ parse_opt(int key, char *arg, struct argp_state *state)
     case ARGP_KEY_ARG:
 
       /* See what type of input value it is and put it in. */
-      if( nameisfits(arg) )
+      if( gal_fitsarray_name_is_fits(arg) )
         {
           if(p->up.inputname)
-            argp_error(state, "Only one input image should be given.");
+            argp_error(state, "only one input image should be given");
           else
             p->up.inputname=arg;
 	}
       else
-        argp_error(state, "%s is not a valid file type.", arg);
+        argp_error(state, "%s is not a valid file type", arg);
       break;
 
 
@@ -808,9 +810,9 @@ parse_opt(int key, char *arg, struct argp_state *state)
 	 && p->cp.printparams==0)
 	{
 	  if(state->arg_num==0)
-	    argp_error(state, "No argument given!");
+	    argp_error(state, "no argument given");
 	  if(p->up.inputname==NULL)
-	    argp_error(state, "No input FITS image(s) provided!");
+	    argp_error(state, "no input FITS image(s) provided");
 	}
       break;
 
