@@ -18,7 +18,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with gnuastro. If not, see <http://www.gnu.org/licenses/>.
+along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 **********************************************************************/
 #include <config.h>
 
@@ -29,20 +29,20 @@ along with gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <string.h>
 #include <stdlib.h>
 
-#include "timing.h"
-#include "checkset.h"
-#include "forqsort.h"
-#include "neighbors.h"
-#include "arraymanip.h"
-#include "linkedlist.h"
-#include "statistics.h"
-#include "fitsarrayvv.h"
+#include <gnuastro/qsort.h>
+#include <gnuastro/timing.h>
+#include <gnuastro/checkset.h>
+#include <gnuastro/neighbors.h>
+#include <gnuastro/arraymanip.h>
+#include <gnuastro/linkedlist.h>
+#include <gnuastro/statistics.h>
+#include <gnuastro/fitsarrayvv.h>
 
 #include "main.h"
 
 #include "clumps.h"
 #include "binary.h"
-#include "detection.h"
+#include "thresh.h"
 
 
 
@@ -77,21 +77,21 @@ oversegment(struct clumpsthreadparams *ctp)
   struct noisechiselparams *p=ctp->p;
 
   /* pix is not actually used its self, however, the pointer to it
-     will be extensively used (ind) for the macro FILL_NGB_8_REGION.
+     will be extensively used (ind) for the macro GAL_NEIGHBORS_FILL_8_REGION.
      This macro works on the pointer to the index, not the index its
      self. `pix` is filled with different index values, so the pointer
      to it doesn't change. */
   size_t pix;
 
   float *arr=p->conv;
-  struct sll *Q=NULL, *cleanup=NULL;
+  struct gal_linkedlist_sll *Q=NULL, *cleanup=NULL;
   long n1, rlab, nlab, curlab=1, *clab=p->clab;
   size_t x0=ctp->x0, y0=ctp->y0, x1=ctp->x1, y1=ctp->y1;
   size_t *n, *nf, *indf, *pind, *ind=&pix, is1=p->lmp.s1;
   size_t ng, *rn, *rnf, numngb, ngb[8], *relngb=p->relngb;
 
   /* Sort the indexs based on the flux within them. */
-  qsort(ctp->inds, ctp->area, sizeof(size_t), indexfloatdecreasing);
+  qsort(ctp->inds, ctp->area, sizeof(size_t), gal_qsort_index_float_decreasing);
 
   /* Initialize the region you want to over-segment. */
   indf=(pind=ctp->inds)+ctp->area;
@@ -164,10 +164,10 @@ oversegment(struct clumpsthreadparams *ctp)
           {
             n1=0;
             if(Q!=NULL || cleanup!=NULL)
-              error(EXIT_FAILURE, 0, "A bug! Please contact us at %s so we "
+              error(EXIT_FAILURE, 0, "a bug! Please contact us at %s so we "
                     "can fix this problem. In oversegment (clumps.c) Q and "
                     "cleanup should be NULL but while checking the equal "
-                    "flux regions they aren't.", PACKAGE_BUGREPORT);
+                    "flux regions they aren't", PACKAGE_BUGREPORT);
             add_to_sll(&Q, *pind);
             add_to_sll(&cleanup, *pind);
             clab[*pind]=SEGMENTTMPCHECK;
@@ -182,7 +182,7 @@ oversegment(struct clumpsthreadparams *ctp)
                 /* Check the vicinity of this pixel that was just
                    popped to see if it can find any already labeled
                    neighbour or not. */
-                FILL_NGB_8_REGION;
+                GAL_NEIGHBORS_FILL_8_REGION;
 
                 /* If the pixel is on the side of the region, set it
                    as a river, no more need to look around it. */
@@ -235,46 +235,14 @@ oversegment(struct clumpsthreadparams *ctp)
             /* Set the label that is to be given to this equal flux
                region. If n1 was set to any value, then that label
                should be used for the whole region. Otherwise, this is
-               a new label.
-
-               About the topinds checks:
-               =========================
-
-               The knowing the top index of a profile is very
-               important in order to be able to successfully remove
-               false detections. However, our convention of setting
-               the masked pixels as the maximum floating point value
-               is going to make some troubles in this regard.
-
-               Masked pixels will always be checked first for each
-               clump. The moment a new label is definied (curlab++),
-               we are on the top index of a profile. If it is a NaN
-               pixel (note that the NaN pixels do not change after
-               convolution), then we can't assign it to the top flux
-               of that profile. So we mark this by setting the value
-               to the topinds for this profile to NOTOPIND which is an
-               impossible index. later on, for every other pixel, we
-               check to see if topinds has been set for this clump or
-               not. If it hasn't, then since the pixels are ordered by
-               flux, this index is the top index.
-            */
+               a new label, see the case for a non-flat region. */
             if(n1)
-              {
-                rlab = n1;
-                if(ctp->topinds && ctp->topinds[rlab]==NOTOPIND
-                   && n1>0 && !isnan(p->img[*pind]))
-                  ctp->topinds[rlab]=*pind;
-              }
+              rlab = n1;
             else
               {
                 rlab = curlab++;
                 if( ctp->topinds)
-                  {
-                    if( isnan(p->img[*pind]) )
-                      ctp->topinds[rlab]=NOTOPIND;
-                    else
-                      ctp->topinds[rlab]=*pind;
-                  }
+                  ctp->topinds[rlab]=*pind;
               }
 
             /* Give the same label to the whole connected equal flux
@@ -303,10 +271,10 @@ oversegment(struct clumpsthreadparams *ctp)
                 /* Set the first neighbour's label to zero. */
                 n1=0;
                 if(Q!=NULL || cleanup!=NULL)
-                  error(EXIT_FAILURE, 0, "A bug! Please contact us at %s "
+                  error(EXIT_FAILURE, 0, "a bug! Please contact us at %s "
                         "so we can fix this problem. In oversegment "
                         "(clumps.c) Q and cleanup should be NULL but while "
-                        "checking the equal flux regions they aren't.",
+                        "checking the equal flux regions they aren't",
                         PACKAGE_BUGREPORT);
 
                 /* Go over all the 8 neighbors of this pixel and see
@@ -346,26 +314,16 @@ oversegment(struct clumpsthreadparams *ctp)
                one of its neighbors. If n1 equals zero, then it is a
                new peak, and a new label should be created.  But if
                n1!=0, it is either a river pixel (has more than one
-               labeled neighbor) or all its neighbors have the same
-               label. */
+               labeled neighbor and has been set to SEGMENTRIVER) or
+               all its neighbors have the same label. In both such
+               cases, rlab should be set to n1.*/
             if(n1)
-              {
-                rlab = n1;
-                if(ctp->topinds && ctp->topinds[rlab]==NOTOPIND
-                   && n1>0 && !isnan(p->img[*pind]))
-                  ctp->topinds[rlab]=*pind;
-              }
+              rlab = n1;
             else
               {
                 rlab = curlab++;
                 if( ctp->topinds )
-                  {
-                    if( isnan(p->img[*pind]) )
-                      ctp->topinds[rlab]=NOTOPIND;
-                    else
-                      ctp->topinds[rlab]=*pind;
-                  }
-
+                  ctp->topinds[rlab]=*pind;
               }
 
             /* Put the found label in the pixel. */
@@ -409,10 +367,18 @@ oversegment(struct clumpsthreadparams *ctp)
 void
 growclumps(struct clumpsthreadparams *ctp, int withrivers)
 {
-  long n1, *olab=ctp->p->olab;
+  struct noisechiselparams *p=ctp->p;
+
+  long n1, *olab=p->olab;
   size_t *ind, *indf, thisround, numngb;
   size_t *n, *nf, numblanks=ctp->numblanks;
-  size_t ngb[8], is0=ctp->p->lmp.s0, is1=ctp->p->lmp.s1;
+  size_t ngb[8], is0=p->lmp.s0, is1=p->lmp.s1;
+
+  /* It might happen that the growth threshold is larger than any of
+     the non-clump pixels. So, if the number of blanks is zero, just
+     leave this function. */
+  if(numblanks==0) return;
+
 
   /* The basic idea is this: after growing, not all the blank pixels
      are necessarily filled, for example the pixels might belong to
@@ -447,7 +413,7 @@ growclumps(struct clumpsthreadparams *ctp, int withrivers)
 	  n1=0;
 
 	  /* Check the 4 connected neighbors of the pixel: */
-          FILL_NGB_4_ALLIMG;
+          GAL_NEIGHBORS_FILL_4_ALLIMG;
 	  nf=(n=ngb)+numngb;
 	  do
             if( olab[*n]>0 )	             /* This neighbor is labeled. */
@@ -512,9 +478,9 @@ growclumps(struct clumpsthreadparams *ctp, int withrivers)
 
    The output is an array of size numseg*INFOTABCOLS. INFOTABCOLS=4. The
    columns are:
-   0: Average signal in clump.
+   0: Average signal (flux) in clump.
    1: Number of pixels in clump.
-   2: Average signal around clump.
+   2: Average signal (flux) around clump.
    3: Number of pixels around clump.
    4: Standard deviation on flux weighted center of clump.
 */
@@ -522,7 +488,7 @@ void
 getclumpinfo(struct clumpsthreadparams *ctp, double **outclumpinfo)
 {
   struct noisechiselparams *p=ctp->p;
-  struct meshparams *smp=&p->smp;
+  struct gal_mesh_params *smp=&p->smp;
 
   double *xys, *clumpinfo;
   float *img=p->img, *smpstd=smp->garray2;
@@ -533,31 +499,36 @@ getclumpinfo(struct clumpsthreadparams *ctp, double **outclumpinfo)
 
   /* Just make sure that the box size is not only around one pixel! */
   if(x1-x0<=1 || y1-y0<=1)
-    error(EXIT_FAILURE, 0, "A bug! Please contact us at %s so we can find "
+    error(EXIT_FAILURE, 0, "a bug! Please contact us at %s so we can find "
           "and fix the problem in clumpinfo (clumps.c). For some reason, "
-          "the specified input region is %lu by %lu wide.",
+          "the specified input region is %lu by %lu wide",
           PACKAGE_BUGREPORT, y1-y0, x1-x0);
 
   /* Allocate the clump information array. */
   errno=0;
   clumpinfo=calloc(ctp->numclumps*INFOTABCOLS, sizeof *clumpinfo);
   if(clumpinfo==NULL)
-    error(EXIT_FAILURE, errno, "%lu bytes for clumpinfo (clumps.c)",
-          ctp->numclumps*INFOTABCOLS*sizeof *clumpinfo);
+    error(EXIT_FAILURE, errno, "%lu bytes for clumpinfo in getclumpinfo "
+          "(clumps.c)", ctp->numclumps*INFOTABCOLS*sizeof *clumpinfo);
   *outclumpinfo=clumpinfo;
 
-  /* If the image is sky subtracted or if we are working on the
-     detections (not noise), we will need the light weighted center of
-     each clump. In the former, for finding the error in measuring the
-     sky (through the smp->garray2) which is done in this same
-     function. In the latter case, in order to find the signal to
-     noise threshold on lmp->garray1. */
-  if(p->skysubtracted || p->b0f1)
+  /* If the image is sky subtracted, we will need the light weighted
+     center of each clump for finding the error in measuring the sky
+     (through the smp->garray2).
+
+     Since we are also segmenting the undetected regions, negative
+     pixel values are common and they will mess up the flux weighted
+     center (since all the weights have to be positive). So the xys
+     array has three columns: the first is the total flux calculated
+     from the positive pixels. The second is the x axis center and the
+     third is the y axis center.
+  */
+  if(p->skysubtracted)
     {
-      errno=0; xys=ctp->xys=calloc(2*ctp->numclumps, sizeof *xys);
+      errno=0; xys=calloc(3*ctp->numclumps, sizeof *xys);
       if(xys==NULL)
         error(EXIT_FAILURE, errno, "%lu bytes for xys (clumps.c)",
-              2*ctp->numclumps*sizeof *xys);
+              3*ctp->numclumps*sizeof *xys);
     }
   else xys=NULL; /* To check if it is needed below. */
 
@@ -574,8 +545,8 @@ getclumpinfo(struct clumpsthreadparams *ctp, double **outclumpinfo)
                the neighbors within a region. Otherwise (when working
                on the detections) we want the neighbors on the full
                image.*/
-            if(p->b0f1) {FILL_NGB_8_ALLIMG}
-            else        {FILL_NGB_8_REGION}
+            if(p->b0f1) {GAL_NEIGHBORS_FILL_8_ALLIMG}
+            else        {GAL_NEIGHBORS_FILL_8_REGION}
 
 
             /* We are on a river pixel. So its value has to be added
@@ -615,10 +586,11 @@ getclumpinfo(struct clumpsthreadparams *ctp, double **outclumpinfo)
             lab=clab[*ind];
             ++clumpinfo[ lab * INFOTABCOLS + 1 ];
             clumpinfo[ lab * INFOTABCOLS ] += img[*ind];
-            if(xys)
+            if(p->skysubtracted && img[*ind]>0.0f)
               {
-                xys[ 2 * lab     ] += (*ind/is1) * img[*ind];
-                xys[ 2 * lab + 1 ] += (*ind%is1) * img[*ind];
+                xys[ 3 * lab     ] += img[*ind];
+                xys[ 3 * lab + 1 ] += (*ind/is1) * img[*ind];
+                xys[ 3 * lab + 2 ] += (*ind%is1) * img[*ind];
               }
 
           }
@@ -639,50 +611,41 @@ getclumpinfo(struct clumpsthreadparams *ctp, double **outclumpinfo)
              subtracted. If it wasn't, then we are not subtracting the
              sky to worry about its error! The error in a pixel flux
              measurement is simply its square root. */
-          if(xys)
+          if(p->skysubtracted)
             {
-              /* Especially for noise, it might happen that the total
-                 flux or any of the positions multiplied by flux
-                 becomes negative. If left unchecked, it will result
-                 in checking the wrong pointers, so we need to check
-                 them at this point. */
-              if(clumpinfo[row]<0.0f || xys[2*lab]<0.0f
-                 || xys[2*lab+1]<0.0f)
+              /* Especially for noise, it might happen that no pixel
+                 within the clump was positive! If so, then the center
+                 cannot be calculated and the clump must not be
+                 used, so just set its area to zero. */
+              if(xys[3*lab]==0.0f)
                 {
                   clumpinfo[row+1]=0;
-                  xys[2*lab]=xys[2*lab+1]=NAN;
                   continue;
                 }
               else
                 {
-                  /* It is important that xys are corrected
-                     (divided by the total flux here). Because they
-                     are used later when we don't have access to
-                     clumpinfo any more. */
-                  xys[2*lab  ] /= clumpinfo[row];
-                  xys[2*lab+1] /= clumpinfo[row];
+                  xys[3*lab+1] /= xys[3*lab];
+                  xys[3*lab+2] /= xys[3*lab];
                   if(p->skysubtracted)
-                    clumpinfo[row+4]=
-                      smpstd[imgxytomeshid(smp, xys[2*lab],
-                                           xys[2*lab+1])];
+                      clumpinfo[row+4]=
+                        smpstd[gal_mesh_img_xy_to_mesh_id(smp,xys[3*lab+1],
+                                                          xys[3*lab+2])];
 
                   /* For a check:
                   printf("%lu: (%lu, %lu) --> %f\n", lab,
-                         (size_t)(xys[2*lab+1] / clumpinfo[row]) +1,
-                         (size_t)(xys[2*lab] / clumpinfo[row]) +1,
+                         (size_t)(xys[3*lab+2] / clumpinfo[row]) +1,
+                         (size_t)(xys[3*lab+1] / clumpinfo[row]) +1,
                          clumpinfo[row+4]);
                   */
                 }
             }
 
-          /* Convert sum to average. IMPORTANT NOTE: This has to be
-             done after the step above, because there, we need the
-             total flux to find the flux weighted center positions. */
+          /* Convert sum to average. We are doing this after so if a
+             clump should be ignored, control doesn't get to this
+             point. */
           clumpinfo[row  ] /= clumpinfo[row+1];
           clumpinfo[row+2] /= clumpinfo[row+3];
         }
-      else
-        if(xys) xys[2*lab]=xys[2*lab+1]=NAN;
     }
 
   /* To check a specific detection (p->b0f1==1) or a noise mesh grid
@@ -697,7 +660,7 @@ getclumpinfo(struct clumpsthreadparams *ctp, double **outclumpinfo)
 
   /* If we are on the noise clumps, then xys is not needed any more.
      So if it was allocated, fre the space. */
-  if(p->b0f1==0 && xys) free(xys);
+  if(p->skysubtracted) free(xys);
 }
 
 
@@ -772,12 +735,14 @@ clumpsntable(struct clumpsthreadparams *ctp, float **sntable)
 
   /* To check a specific detection (p->b0f1==1) or a noise mesh grid
      (p->b0f1==0):
-  if(p->b0f1==1 && ctp->thislabel==1)
-    for(i=1;i<ctp->numclumps;++i)
-      printf("%lu: %-10.3f %-5.0f %-10.3f --> %-10.3f (%-6.0f, %-6.0f)\n", i,
-             clumpinfo[i*INFOTABCOLS], clumpinfo[i*INFOTABCOLS+1],
-             clumpinfo[i*INFOTABCOLS+2], sntab[i],
-             ctp->xys[i*2], ctp->xys[i*2+1]);
+  if(p->b0f1==1 && ctp->thislabel==151)
+    {
+      for(i=1;i<ctp->numclumps;++i)
+        printf("%lu: %-10.3g %-5.0f %-10.3g(%-5.0f) --> %-10.3g\n", i,
+               clumpinfo[i*INFOTABCOLS], clumpinfo[i*INFOTABCOLS+1],
+               clumpinfo[i*INFOTABCOLS+2], clumpinfo[i*INFOTABCOLS+3],
+               sntab[i]);
+    }
   */
   /* Clean up */
   free(clumpinfo);
@@ -804,21 +769,19 @@ clumpsntable(struct clumpsthreadparams *ctp, float **sntable)
 /*************           S/N threshold           ******************/
 /******************************************************************/
 void *
-clumpsnthreshonmesh(void *inparams)
+clumpsntableonmesh(void *inparams)
 {
   struct clumpsthreadparams ctp;
-  struct meshthreadparams *mtp=(struct meshthreadparams *)inparams;
-  struct meshparams *mp=mtp->mp;
+  struct gal_mesh_thread_params *mtp=(struct gal_mesh_thread_params *)inparams;
+  struct gal_mesh_params *mp=mtp->mp;
   struct noisechiselparams *p=(struct noisechiselparams *)mp->params;
 
   size_t *mponeforall=mp->oneforall;
 
+  int anyblank;
   float *sntable;
-  char cline[1000];
-  long *clab=p->clab;
   size_t s0, s1, nf, ind, startind, is1=mp->s1;
   size_t i, *indexs=&mp->indexs[mtp->id*mp->thrdcols];
-  char suffix[50], *histname, *segmentationname=p->segmentationname;
 
   /* Set the necessary pointers for the clumpsthreadparams
      structure. */
@@ -827,7 +790,7 @@ clumpsnthreshonmesh(void *inparams)
   ctp.inds=&mponeforall[mtp->id*mp->maxs0*mp->maxs1];
 
   /* Go over all the meshs that are assigned to this thread. */
-  for(i=0;indexs[i]!=NONTHRDINDEX;++i)
+  for(i=0;indexs[i]!=GAL_THREADS_NON_THRD_INDEX;++i)
     {
       /* Set index of this mesh: */
       ind=ctp.thislabel=indexs[i];
@@ -842,11 +805,12 @@ clumpsnthreshonmesh(void *inparams)
 
       /* Check to see if we have enough blank area for getting the
 	 background noise statistics. */
-      count_f_b_onregion(p->byt, startind, s0, s1, is1, &nf, &ctp.area);
+      count_f_b_onregion(p->byt, startind, s0, s1, is1, &nf, &ctp.area,
+                         &anyblank);
       if( (float)ctp.area < (float)(s0*s1)*p->minbfrac )
         {
-          if(segmentationname)
-            longinitonregion(clab, 0, startind, s0, s1, is1);
+          p->numclumpsarr[ind]=0;
+          p->sntablearr[ind]=NULL;
           continue;
         }
 
@@ -861,70 +825,15 @@ clumpsnthreshonmesh(void *inparams)
       /* Do the over-segmentation and put the number of clumps in
          ctp.numclumps */
       oversegment(&ctp);
-      if(ctp.numclumps<p->minnumfalse)
-        {
-          if(segmentationname)
-            longinitonregion(clab, 0, startind, s0, s1, is1);
-          continue;
-        }
 
 
       /* Find the signal to noise of all the clumps. */
       clumpsntable(&ctp, &sntable);
 
 
-      /* Continue if the number of clumps is enough: */
-      if(ctp.numclumps<p->minnumfalse)
-        {
-          free(sntable);
-          if(segmentationname)
-            longinitonregion(clab, 0, startind, s0, s1, is1);
-          continue;
-        }
-
-
-      /* Sort the Signal to noise ratio values and remove outliers. */
-      qsort(sntable, ctp.numclumps, sizeof *sntable, floatincreasing);
-      removeoutliers_flatcdf(sntable, &ctp.numclumps);
-      if(ctp.numclumps<p->minnumfalse)
-        {
-          free(sntable);
-          if(segmentationname)
-            longinitonregion(clab, 0, startind, s0, s1, is1);
-          continue;
-        }
-
-
-      /* Put the signal to noise quantile into the mesh grid. */
-      mp->garray1[ind]=sntable[indexfromquantile(ctp.numclumps, p->segquant)];
-
-
-      /* If the user has asked for it, make the histogram of the
-         S/N distribution. */
-      if(p->segsnhistnbins)
-        {
-          /* histname has to be set to NULL so automaticoutput can
-             safey free it. */
-          histname=NULL;
-          sprintf(suffix, "_%lu_segsn.txt", ind);
-          sprintf(cline, "# %s\n# %s started on %s"
-                  "# Input: %s (hdu: %s)\n"
-                  "# Histogram for S/N distribution of false "
-                  "clumps.\n"
-                  "# On large mesh id %lu.\n"
-                  "# The %.3f quantile has a value of %.4f on "
-                  "this bin.", SPACK_STRING, SPACK_NAME,
-                  ctime(&p->rawtime), p->up.inputname, p->cp.hdu,
-                  ind, p->segquant, mp->garray1[ind]);
-          automaticoutput(p->up.inputname, suffix, p->cp.removedirinfo,
-                          p->cp.dontdelete, &histname);
-          savehist(sntable, ctp.numclumps, p->segsnhistnbins,
-                   histname, cline);
-          free(histname);
-        }
-
-      /* Cleanup: */
-      free(sntable);
+      /* Keep the relevant information for this mesh: */
+      p->sntablearr[ind]=sntable;
+      p->numclumpsarr[ind]=ctp.numclumps;
     }
 
   /* Free any allocated space and if multiple threads were used, wait
@@ -938,30 +847,79 @@ clumpsnthreshonmesh(void *inparams)
 
 
 
+/* The job of this function is to find the best signal to noise value
+   to use as a threshold to detect real clumps.
+
+   Each thread will find the useful signal to noise values for the
+   meshs that have been assigned to it. It will then store the pointer
+   to the S/N table into the sntablearr array (with the size of the
+   number of meshs). If no clumps could be found in a mesh, then
+   sntablearr[i]=NULL. Otherwise, it points to an array of the useful
+   S/N values in that clump. Note that we don't care about the order
+   of S/N values any more! There is also an accompanying array to keep
+   the number of elements in the final S/N array of each mesh:
+   numclumpsarr.
+
+   Using these two arrays, after all the threads are finished, we can
+   concatenate all the S/N values into one array and send it to the
+   main findsnthresh function in thresh.c.
+*/
 void
-clumpsngrid(struct noisechiselparams *p)
+findclumpsn(struct noisechiselparams *p)
 {
-  float snave;
-  char report[VERBMSGLENGTHS2_V];
-  struct meshparams *lmp=&p->lmp;
+  float *sntable;
+  struct gal_mesh_params *lmp=&p->lmp;
+  size_t i, j, numclumps=0, nmeshi=lmp->nmeshi;
+
 
   /* Set the convolved image as the basis for sorting the indexs and
      finding clumps. */
-  forqsortindexarr=p->conv;
+  gal_qsort_index_arr=p->conv;
+
+
+  /* Allocate the two arrays to keep the number and values of the S/Ns
+     in each mesh. */
+  errno=0; p->numclumpsarr=malloc(nmeshi*sizeof*p->numclumpsarr);
+  if(p->numclumpsarr==NULL)
+    error(EXIT_FAILURE, errno, "%lu bytes for p->numclumpsarr in "
+          "findclumpsn (clumps.c)", nmeshi*sizeof*p->numclumpsarr);
+  errno=0; p->sntablearr=malloc(nmeshi*sizeof*p->sntablearr);
+  if(p->sntablearr==NULL)
+    error(EXIT_FAILURE, errno, "%lu bytes for p->sntablearr in "
+          "findclumpsn (clumps.c)", nmeshi*sizeof*p->sntablearr);
+
 
   /* Find the clump Signal to noise ratio on successful meshs then on
      all the meshs. findsnthreshongrid is in detection.c. */
-  operateonmesh(lmp, clumpsnthreshonmesh, sizeof(size_t), 0, 1);
-  findsnthreshongrid(&p->lmp, p->clumpsnname, "Interpolating the "
-                     "CLUMP Signal to noise ratio threshold", p->wcs);
-  if(p->cp.verb)
-    {
-      snave=floataverage(lmp->garray1, lmp->nmeshi);
-      sprintf(report, "Clump S/N limit found (Average: %.3f).",
-              snave);
-      reporttiming(NULL, report, 2);
-    }
+  gal_mesh_operate_on_mesh(lmp, clumpsntableonmesh, sizeof(size_t), 0, 0);
 
+
+  /* Find the total number of useful clumps and allocate sntable.  */
+  for(i=0;i<nmeshi;++i) numclumps+=p->numclumpsarr[i];
+  errno=0; sntable=malloc(numclumps*sizeof*sntable);
+  if(sntable==NULL)
+    error(EXIT_FAILURE, errno, "%lu bytes for sntable in "
+          "findclumpsn (clumps.c)", numclumps*sizeof*sntable);
+
+
+  /* Fill in the sntable array: */
+  numclumps=0;
+  for(i=0;i<nmeshi;++i)
+    for(j=0;j<p->numclumpsarr[i];++j)
+      sntable[numclumps++]=p->sntablearr[i][j];
+
+
+  /* Set the clump signal to noise value: */
+  snthresh(p, sntable, numclumps, 1);
+
+
+  /* Clean up. In clumpsntableonmesh we put an allocated array in each
+     member of p->sntablearr[i]. */
+  for(i=0;i<nmeshi;++i)
+    free(p->sntablearr[i]);
+  free(p->numclumpsarr);
+  free(p->sntablearr);
+  free(sntable);
 }
 
 
@@ -992,10 +950,8 @@ clumpsngrid(struct noisechiselparams *p)
 void
 removefalseclumps(struct clumpsthreadparams *ctp, float *sntable)
 {
-  struct meshparams *lmp=&ctp->p->lmp;
+  struct gal_mesh_params *lmp=&ctp->p->lmp;
 
-  double *xys=ctp->xys;
-  float *snonmesh=lmp->garray1;
   long *newlabs, *clab=ctp->p->clab;
   size_t *n, *nf, is0=lmp->s0, is1=lmp->s1;
   size_t i, curlab=1, *ind, *indf, ngb[8], numngb;
@@ -1007,14 +963,13 @@ removefalseclumps(struct clumpsthreadparams *ctp, float *sntable)
           "removefalsedetections (clumps.c)", ctp->numclumps*sizeof *newlabs);
 
   /* We want the removed regions to become SEGMENTINIT. */
-  longinit(newlabs, ctp->numclumps, SEGMENTINIT);
+  gal_arraymanip_long_init(newlabs, ctp->numclumps, SEGMENTINIT);
 
   /* Set the new labels: */
   if(ctp->p->keepmaxnearriver)
     {       /* `{' because We don't want to confuse the `else' below. */
       for(i=1;i<ctp->numclumps;++i)
-        if( !isnan(xys[2*i]) && sntable[i] >
-            snonmesh[ imgxytomeshid(lmp, xys[2*i], xys[2*i+1]) ] )
+        if( sntable[i] > ctp->p->clumpsn )
           newlabs[i]=curlab++;
     }
   else
@@ -1024,21 +979,20 @@ removefalseclumps(struct clumpsthreadparams *ctp, float *sntable)
           /* Check to see if the brightest pixel in this clump is
              touching a river or not. */
           ind=&ctp->topinds[i];
-          FILL_NGB_8_ALLIMG;
+          GAL_NEIGHBORS_FILL_8_ALLIMG;
           nf=(n=ngb)+numngb;
           do if(clab[*n++]==SEGMENTRIVER) break; while(n<nf);
 
           /* If the brightest pixel of this clump was not touching a
              river and its Signal to noise ratio is larger than the
              threshold, then give it a new label.*/
-          if(n==nf && !isnan(xys[2*i]) && sntable[i] >
-             snonmesh[ imgxytomeshid(lmp, xys[2*i], xys[2*i+1]) ] )
+          if( n==nf  &&  sntable[i] > ctp->p->clumpsn )
             newlabs[i]=curlab++;
         }
   ctp->numclumps=curlab;
 
   /* Change the values of the false clumps. Note that the labels are
-     either SEGMENTRIVER or a label */
+     either SEGMENTRIVER or a label. */
   indf = (ind=ctp->inds) + ctp->area;
   do
     {
