@@ -18,7 +18,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with gnuastro. If not, see <http://www.gnu.org/licenses/>.
+along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 **********************************************************************/
 #include <config.h>
 
@@ -28,9 +28,9 @@ along with gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <string.h>
 #include <stdlib.h>
 
-#include "forqsort.h"
-#include "neighbors.h"
-#include "fitsarrayvv.h"
+#include <gnuastro/qsort.h>
+#include <gnuastro/neighbors.h>
+#include <gnuastro/fitsarrayvv.h>
 
 #include "main.h"
 
@@ -59,11 +59,11 @@ along with gnuastro. If not, see <http://www.gnu.org/licenses/>.
 void
 prepfirstgrowth(struct clumpsthreadparams *ctp)
 {
-  struct meshparams *smp=&ctp->p->smp;
+  struct gal_mesh_params *smp=&ctp->p->smp;
 
   size_t *ind, *indf, is1=smp->s1;
-  double x=0.0f, y=0.0f, fluxsum=0.0f;
   float growlimit, *imgss=ctp->p->imgss;
+  double x=0.0f, y=0.0f, brightness=0.0f;
   long *olab=ctp->p->olab, *clab=ctp->p->clab;
 
   /* Try to find the flux weighted center (only on the pixels with
@@ -72,7 +72,7 @@ prepfirstgrowth(struct clumpsthreadparams *ctp)
   do
     if(imgss[*ind]>0.0f)
       {
-        fluxsum+=imgss[*ind];
+        brightness+=imgss[*ind];
         x+=(*ind/is1)*imgss[*ind];
         y+=(*ind%is1)*imgss[*ind];
       }
@@ -80,7 +80,7 @@ prepfirstgrowth(struct clumpsthreadparams *ctp)
 
   /* Calculate the center, if no pixels were positive, use the
      geometric center (irrespective of flux). */
-  if(fluxsum==0.0f)
+  if(brightness==0.0f)
     {
       ind=ctp->inds;
       do { x+=*ind/is1; y+=*ind%is1; } while(++ind<indf);
@@ -89,13 +89,13 @@ prepfirstgrowth(struct clumpsthreadparams *ctp)
     }
   else
     {
-      x/=fluxsum;
-      y/=fluxsum;
+      x/=brightness;
+      y/=brightness;
     }
 
   /* First find the standard deviation on this detection, then use
      it to calculate the growth threshold. */
-  ctp->std=smp->garray2[imgxytomeshid(smp, x, y)];
+  ctp->std=smp->garray2[gal_mesh_img_xy_to_mesh_id(smp, x, y)];
   growlimit=ctp->p->gthresh * ctp->std;
 
   /* Allocate and fill the array for the blank pixels and reset the
@@ -106,7 +106,7 @@ prepfirstgrowth(struct clumpsthreadparams *ctp)
   ctp->blankinds=malloc(ctp->area*sizeof *ctp->blankinds);
   if(ctp->blankinds==NULL)
     error(EXIT_FAILURE, errno, "%lu bytes for ctp->blankpixels in "
-          "growclumps (clumps.c)", ctp->area*sizeof *ctp->blankinds);
+          "prepfirstgrowth (clumps.c)", ctp->area*sizeof *ctp->blankinds);
   ctp->numblanks=0;
   indf=(ind=ctp->inds)+ctp->area;
   do
@@ -190,7 +190,7 @@ adjacencymatrixs(struct clumpsthreadparams *ctp,
 	memset(wngb, 0, sizeof(wngb));
 
 	/* Find which grown clumps this river pixel touches.      */
-	FILL_NGB_8_ALLIMG;
+	GAL_NEIGHBORS_FILL_8_ALLIMG;
 	nf=(n=ngb)+numngb;
 	do
 	  if( olab[*n]>0 )
@@ -222,8 +222,8 @@ adjacencymatrixs(struct clumpsthreadparams *ctp,
               for(j=0;j<ii;++j)
                 if(i!=j)
                   {
-                    nums[ wngb[i]* numclumps + wngb[j] ]++;
-                    sums[ wngb[i]* numclumps + wngb[j] ]+=rpave;
+                    nums[ wngb[i] * numclumps + wngb[j] ]++;
+                    sums[ wngb[i] * numclumps + wngb[j] ]+=rpave;
                   }
           }
       }
@@ -277,12 +277,13 @@ adjacencymatrixs(struct clumpsthreadparams *ctp,
 
 
 /* The true clumps were expanded. Here the job is to create two
-   adjacency matrixs, which will keep the total flux on the rivers
-   between two neighbouring objects. Note that since growth was not
-   extended to cover the whole image, some objects might be totally
-   isolated from others with no rivers existing between them. To find
-   the unique neighbours of each river pixel, I will be using the same
-   `wngb[]` and `ii` introduced in segmentinfo() of clumps.c.
+   adjacency matrixs, which will keep the brightness (sum of pixel
+   values) on the rivers between two neighbouring objects. Note that
+   since growth was not extended to cover the whole image, some
+   objects might be totally isolated from others with no rivers
+   existing between them. To find the unique neighbours of each river
+   pixel, I will be using the same `wngb[]` and `ii` introduced in
+   segmentinfo() of clumps.c.
 */
 void
 grownclumpstoobjects(struct clumpsthreadparams *ctp)
@@ -365,12 +366,12 @@ newclumplabels(struct clumpsthreadparams *ctp)
 
   /* A simple sanity check. */
   if(ctp->numclumps<=2 || ctp->numobjects<=2)
-    error(EXIT_FAILURE, 0, "A bug! Please contact us at %s so we can fix it. "
+    error(EXIT_FAILURE, 0, "a bug! Please contact us at %s so we can fix it. "
           "For some reason, the newclumplabels function (in segmentation.c) "
           "was called for the %lu detected region, even though this region "
           "has ctp->numclumps=%lu and ctp->numobjects=%lu! This function "
           "should only be used when there is more than one object and "
-          "clump over a detected region.", PACKAGE_BUGREPORT,
+          "clump over a detected region", PACKAGE_BUGREPORT,
           ctp->thislabel, ctp->numclumps, ctp->numobjects);
 
   /* 'numclumpsinobj' will keep the number of clumps in each object.
@@ -424,6 +425,9 @@ newclumplabels(struct clumpsthreadparams *ctp)
 void
 nextavailablelabel(struct clumpsthreadparams *ctp)
 {
+  size_t numclumps = ( (ctp->p->grownclumps && ctp->numclumps<3) ?
+                       1 : ctp->numclumps-1 );
+
   /* If we are using multiple threads, lock the mutex, so only this
      thread can change the p->numobjects and p->numclumps values. */
   if(ctp->p->cp.numthreads>1)
@@ -431,7 +435,7 @@ nextavailablelabel(struct clumpsthreadparams *ctp)
 
   ctp->firstavailablelab  = ctp->p->numobjects;
   ctp->p->numobjects     += ctp->numobjects-1;
-  ctp->p->numclumps      += ctp->numclumps-1;
+  ctp->p->numclumps      += numclumps;
 
   /* This thread is finished with the mutex, unlock it for the
      other threads: */
@@ -477,7 +481,7 @@ segmentonthread(void *inparam)
 
   /* Go over all the initial detections that were assigned to this
      thread. */
-  for(i=0;ctp->indexs[i]!=NONTHRDINDEX;++i)
+  for(i=0;ctp->indexs[i]!=GAL_THREADS_NON_THRD_INDEX;++i)
     if(ctp->indexs[i])   /* sp->indexs[i]==0 for the background. */
       {
         /* Keep the initial label of this detection. The initial label
@@ -488,9 +492,9 @@ segmentonthread(void *inparam)
 
 
         /* Allocate the space for the indexs of the brightest pixels
-           in each clump. We don't know how many clumps there are
-           before hand, so to be safe just allocate a space equal to
-           the number of pixels*/
+           in each clump of this detection. We don't know how many
+           clumps there are before hand, so to be safe just allocate a
+           space equal to the number of pixels*/
         errno=0; ctp->topinds=malloc(ctp->area*sizeof *ctp->topinds);
         if(ctp->topinds==NULL)
           error(EXIT_FAILURE, errno, "%lu bytes for ctp->topinds in "
@@ -508,13 +512,11 @@ segmentonthread(void *inparam)
            detection. */
         clumpsntable(ctp, &sntable);
 
-
         /* Remove clumps with smaller signal to noise ratios and free
            all the nolonger necessary arrays: */
         removefalseclumps(ctp, sntable);
-        free(sntable);
-        free(ctp->xys);
         free(ctp->topinds);
+        free(sntable);
         if(segmentationname && p->stepnum==2) continue;
 
 
@@ -531,6 +533,15 @@ segmentonthread(void *inparam)
             thisdetectionisoneobject(ctp);
             if(segmentationname && ( p->stepnum>=3 && p->stepnum<=6) )
               continue;
+
+            /* If the grown clumps are desired in the output, then
+               replace all these grown clumps with those in
+               p->clab. */
+            if(p->grownclumps)
+              {
+                indf=(ind=ctp->inds)+ctp->area;
+                do p->clab[*ind]=p->olab[*ind]; while(++ind<indf);
+              }
           }
         else
           {
@@ -540,6 +551,14 @@ segmentonthread(void *inparam)
             if(segmentationname && p->stepnum==3)
               { free(ctp->blankinds); continue; }
 
+            /* If the grown clumps are desired in the output, then
+               replace all these grown clumps with those in
+               p->clab. */
+            if(p->grownclumps)
+              {
+                indf=(ind=ctp->inds)+ctp->area;
+                do p->clab[*ind]=p->olab[*ind]; while(++ind<indf);
+              }
 
             /* Identify the objects within the grown clumps and set
                ctp->numobjects. This function allocates
@@ -548,7 +567,6 @@ segmentonthread(void *inparam)
             if(segmentationname && p->stepnum==4)
               { free(ctp->blankinds); free(ctp->segtoobjlabs); continue; }
 
-
             /* Fill in the full detected area. */
             if(ctp->numclumps<=2)
               /* There is only one object in this whole detection. So
@@ -628,7 +646,7 @@ segmentdetections(struct noisechiselparams *p, size_t numobjsinit,
 
 
   /* Distribute the initial labels between all the threads.  */
-  distinthreads(numobjsinit, numthreads, &indexs, &thrdcols);
+  gal_threads_dist_in_threads(numobjsinit, numthreads, &indexs, &thrdcols);
 
   /* Spin off the threads to work on each object if more than one
      thread will be used. If not, simply start working on all the
@@ -650,14 +668,14 @@ segmentdetections(struct noisechiselparams *p, size_t numobjsinit,
 	 threads spinned off. */
       if(numobjsinit<numthreads) nb=numobjsinit+1;
       else                       nb=numthreads+1;
-      attrbarrierinit(&attr, &b, nb);
+      gal_threads_attr_barrier_init(&attr, &b, nb);
 
       /* Initialize the mutex for the number of objects. */
       pthread_mutex_init(&totalnummtx, NULL);
 
       /* Spin off the threads: */
       for(i=0;i<numthreads;++i)
-	if(indexs[i*thrdcols]!=NONTHRDINDEX)
+	if(indexs[i*thrdcols]!=GAL_THREADS_NON_THRD_INDEX)
 	  {
             ctp[i].p=p;
             ctp[i].id=i;
@@ -667,7 +685,7 @@ segmentdetections(struct noisechiselparams *p, size_t numobjsinit,
             ctp[i].totalnummtx=&totalnummtx;
             ctp[i].indexs=&indexs[i*thrdcols];
 	    err=pthread_create(&t, &attr, segmentonthread, &ctp[i]);
-	    if(err) error(EXIT_FAILURE, 0, "Can't create thread %lu.", i);
+	    if(err) error(EXIT_FAILURE, 0, "can't create thread %lu", i);
 	  }
 
       /* Wait for all threads to finish and free the spaces. */
@@ -703,16 +721,43 @@ segmentdetections(struct noisechiselparams *p, size_t numobjsinit,
 /******************************************************************/
 /*****************         Main function        *******************/
 /******************************************************************/
+/* This function is intended for cases when the user doesn't want
+   segmentation. In such cases, a suitable clumps array is necessary
+   for consistency with cases when they do want clumps. So this array
+   will just initialize all the elements in clab that lie over an
+   object with SEGMENTINIT and it will set all the non-masked pixels
+   in it to zero.*/
+void
+clabwithnoseg(long *olab, long *clab, size_t size, int anyblank)
+{
+  long *end=olab+size;
+
+  if(anyblank)
+    do
+      *clab++ = ( *olab==GAL_FITSARRAY_LONG_BLANK ? GAL_FITSARRAY_LONG_BLANK
+                  : ( *olab>0 ? SEGMENTINIT : 0 ) );
+    while(++olab<end);
+  else
+    do
+      *clab++ =  *olab>0 ? SEGMENTINIT : 0 ;
+    while(++olab<end);
+}
+
+
+
+
+
 void
 segmentation(struct noisechiselparams *p)
 {
-  float *f, *fp;
+  unsigned char *b;
   char *extname=NULL;
-  long *forfits=NULL;
+  long *l, *lf, *forfits=NULL;
   size_t i, s0=p->smp.s0, s1=p->smp.s1;
   char *segmentationname=p->segmentationname;
   size_t *labareas, **labinds, numobjsinit=p->numobjects;
 
+
   /* Start off the counter for the number of objects and clumps. The
      value to these variables will be the label that is given to the
      next clump or object found. Note that we stored a copy of the
@@ -721,45 +766,61 @@ segmentation(struct noisechiselparams *p)
   p->numobjects=1;
 
 
+
   /* Start the steps image: */
   if(segmentationname)
     {
-      arraytofitsimg(segmentationname, "Input", FLOAT_IMG, p->img,
-                     s0, s1, p->numblank, p->wcs, NULL, SPACK_STRING);
-      arraytofitsimg(segmentationname, "Convolved-SkySubtracted",
-                     FLOAT_IMG, p->conv, s0, s1, p->numblank,
-                     p->wcs, NULL, SPACK_STRING);
-      arraytofitsimg(segmentationname, "InitialLabels",
-                     LONG_IMG, p->olab, s0, s1, 0, p->wcs,
-                     NULL, SPACK_STRING);
+      gal_fitsarray_array_to_fits_img(segmentationname, "Input", FLOAT_IMG,
+                                      p->img, s0, s1, p->anyblank, p->wcs,
+                                      NULL, SPACK_STRING);
+      gal_fitsarray_array_to_fits_img(segmentationname,
+                                      "Convolved-SkySubtracted", FLOAT_IMG,
+                                      p->conv, s0, s1, p->anyblank, p->wcs,
+                                      NULL, SPACK_STRING);
+      gal_fitsarray_array_to_fits_img(segmentationname, "InitialLabels",
+                                      LONG_IMG, p->olab, s0, s1, p->anyblank,
+                                      p->wcs, NULL, SPACK_STRING);
     }
 
-  /* All possible NaN pixels should be given the largest possible
-     float flux in the convolved image (which is used for
-     over-segmentation). NOTE that the convolved image is used for
-     relative pixel values, not absolute ones. This is because NaN
-     pixels might be in the centers of stars or bright objects (they
-     might slice through a connected region). So we can't allow them
-     to cut our objects. The safest approach is to start segmentation
-     of each object or noise mesh with the NaNs it might contain. A
-     NaN region will never be calculated in any flux measurement any
-     way and if it is connecting two bright objects, they will be
-     segmented because on the two sides of the NaN region, they have
-     different fluxes.*/
-  if(p->numblank)
+
+
+  /* p->clab was used once during detection, we have to reset it to
+     zero. Note that olab will be initialized for each object later
+     on. Since we don't want to care about how clab was set an used,
+     here we simply re-set all the blank pixels too. If there aren't
+     any blank pixels, then just do a memset with no conditionals.*/
+  if(p->anyblank)
     {
-      fp=(f=p->conv)+s0*s1;
-      do if(isnan(*f)) *f=FLT_MAX; while(++f<fp);
+      b=p->byt;lf=(l=p->clab)+s0*s1;
+      do *l = *b++==GAL_FITSARRAY_BYTE_BLANK ? GAL_FITSARRAY_LONG_BLANK
+           : 0; while(++l<lf);
+    }
+  else
+    memset(p->clab, 0, s0*s1*sizeof *p->clab);
+
+
+
+  /* Find the true clump S/N threshold over the image. */
+  p->b0f1=0;
+  findclumpsn(p);
+  if(p->segmentationname)
+    {
+      gal_fitsarray_array_to_fits_img(p->segmentationname,
+                                      "Noise Oversegmentaion", LONG_IMG,
+                                      p->clab, p->smp.s0, p->smp.s1,
+                                      p->anyblank, p->wcs, NULL, SPACK_STRING);
     }
 
 
-  /* Find the true clump S/N threshold andput it in p->lmp.garray1. */
-  p->b0f1=0;
-  clumpsngrid(p);
-  if(p->segmentationname)
-    arraytofitsimg(p->segmentationname, "NoiseOversegmentaion",
-                   LONG_IMG, p->clab, p->smp.s0, p->smp.s1, 0, p->wcs,
-                   NULL, SPACK_STRING);
+
+  /* Initialize p->clab for the clumps within objects. */
+  if(p->anyblank)
+    {
+      lf=(l=p->clab)+s0*s1;
+      do *l = *l==GAL_FITSARRAY_LONG_BLANK ? GAL_FITSARRAY_LONG_BLANK
+           : 0; while(++l<lf);
+    }
+  else memset(p->clab, 0, s0*s1*sizeof *p->clab);
 
 
 
@@ -768,11 +829,6 @@ segmentation(struct noisechiselparams *p)
   labindexs(p->olab, s0*s1, numobjsinit, &labareas, &labinds);
 
 
-  /* p->clab was used for the noise clumps once, we have to reset it
-     to zero. Note that olab will be initialized for each object later
-     on. */
-  memset(p->clab, 0, s0*s1*sizeof *p->clab);
-
 
   /* Segment the detections. When the viewer wants to check the steps,
      the operations have to be repeated multiple times to output each
@@ -782,9 +838,18 @@ segmentation(struct noisechiselparams *p)
     {
       p->stepnum=1;
       while(p->stepnum<8)
-	{
-	  memset(p->clab, 0, s0*s1*sizeof *p->clab);
+        {
+
+          if(p->anyblank)
+            {
+              lf=(l=p->clab)+s0*s1;
+              do *l = *l==GAL_FITSARRAY_LONG_BLANK ? GAL_FITSARRAY_LONG_BLANK
+                   : 0; while(++l<lf);
+            }
+          else memset(p->clab, 0, s0*s1*sizeof *p->clab);
+
           segmentdetections(p, numobjsinit, labareas, labinds);
+
           switch(p->stepnum)
             {
             case 1:
@@ -802,14 +867,16 @@ segmentation(struct noisechiselparams *p)
             case 7:
               extname="Final object labels"; forfits=p->olab; break;
             default:
-              error(EXIT_FAILURE, 0, "A bug! Please contact us at %s to "
+              error(EXIT_FAILURE, 0, "a bug! Please contact us at %s to "
                     "fix the problem. For some reason, the variable "
                     "p->stepnum in segmenation (segmentation.c) has the "
-                    "unrecognized value of %d.", PACKAGE_BUGREPORT,
+                    "unrecognized value of %d", PACKAGE_BUGREPORT,
                     p->stepnum);
             }
-          arraytofitsimg(p->segmentationname, extname, LONG_IMG, forfits,
-                         s0, s1, 0, p->wcs, NULL, SPACK_STRING);
+          gal_fitsarray_array_to_fits_img(p->segmentationname, extname,
+                                          LONG_IMG, forfits, s0, s1,
+                                          p->anyblank, p->wcs, NULL,
+                                          SPACK_STRING);
           ++p->stepnum;
         }
     }
