@@ -18,7 +18,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with gnuastro. If not, see <http://www.gnu.org/licenses/>.
+along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 **********************************************************************/
 #include <config.h>
 
@@ -30,14 +30,16 @@ along with gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <string.h>
 #include <fitsio.h>
 
-#include "timing.h"	/* Includes time.h and sys/time.h   */
-#include "checkset.h"
-#include "txtarrayvv.h"
-#include "commonargs.h"
-#include "arraymanip.h"
-#include "statistics.h"
-#include "configfiles.h"
-#include "fitsarrayvv.h"
+#include <nproc.h>              /* From Gnulib.                     */
+
+#include <gnuastro/timing.h>    /* includes time.h and sys/time.h   */
+#include <gnuastro/checkset.h>
+#include <gnuastro/txtarrayvv.h>
+#include <gnuastro/commonargs.h>
+#include <gnuastro/arraymanip.h>
+#include <gnuastro/statistics.h>
+#include <gnuastro/configfiles.h>
+#include <gnuastro/fitsarrayvv.h>
 
 #include "main.h"
 
@@ -71,7 +73,7 @@ readconfig(char *filename, struct noisechiselparams *p)
   size_t lineno=0, len=200;
   char *line, *name, *value;
   struct uiparams *up=&p->up;
-  struct commonparams *cp=&p->cp;
+  struct gal_commonparams *cp=&p->cp;
   char key='a';	/* Not used, just a place holder. */
 
   /* When the file doesn't exist or can't be opened, it is ignored. It
@@ -95,7 +97,7 @@ readconfig(char *filename, struct noisechiselparams *p)
   while(getline(&line, &len, fp) != -1)
     {
       /* Prepare the "name" and "value" strings, also set lineno. */
-      STARTREADINGLINE;
+      GAL_CONFIGFILES_START_READING_LINE;
 
 
       /* Inputs: */
@@ -105,7 +107,7 @@ readconfig(char *filename, struct noisechiselparams *p)
 	  errno=0;
 	  cp->hdu=malloc(strlen(value)+1);
 	  if(cp->hdu==NULL)
-	    error(EXIT_FAILURE, errno, "Space for HDU.");
+	    error(EXIT_FAILURE, errno, "space for HDU");
 	  strcpy(cp->hdu, value);
 	  cp->hduset=1;
 	}
@@ -115,7 +117,7 @@ readconfig(char *filename, struct noisechiselparams *p)
 	  errno=0;
 	  up->maskname=malloc(strlen(value)+1);
 	  if(up->maskname==NULL)
-	    error(EXIT_FAILURE, errno, "Space for mask name.");
+	    error(EXIT_FAILURE, errno, "space for mask name");
 	  strcpy(up->maskname, value);
 	  up->masknameset=1;
 	}
@@ -125,7 +127,7 @@ readconfig(char *filename, struct noisechiselparams *p)
 	  errno=0;
 	  up->mhdu=malloc(strlen(value)+1);
 	  if(up->mhdu==NULL)
-	    error(EXIT_FAILURE, errno, "Space for mask HDU.");
+	    error(EXIT_FAILURE, errno, "space for mask HDU");
 	  strcpy(up->mhdu, value);
 	  up->mhduset=1;
 	}
@@ -135,7 +137,7 @@ readconfig(char *filename, struct noisechiselparams *p)
 	  errno=0;
 	  up->kernelname=malloc(strlen(value)+1);
 	  if(up->kernelname==NULL)
-	    error(EXIT_FAILURE, errno, "Space for kernel name.");
+	    error(EXIT_FAILURE, errno, "space for kernel name");
 	  strcpy(up->kernelname, value);
 	  up->kernelnameset=1;
 	}
@@ -145,29 +147,29 @@ readconfig(char *filename, struct noisechiselparams *p)
 	  errno=0;
 	  up->khdu=malloc(strlen(value)+1);
 	  if(up->khdu==NULL)
-	    error(EXIT_FAILURE, errno, "Space for kernel HDU.");
+	    error(EXIT_FAILURE, errno, "space for kernel HDU");
 	  strcpy(up->khdu, value);
 	  up->khduset=1;
 	}
       else if(strcmp(name, "skysubtracted")==0)
 	{
 	  if(up->skysubtractedset) continue;
-          intzeroorone(value, &p->skysubtracted, name, key, SPACK,
-                       filename, lineno);
+          gal_checkset_int_zero_or_one(value, &p->skysubtracted, name, key,
+                                       SPACK, filename, lineno);
 	  up->skysubtractedset=1;
 	}
       else if(strcmp(name, "minbfrac")==0)
 	{
 	  if(up->minbfracset) continue;
-          floatl0s1(value, &p->minbfrac, name, key, SPACK,
-                    filename, lineno);
+          gal_checkset_float_l_0_s_1(value, &p->minbfrac, name, key, SPACK,
+                                     filename, lineno);
 	  up->minbfracset=1;
 	}
       else if(strcmp(name, "minnumfalse")==0)
 	{
 	  if(up->minnumfalseset) continue;
-          sizetlzero(value, &p->minnumfalse, name, key, SPACK,
-                     filename, lineno);
+          gal_checkset_sizet_l_zero(value, &p->minnumfalse, name, key, SPACK,
+                                    filename, lineno);
 	  up->minnumfalseset=1;
 	}
 
@@ -180,95 +182,102 @@ readconfig(char *filename, struct noisechiselparams *p)
 	  errno=0;
 	  cp->output=malloc(strlen(value)+1);
 	  if(cp->output==NULL)
-	    error(EXIT_FAILURE, errno, "Space for output");
+	    error(EXIT_FAILURE, errno, "space for output");
 	  strcpy(cp->output, value);
 	  cp->outputset=1;
 	}
+      else if(strcmp(name, "grownclumps")==0)
+	{
+	  if(up->grownclumpsset) continue;
+          gal_checkset_int_zero_or_one(value, &p->grownclumps, name, key, SPACK,
+                                       filename, lineno);
+	  up->grownclumpsset=1;
+	}
 
 
       /* Mesh grid: */
       else if(strcmp(name, "smeshsize")==0)
 	{
 	  if(up->smeshsizeset) continue;
-          sizetlzero(value, &p->smp.meshsize, name, key, SPACK,
-                     filename, lineno);
+          gal_checkset_sizet_l_zero(value, &p->smp.meshsize, name, key, SPACK,
+                                    filename, lineno);
 	  up->smeshsizeset=1;
 	}
       else if(strcmp(name, "lmeshsize")==0)
 	{
 	  if(up->lmeshsizeset) continue;
-          sizetlzero(value, &p->lmp.meshsize, name, key, SPACK,
-                     filename, lineno);
+          gal_checkset_sizet_l_zero(value, &p->lmp.meshsize, name, key, SPACK,
+                                    filename, lineno);
 	  up->lmeshsizeset=1;
 	}
       else if(strcmp(name, "nch1")==0)
 	{
 	  if(up->nch1set) continue;
-          sizetlzero(value, &p->smp.nch1, name, key, SPACK,
-                     filename, lineno);
+          gal_checkset_sizet_l_zero(value, &p->smp.nch1, name, key, SPACK,
+                                    filename, lineno);
 	  up->nch1set=1;
 	}
       else if(strcmp(name, "nch2")==0)
 	{
 	  if(up->nch2set) continue;
-          sizetlzero(value, &p->smp.nch2, name, key, SPACK,
-                     filename, lineno);
+          gal_checkset_sizet_l_zero(value, &p->smp.nch2, name, key, SPACK,
+                                    filename, lineno);
 	  up->nch2set=1;
 	}
       else if(strcmp(name, "lastmeshfrac")==0)
 	{
 	  if(up->lastmeshfracset) continue;
-          floatl0s1(value, &p->smp.lastmeshfrac, name, key, SPACK,
-                    filename, lineno);
+          gal_checkset_float_l_0_s_1(value, &p->smp.lastmeshfrac, name, key,
+                                     SPACK, filename, lineno);
 	  up->lastmeshfracset=1;
 	}
       else if(strcmp(name, "mirrordist")==0)
 	{
 	  if(up->mirrordistset) continue;
-          floatl0(value, &p->smp.mirrordist, name, key, SPACK,
-                  filename, lineno);
+          gal_checkset_float_l_0(value, &p->smp.mirrordist, name, key, SPACK,
+                                 filename, lineno);
 	  up->mirrordistset=1;
 	}
       else if(strcmp(name, "minmodeq")==0)
 	{
 	  if(up->minmodeqset) continue;
-          floatl0s1(value, &p->smp.minmodeq, name, key, SPACK,
-                  filename, lineno);
+          gal_checkset_float_l_0_s_1(value, &p->smp.minmodeq, name, key, SPACK,
+                                     filename, lineno);
 	  up->minmodeqset=1;
 	}
       else if(strcmp(name, "numnearest")==0)
 	{
 	  if(up->numnearestset) continue;
-          sizetlzero(value, &p->smp.numnearest, name, key, SPACK,
-                     filename, lineno);
+          gal_checkset_sizet_l_zero(value, &p->smp.numnearest, name, key, SPACK,
+                                    filename, lineno);
 	  up->numnearestset=1;
 	}
       else if(strcmp(name, "smoothwidth")==0)
 	{
 	  if(up->smoothwidthset) continue;
-          sizetpodd(value, &p->smp.smoothwidth, name, key, SPACK,
-                    filename, lineno);
+          gal_checkset_sizet_p_odd(value, &p->smp.smoothwidth, name, key, SPACK,
+                                   filename, lineno);
 	  up->smoothwidthset=1;
 	}
       else if(strcmp(name, "fullconvolution")==0)
 	{
 	  if(up->fullconvolutionset) continue;
-          intzeroorone(value, &p->smp.fullconvolution, name, key, SPACK,
-                       filename, lineno);
+          gal_checkset_int_zero_or_one(value, &p->smp.fullconvolution, name,
+                                       key, SPACK, filename, lineno);
 	  up->fullconvolutionset=1;
 	}
       else if(strcmp(name, "fullinterpolation")==0)
 	{
 	  if(up->fullinterpolationset) continue;
-          intzeroorone(value, &p->smp.fullinterpolation, name, key, SPACK,
-                       filename, lineno);
+          gal_checkset_int_zero_or_one(value, &p->smp.fullinterpolation, name,
+                                       key, SPACK, filename, lineno);
 	  up->fullinterpolationset=1;
 	}
       else if(strcmp(name, "fullsmooth")==0)
 	{
 	  if(up->fullsmoothset) continue;
-          intzeroorone(value, &p->smp.fullsmooth, name, key, SPACK,
-                       filename, lineno);
+          gal_checkset_int_zero_or_one(value, &p->smp.fullsmooth, name, key,
+                                       SPACK, filename, lineno);
 	  up->fullsmoothset=1;
 	}
 
@@ -277,83 +286,85 @@ readconfig(char *filename, struct noisechiselparams *p)
       else if(strcmp(name, "qthresh")==0)
 	{
 	  if(up->qthreshset) continue;
-          floatl0s1(value, &p->qthresh, name, key, SPACK,
-                  filename, lineno);
+          gal_checkset_float_l_0_s_1(value, &p->qthresh, name, key, SPACK,
+                                     filename, lineno);
 	  up->qthreshset=1;
 	}
       else if(strcmp(name, "erode")==0)
 	{
 	  if(up->erodeset) continue;
-          sizetelzero(value, &p->erode, name, key, SPACK,
-                      filename, lineno);
+          gal_checkset_sizet_el_zero(value, &p->erode, name, key, SPACK,
+                                     filename, lineno);
 	  up->erodeset=1;
 	}
       else if(strcmp(name, "erodengb")==0)
 	{
 	  if(up->erodengbset) continue;
-          int4or8(value, &p->erodengb, name, key, SPACK,
-                  filename, lineno);
+          gal_checkset_int_4_or_8(value, &p->erodengb, name, key, SPACK,
+                                  filename, lineno);
 	  up->erodengbset=1;
 	}
       else if(strcmp(name, "opening")==0)
 	{
 	  if(up->openingset) continue;
-          sizetelzero(value, &p->opening, name, key, SPACK,
-                      filename, lineno);
+          gal_checkset_sizet_el_zero(value, &p->opening, name, key, SPACK,
+                                     filename, lineno);
 	  up->openingset=1;
 	}
       else if(strcmp(name, "openingngb")==0)
 	{
 	  if(up->openingngbset) continue;
-          int4or8(value, &p->openingngb, name, key, SPACK,
-                  filename, lineno);
+          gal_checkset_int_4_or_8(value, &p->openingngb, name, key, SPACK,
+                                  filename, lineno);
 	  up->openingngbset=1;
 	}
       else if(strcmp(name, "sigclipmultip")==0)
 	{
 	  if(up->sigclipmultipset) continue;
-          floatl0(value, &p->sigclipmultip, name, key, SPACK,
-                  filename, lineno);
+          gal_checkset_float_l_0(value, &p->sigclipmultip, name, key, SPACK,
+                                 filename, lineno);
 	  up->sigclipmultipset=1;
 	}
       else if(strcmp(name, "sigcliptolerance")==0)
 	{
 	  if(up->sigcliptoleranceset) continue;
-          floatl0s1(value, &p->sigcliptolerance, name, key, SPACK,
-                  filename, lineno);
+          gal_checkset_float_l_0_s_1(value, &p->sigcliptolerance, name, key,
+                                     SPACK, filename, lineno);
 	  up->sigcliptoleranceset=1;
 	}
       else if(strcmp(name, "dthresh")==0)
 	{
 	  if(up->dthreshset) continue;
-          anyfloat(value, &p->dthresh, name, key, SPACK,
-                   filename, lineno);
+          gal_checkset_any_float(value, &p->dthresh, name, key, SPACK,
+                                 filename, lineno);
 	  up->dthreshset=1;
 	}
       else if(strcmp(name, "detsnminarea")==0)
 	{
 	  if(up->detsnminareaset) continue;
-          sizetlzero(value, &p->detsnminarea, name, key, SPACK,
-                     filename, lineno);
+          gal_checkset_sizet_l_zero(value, &p->detsnminarea, name, key, SPACK,
+                                    filename, lineno);
 	  up->detsnminareaset=1;
 	}
       else if(strcmp(name, "detsnhistnbins")==0)
 	{
 	  if(up->detsnhistnbinsset) continue;
-          sizetelzero(value, &p->detsnhistnbins, name, key, SPACK,
-                      filename, lineno);
+          gal_checkset_sizet_el_zero(value, &p->detsnhistnbins, name, key,
+                                     SPACK, filename, lineno);
 	  up->detsnhistnbinsset=1;
 	}
       else if(strcmp(name, "detquant")==0)
 	{
 	  if(up->detquantset) continue;
-          floatl0s1(value, &p->detquant, name, key, SPACK, filename, lineno);
+          gal_checkset_float_l_0_s_1(value, &p->detquant, name, key, SPACK,
+                                     filename, lineno);
 	  up->detquantset=1;
 	}
       else if(strcmp(name, "dilate")==0)
 	{
 	  if(up->dilateset) continue;
-          sizetelzero(value, &p->dilate, name, key, SPACK, filename, lineno);
+          gal_checkset_sizet_el_zero(value, &p->dilate, name, key, SPACK,
+                                     filename, lineno);
 	  up->dilateset=1;
 	}
 
@@ -362,48 +373,57 @@ readconfig(char *filename, struct noisechiselparams *p)
       else if(strcmp(name, "segsnminarea")==0)
 	{
 	  if(up->segsnminareaset) continue;
-          sizetlzero(value, &p->segsnminarea, name, key, SPACK,
-                     filename, lineno);
+          gal_checkset_sizet_l_zero(value, &p->segsnminarea, name, key, SPACK,
+                                    filename, lineno);
 	  up->segsnminareaset=1;
 	}
+      else if(strcmp(name, "keepmaxnearriver")==0)
+	{
+	  if(up->keepmaxnearriverset) continue;
+          gal_checkset_int_zero_or_one(value, &p->keepmaxnearriver, name, key,
+                                       SPACK, filename, lineno);
+	  up->keepmaxnearriverset=1;
+	}
       else if(strcmp(name, "segquant")==0)
 	{
 	  if(up->segquantset) continue;
-          floatl0s1(value, &p->segquant, name, key, SPACK, filename, lineno);
+          gal_checkset_float_l_0_s_1(value, &p->segquant, name, key, SPACK,
+                                     filename, lineno);
 	  up->segquantset=1;
 	}
-      else if(strcmp(name, "segsnhistnbins")==0)
+      else if(strcmp(name, "clumpsnhistnbins")==0)
 	{
-	  if(up->segsnhistnbinsset) continue;
-          sizetelzero(value, &p->segsnhistnbins, name, key, SPACK,
-                      filename, lineno);
-	  up->segsnhistnbinsset=1;
+	  if(up->clumpsnhistnbinsset) continue;
+          gal_checkset_sizet_el_zero(value, &p->clumpsnhistnbins, name, key,
+                                     SPACK,filename, lineno);
+	  up->clumpsnhistnbinsset=1;
 	}
       else if(strcmp(name, "gthresh")==0)
 	{
 	  if(up->gthreshset) continue;
-          anyfloat(value, &p->gthresh, name, key, SPACK,
-                   filename, lineno);
+          gal_checkset_any_float(value, &p->gthresh, name, key, SPACK,
+                                 filename, lineno);
 	  up->gthreshset=1;
 	}
       else if(strcmp(name, "minriverlength")==0)
 	{
 	  if(up->minriverlengthset) continue;
-          sizetlzero(value, &p->minriverlength, name, key, SPACK,
-                     filename, lineno);
+          gal_checkset_sizet_l_zero(value, &p->minriverlength, name, key, SPACK,
+                                    filename, lineno);
 	  up->minriverlengthset=1;
 	}
       else if(strcmp(name, "objbordersn")==0)
 	{
 	  if(up->objbordersnset) continue;
-          floatl0(value, &p->objbordersn, name, key, SPACK,
-                  filename, lineno);
+          gal_checkset_float_l_0(value, &p->objbordersn, name, key, SPACK,
+                                 filename, lineno);
 	  up->objbordersnset=1;
 	}
 
 
+      /* Operating modes: */
       /* Read options common to all programs */
-      READ_COMMONOPTIONS_FROM_CONF
+      GAL_CONFIGFILES_READ_COMMONOPTIONS_FROM_CONF
 
 
       else
@@ -423,47 +443,22 @@ void
 printvalues(FILE *fp, struct noisechiselparams *p)
 {
   struct uiparams *up=&p->up;
-  struct commonparams *cp=&p->cp;
-  struct meshparams *smp=&p->smp, *lmp=&p->lmp;
+  struct gal_commonparams *cp=&p->cp;
+  struct gal_mesh_params *smp=&p->smp, *lmp=&p->lmp;
 
   /* Print all the options that are set. Separate each group with a
      commented line explaining the options in that group. */
   fprintf(fp, "\n# Input:\n");
   if(cp->hduset)
-    {
-      if(stringhasspace(cp->hdu))
-	fprintf(fp, CONF_SHOWFMT"\"%s\"\n", "hdu", cp->hdu);
-      else
-	fprintf(fp, CONF_SHOWFMT"%s\n", "hdu", cp->hdu);
-    }
+    GAL_CHECKSET_PRINT_STRING_MAYBE_WITH_SPACE("hdu", cp->hdu);
   if(up->masknameset)
-    {
-      if(stringhasspace(up->maskname))
-	fprintf(fp, CONF_SHOWFMT"\"%s\"\n", "mask", up->maskname);
-      else
-	fprintf(fp, CONF_SHOWFMT"%s\n", "mask", up->maskname);
-    }
+    GAL_CHECKSET_PRINT_STRING_MAYBE_WITH_SPACE("mask", up->maskname);
   if(up->mhdu)
-    {
-      if(stringhasspace(up->mhdu))
-	fprintf(fp, CONF_SHOWFMT"\"%s\"\n", "mhdu", up->mhdu);
-      else
-	fprintf(fp, CONF_SHOWFMT"%s\n", "mhdu", up->mhdu);
-    }
+    GAL_CHECKSET_PRINT_STRING_MAYBE_WITH_SPACE("mhdu", up->mhdu);
   if(up->kernelnameset)
-    {
-      if(stringhasspace(up->kernelname))
-	fprintf(fp, CONF_SHOWFMT"\"%s\"\n", "kernel", up->kernelname);
-      else
-	fprintf(fp, CONF_SHOWFMT"%s\n", "kernel", up->kernelname);
-    }
+    GAL_CHECKSET_PRINT_STRING_MAYBE_WITH_SPACE("kernel", up->kernelname);
   if(up->khdu)
-    {
-      if(stringhasspace(up->khdu))
-	fprintf(fp, CONF_SHOWFMT"\"%s\"\n", "khdu", up->khdu);
-      else
-	fprintf(fp, CONF_SHOWFMT"%s\n", "khdu", up->khdu);
-    }
+    GAL_CHECKSET_PRINT_STRING_MAYBE_WITH_SPACE("khdu", up->khdu);
   if(up->skysubtractedset)
     fprintf(fp, CONF_SHOWFMT"%d\n", "skysubtracted", p->skysubtracted);
   if(up->minbfracset)
@@ -475,6 +470,8 @@ printvalues(FILE *fp, struct noisechiselparams *p)
   fprintf(fp, "\n# Output:\n");
   if(cp->outputset)
     fprintf(fp, CONF_SHOWFMT"%s\n", "output", cp->output);
+  if(up->grownclumpsset)
+    fprintf(fp, CONF_SHOWFMT"%d\n", "grownclumps", p->grownclumps);
 
 
   fprintf(fp, "\n# Mesh grid:\n");
@@ -537,10 +534,12 @@ printvalues(FILE *fp, struct noisechiselparams *p)
   fprintf(fp, "\n# Segmentation:\n");
   if(up->segsnminareaset)
     fprintf(fp, CONF_SHOWFMT"%lu\n", "segsnminarea", p->segsnminarea);
+  if(up->keepmaxnearriverset)
+    fprintf(fp, CONF_SHOWFMT"%d\n", "keepmaxnearriver", p->keepmaxnearriver);
   if(up->segquantset)
     fprintf(fp, CONF_SHOWFMT"%.3f\n", "segquant", p->segquant);
-  if(up->segsnhistnbinsset)
-    fprintf(fp, CONF_SHOWFMT"%lu\n", "segsnhistnbins", p->segsnhistnbins);
+  if(up->clumpsnhistnbinsset)
+    fprintf(fp, CONF_SHOWFMT"%lu\n", "clumpsnhistnbins", p->clumpsnhistnbins);
   if(up->gthreshset)
     fprintf(fp, CONF_SHOWFMT"%.3f\n", "gthresh", p->gthresh);
   if(up->minriverlengthset)
@@ -548,10 +547,12 @@ printvalues(FILE *fp, struct noisechiselparams *p)
   if(up->objbordersnset)
     fprintf(fp, CONF_SHOWFMT"%.3f\n", "objbordersn", p->objbordersn);
 
+
   /* For the operating mode, first put the macro to print the common
      options, then the (possible options particular to this
-     program) */
-  PRINT_COMMONOPTIONS;
+     program). */
+  fprintf(fp, "\n# Operating mode:\n");
+  GAL_CONFIGFILES_PRINT_COMMONOPTIONS;
 }
 
 
@@ -565,87 +566,93 @@ void
 checkifset(struct noisechiselparams *p)
 {
   struct uiparams *up=&p->up;
-  struct commonparams *cp=&p->cp;
+  struct gal_commonparams *cp=&p->cp;
 
   int intro=0;
   if(cp->hduset==0)
-    REPORT_NOTSET("hdu");
+    GAL_CONFIGFILES_REPORT_NOTSET("hdu");
   if(up->khduset==0)
-    REPORT_NOTSET("khdu");
+    GAL_CONFIGFILES_REPORT_NOTSET("khdu");
   if(up->skysubtractedset==0)
-    REPORT_NOTSET("skysubtracted");
+    GAL_CONFIGFILES_REPORT_NOTSET("skysubtracted");
   if(up->minbfracset==0)
-    REPORT_NOTSET("minbfrac");
+    GAL_CONFIGFILES_REPORT_NOTSET("minbfrac");
   if(up->minnumfalseset==0)
-    REPORT_NOTSET("minnumfalse");
+    GAL_CONFIGFILES_REPORT_NOTSET("minnumfalse");
+
+  /* Output */
+  if(up->grownclumpsset==0)
+    GAL_CONFIGFILES_REPORT_NOTSET("grownclumps");
 
   /* Mesh grid: */
   if(up->smeshsizeset==0)
-    REPORT_NOTSET("smeshsize");
+    GAL_CONFIGFILES_REPORT_NOTSET("smeshsize");
   if(up->lmeshsizeset==0)
-    REPORT_NOTSET("lmeshsize");
+    GAL_CONFIGFILES_REPORT_NOTSET("lmeshsize");
   if(up->nch1set==0)
-    REPORT_NOTSET("nch1");
+    GAL_CONFIGFILES_REPORT_NOTSET("nch1");
   if(up->nch2set==0)
-    REPORT_NOTSET("nch2");
+    GAL_CONFIGFILES_REPORT_NOTSET("nch2");
   if(up->lastmeshfracset==0)
-    REPORT_NOTSET("lastmeshfrac");
+    GAL_CONFIGFILES_REPORT_NOTSET("lastmeshfrac");
   if(up->mirrordistset==0)
-    REPORT_NOTSET("mirrordist");
+    GAL_CONFIGFILES_REPORT_NOTSET("mirrordist");
   if(up->minmodeqset==0)
-    REPORT_NOTSET("minmodeq");
+    GAL_CONFIGFILES_REPORT_NOTSET("minmodeq");
   if(up->numnearestset==0)
-    REPORT_NOTSET("numnearest");
+    GAL_CONFIGFILES_REPORT_NOTSET("numnearest");
   if(up->smoothwidthset==0)
-    REPORT_NOTSET("smoothwidth");
+    GAL_CONFIGFILES_REPORT_NOTSET("smoothwidth");
   if(up->fullconvolutionset==0)
-    REPORT_NOTSET("fullconvolution");
+    GAL_CONFIGFILES_REPORT_NOTSET("fullconvolution");
   if(up->fullinterpolationset==0)
-    REPORT_NOTSET("fullinterpolation");
+    GAL_CONFIGFILES_REPORT_NOTSET("fullinterpolation");
   if(up->fullsmoothset==0)
-    REPORT_NOTSET("fullsmooth");
+    GAL_CONFIGFILES_REPORT_NOTSET("fullsmooth");
 
   /* Detection: */
   if(up->qthreshset==0)
-    REPORT_NOTSET("qthresh");
+    GAL_CONFIGFILES_REPORT_NOTSET("qthresh");
   if(up->erodeset==0)
-    REPORT_NOTSET("erode");
+    GAL_CONFIGFILES_REPORT_NOTSET("erode");
   if(up->erodengbset==0)
-    REPORT_NOTSET("erodengb");
+    GAL_CONFIGFILES_REPORT_NOTSET("erodengb");
   if(up->openingset==0)
-    REPORT_NOTSET("opening");
+    GAL_CONFIGFILES_REPORT_NOTSET("opening");
   if(up->openingngbset==0)
-    REPORT_NOTSET("openingngb");
+    GAL_CONFIGFILES_REPORT_NOTSET("openingngb");
   if(up->sigclipmultipset==0)
-    REPORT_NOTSET("sigclipmultip");
+    GAL_CONFIGFILES_REPORT_NOTSET("sigclipmultip");
   if(up->sigcliptoleranceset==0)
-    REPORT_NOTSET("sigcliptolerance");
+    GAL_CONFIGFILES_REPORT_NOTSET("sigcliptolerance");
   if(up->dthreshset==0)
-    REPORT_NOTSET("dthresh");
+    GAL_CONFIGFILES_REPORT_NOTSET("dthresh");
   if(up->detsnminareaset==0)
-    REPORT_NOTSET("detsnminarea");
+    GAL_CONFIGFILES_REPORT_NOTSET("detsnminarea");
   if(up->detsnhistnbinsset==0)
-    REPORT_NOTSET("detsnhistnbins");
+    GAL_CONFIGFILES_REPORT_NOTSET("detsnhistnbins");
   if(up->detquantset==0)
-    REPORT_NOTSET("detquant");
+    GAL_CONFIGFILES_REPORT_NOTSET("detquant");
   if(up->dilateset==0)
-    REPORT_NOTSET("dilate");
+    GAL_CONFIGFILES_REPORT_NOTSET("dilate");
 
   /* Segmentation: */
   if(up->segsnminareaset==0)
-    REPORT_NOTSET("segsnminarea");
+    GAL_CONFIGFILES_REPORT_NOTSET("segsnminarea");
+  if(up->keepmaxnearriverset==0)
+    GAL_CONFIGFILES_REPORT_NOTSET("keepmaxnearriver");
   if(up->segquantset==0)
-    REPORT_NOTSET("segquant");
-  if(up->segsnhistnbinsset==0)
-    REPORT_NOTSET("segsnhistnbins");
+    GAL_CONFIGFILES_REPORT_NOTSET("segquant");
+  if(up->clumpsnhistnbinsset==0)
+    GAL_CONFIGFILES_REPORT_NOTSET("clumpsnhistnbins");
   if(up->gthreshset==0)
-    REPORT_NOTSET("gthresh");
+    GAL_CONFIGFILES_REPORT_NOTSET("gthresh");
   if(up->minriverlengthset==0)
-    REPORT_NOTSET("minriverlength");
+    GAL_CONFIGFILES_REPORT_NOTSET("minriverlength");
   if(up->objbordersnset==0)
-    REPORT_NOTSET("objbordersn");
+    GAL_CONFIGFILES_REPORT_NOTSET("objbordersn");
 
-  END_OF_NOTSET_REPORT;
+  GAL_CONFIGFILES_END_OF_NOTSET_REPORT;
 }
 
 
@@ -673,87 +680,102 @@ checkifset(struct noisechiselparams *p)
 void
 sanitycheck(struct noisechiselparams *p)
 {
-  struct meshparams *smp=&p->smp;
+  struct gal_mesh_params *smp=&p->smp;
+
+  /* Make sure the input file exists. */
+  gal_checkset_check_file(p->up.inputname);
 
   /* Set the maskname and mask hdu accordingly: */
-  fileorextname(p->up.inputname, p->cp.hdu, p->up.masknameset,
-                &p->up.maskname, p->up.mhdu, p->up.mhduset, "mask");
+  gal_fitsarray_file_or_ext_name(p->up.inputname, p->cp.hdu, p->up.masknameset,
+                                 &p->up.maskname, p->up.mhdu, p->up.mhduset,
+                                 "mask");
 
   /* Set the output name: */
   if(p->cp.output)
-    checkremovefile(p->cp.output, p->cp.dontdelete);
+    {
+      gal_checkset_check_remove_file(p->cp.output, p->cp.dontdelete);
+
+      /* When the output name is given (possibly with directory
+         information), the user certainly wants the directory
+         information, if they have bothered to include it. */
+      p->cp.removedirinfo=0;
+
+    }
   else
-    automaticoutput(p->up.inputname, "_labeled.fits", p->cp.removedirinfo,
-		p->cp.dontdelete, &p->cp.output);
+    gal_checkset_automatic_output(p->up.inputname, "_labeled.fits",
+                                  p->cp.removedirinfo, p->cp.dontdelete,
+                                  &p->cp.output);
 
   /* Set the check image names: */
   if(p->meshname)
     {
       p->meshname=NULL;         /* Was not allocated before!  */
-      automaticoutput(p->up.inputname, "_meshs.fits", p->cp.removedirinfo,
-                      p->cp.dontdelete, &p->meshname);
+      gal_checkset_automatic_output(p->cp.output, "_meshs.fits",
+                                    p->cp.removedirinfo, p->cp.dontdelete,
+                                    &p->meshname);
     }
   if(p->threshname)
     {
       p->threshname=NULL;
-      automaticoutput(p->up.inputname, "_thresh.fits", p->cp.removedirinfo,
-                      p->cp.dontdelete, &p->threshname);
+      gal_checkset_automatic_output(p->cp.output, "_thresh.fits",
+                                    p->cp.removedirinfo, p->cp.dontdelete,
+                                    &p->threshname);
     }
   if(p->detectionname)
     {
       p->detectionname=NULL;
-      automaticoutput(p->up.inputname, "_det.fits", p->cp.removedirinfo,
-                      p->cp.dontdelete, &p->detectionname);
+      gal_checkset_automatic_output(p->cp.output, "_det.fits",
+                                    p->cp.removedirinfo, p->cp.dontdelete,
+                                    &p->detectionname);
     }
   if(p->detectionskyname)
     {
       p->detectionskyname=NULL;
-      automaticoutput(p->up.inputname, "_detsky.fits", p->cp.removedirinfo,
-                      p->cp.dontdelete, &p->detectionskyname);
+      gal_checkset_automatic_output(p->cp.output, "_detsky.fits",
+                                    p->cp.removedirinfo, p->cp.dontdelete,
+                                    &p->detectionskyname);
     }
-  if(p->detectionsnname)
+  if(p->detsnhistnbins)
     {
-      p->detectionsnname=NULL;
-      automaticoutput(p->up.inputname, "_detsn.fits", p->cp.removedirinfo,
-                      p->cp.dontdelete, &p->detectionsnname);
+      p->detectionsnhist=NULL;
+      gal_checkset_automatic_output(p->cp.output, "_detsn.fits",
+                                    p->cp.removedirinfo, p->cp.dontdelete,
+                                    &p->detectionsnhist);
     }
   if(p->skyname)
     {
       p->skyname=NULL;
-      automaticoutput(p->up.inputname, "_sky.fits", p->cp.removedirinfo,
-                      p->cp.dontdelete, &p->skyname);
+      gal_checkset_automatic_output(p->cp.output, "_sky.fits",
+                                    p->cp.removedirinfo, p->cp.dontdelete,
+                                    &p->skyname);
     }
   if(p->segmentationname)
     {
       p->segmentationname=NULL;
-      automaticoutput(p->up.inputname, "_seg.fits", p->cp.removedirinfo,
-                      p->cp.dontdelete, &p->segmentationname);
+      gal_checkset_automatic_output(p->cp.output, "_seg.fits",
+                                    p->cp.removedirinfo,
+                                    p->cp.dontdelete, &p->segmentationname);
     }
-  if(p->clumpsnname)
+  if(p->clumpsnhistnbins)
     {
-      p->clumpsnname=NULL;
-      automaticoutput(p->up.inputname, "_clumpsn.fits", p->cp.removedirinfo,
-                      p->cp.dontdelete, &p->clumpsnname);
-    }
-  if(p->skysubedname)
-    {
-      p->skysubedname=NULL;
-      automaticoutput(p->up.inputname, "_skysubed.fits", p->cp.removedirinfo,
-                      p->cp.dontdelete, &p->skysubedname);
+      p->clumpsnhist=NULL;
+      gal_checkset_automatic_output(p->cp.output, "_clumpsn.txt",
+                                    p->cp.removedirinfo,
+                                    p->cp.dontdelete, &p->clumpsnhist);
     }
   if(p->maskdetname)
     {
       p->maskdetname=NULL;
-      automaticoutput(p->up.inputname, "_maskdet.fits", p->cp.removedirinfo,
-                      p->cp.dontdelete, &p->maskdetname);
+      gal_checkset_automatic_output(p->cp.output, "_maskdet.fits",
+                                    p->cp.removedirinfo, p->cp.dontdelete,
+                                    &p->maskdetname);
     }
 
-
   /* Other checks: */
-  if(smp->numnearest<MINACCEPTABLENEAREST)
-    error(EXIT_FAILURE, 0, "The smallest possible number for `--numnearest' "
-          "(`-n') is %d. You have asked for: %lu.", MINACCEPTABLENEAREST,
-          smp->numnearest);
+  if(smp->numnearest<GAL_MESH_MIN_ACCEPTABLE_NEAREST)
+    error(EXIT_FAILURE, 0, "the smallest possible number for `--numnearest' "
+          "(`-n') is %d. You have asked for: %lu",
+          GAL_MESH_MIN_ACCEPTABLE_NEAREST, smp->numnearest);
 }
 
 
@@ -836,7 +858,7 @@ float defaultkernel[121]=
 void
 preparearrays(struct noisechiselparams *p)
 {
-  struct meshparams *smp=&p->smp, *lmp=&p->lmp;
+  struct gal_mesh_params *smp=&p->smp, *lmp=&p->lmp;
 
   long *meshindexs;
   float *f, *ff, *fp;
@@ -847,17 +869,18 @@ preparearrays(struct noisechiselparams *p)
      done on the convolved image and some on the actual image, we will
      need to change the mesh's img value some times and the p->img
      will be used to keep its actual value. */
-  filetofloat(p->up.inputname, p->up.maskname, p->cp.hdu, p->up.mhdu,
-              (float **)&smp->img, &p->bitpix, &p->numblank, &smp->s0,
-              &smp->s1);
-  readfitswcs(p->up.inputname, p->cp.hdu, &p->nwcs, &p->wcs);
+  gal_fitsarray_file_to_float(p->up.inputname, p->up.maskname, p->cp.hdu,
+                              p->up.mhdu, (float **)&smp->img, &p->bitpix,
+                              &p->anyblank, &smp->s0, &smp->s1);
+  gal_fitsarray_read_fits_wcs(p->up.inputname, p->cp.hdu, 0, 0,
+                              &p->nwcs, &p->wcs);
   s0=smp->s0; s1=smp->s1;
 
   /* make sure the channel sizes fit the channel sizes. */
   if( s0%smp->nch2 || s1%smp->nch1 )
-    error(EXIT_FAILURE, 0, "The input image size (%lu x %lu) is not an "
+    error(EXIT_FAILURE, 0, "the input image size (%lu x %lu) is not an "
           "exact multiple of the number of the given channels (%lu, %lu) "
-          "in the respective axis.", s1, s0, smp->nch1, smp->nch2);
+          "in the respective axis", s1, s0, smp->nch1, smp->nch2);
 
   /* p->imgss (image-sky-subtracted) is the sky subtracted input
      image. For both the removal of false detections and also the
@@ -885,8 +908,8 @@ preparearrays(struct noisechiselparams *p)
 
   /* Read the kernel: */
   if(p->up.kernelnameset)
-    prepfloatkernel(p->up.kernelname, p->up.khdu, &smp->kernel,
-                    &smp->ks0, &smp->ks1);
+    gal_fitsarray_prep_float_kernel(p->up.kernelname, p->up.khdu, &smp->kernel,
+                                    &smp->ks0, &smp->ks1);
   else
     {
       errno=0;
@@ -910,7 +933,7 @@ preparearrays(struct noisechiselparams *p)
   if(p->olab==NULL)
     error(EXIT_FAILURE, errno, "%lu bytes for p->olab (ui.c)",
           s0*s1*sizeof *p->olab);
-  errno=0; p->clab=calloc(s0*s1, sizeof *p->clab);
+  errno=0; p->clab=malloc(s0*s1*sizeof *p->clab);
   if(p->clab==NULL)
     error(EXIT_FAILURE, errno, "%lu bytes for p->clab (ui.c)",
           s0*s1*sizeof *p->clab);
@@ -950,19 +973,22 @@ preparearrays(struct noisechiselparams *p)
 
 
   /* Prepare the mesh structures. */
-  makemesh(smp);
-  makemesh(lmp);
+  gal_mesh_make_mesh(smp);
+  gal_mesh_make_mesh(lmp);
   if(p->meshname)
     {
-      arraytofitsimg(p->meshname, "Input", FLOAT_IMG, smp->img, s0, s1,
-                     p->numblank, p->wcs, NULL, SPACK_STRING);
-      checkmeshid(smp, &meshindexs);
-      arraytofitsimg(p->meshname, "SmallMeshIndexs", LONG_IMG, meshindexs,
-                     s0, s1, 0, p->wcs, NULL, SPACK_STRING);
+      gal_fitsarray_array_to_fits_img(p->meshname, "Input", FLOAT_IMG,
+                                      smp->img, s0, s1, p->anyblank, p->wcs,
+                                      NULL, SPACK_STRING);
+      gal_check_mesh_id(smp, &meshindexs);
+      gal_fitsarray_array_to_fits_img(p->meshname, "SmallMeshIndexs",
+                                      LONG_IMG, meshindexs, s0, s1, 0, p->wcs,
+                                      NULL, SPACK_STRING);
       free(meshindexs);
-      checkmeshid(lmp, &meshindexs);
-      arraytofitsimg(p->meshname, "LargeMeshIndexs", LONG_IMG, meshindexs,
-                     s0, s1, 0, p->wcs, NULL, SPACK_STRING);
+      gal_check_mesh_id(lmp, &meshindexs);
+      gal_fitsarray_array_to_fits_img(p->meshname, "LargeMeshIndexs", LONG_IMG,
+                                      meshindexs, s0, s1, 0, p->wcs,
+                                      NULL, SPACK_STRING);
       free(meshindexs);
     }
 }
@@ -991,29 +1017,32 @@ preparearrays(struct noisechiselparams *p)
 void
 setparams(int argc, char *argv[], struct noisechiselparams *p)
 {
-  struct commonparams *cp=&p->cp;
+  struct gal_commonparams *cp=&p->cp;
 
   /* Set the non-zero initial values, the structure was initialized to
      have a zero value for all elements. */
   cp->spack         = SPACK;
   cp->verb          = 1;
-  cp->numthreads    = DP_NUMTHREADS;
+  cp->numthreads    = num_processors(NPROC_CURRENT);
   cp->removedirinfo = 1;
 
+  /* NoiseChisel parameter initializations. */
+  p->detsnhistnbins=p->clumpsnhistnbins=0;
+
   /* Read the arguments. */
   errno=0;
   if(argp_parse(&thisargp, argc, argv, 0, 0, p))
-    error(EXIT_FAILURE, errno, "Parsing arguments");
+    error(EXIT_FAILURE, errno, "parsing arguments");
 
   /* Add the user default values and save them if asked. */
-  CHECKSETCONFIG;
+  GAL_CONFIGFILES_CHECK_SET_CONFIG;
 
   /* Check if all the required parameters are set. */
   checkifset(p);
 
   /* Print the values for each parameter. */
   if(cp->printparams)
-    REPORT_PARAMETERS_SET;
+    GAL_CONFIGFILES_REPORT_PARAMETERS_SET;
 
   /* Do a sanity check. */
   sanitycheck(p);
@@ -1086,7 +1115,6 @@ freeandreport(struct noisechiselparams *p, struct timeval *t1)
   free(p->meshname);
   free(p->threshname);
   free(p->maskdetname);
-  free(p->skysubedname);
   free(p->detectionname);
   free(p->segmentationname);
   free(p->detectionskyname);
@@ -1096,5 +1124,6 @@ freeandreport(struct noisechiselparams *p, struct timeval *t1)
     wcsvfree(&p->nwcs, &p->wcs);
 
   /* Print the final message. */
-  reporttiming(t1, SPACK_NAME" finished in", 0);
+  if(p->cp.verb)
+    gal_timing_report(t1, SPACK_NAME" finished in", 0);
 }
