@@ -16,37 +16,41 @@
 
 
 
-# Preliminaries:
-################
+# Preliminaries
+# =============
+#
 # Set the variabels (The executable is in the build tree). Do the
 # basic checks to see if the executable is made or if the defaults
 # file exists (basicchecks.sh is in the source tree).
 prog=convertt
+img=imgcrop_section.fits
 execname=../src/$prog/ast$prog
 
 
 
 
 
-# If the executable was not made (the user chose to not install this
-# package), skip this test:
-if [ ! -f $execname ]; then
-    exit 77
-fi
-
-
-
-
-#If GPL GhostScript is not installed skip this test.
-if [ "x$hasghostscript" != "xyes" ]; then
-    exit 77
+# Skip?
+# =====
+#
+# If the dependencies of the test don't exist, then skip it. There are two
+# types of dependencies:
+#
+#   - The executable was not made (for example due to a configure option),
+#
+#   - The input data was not made (for example the test that created the
+#     data file failed).
+#
+#   - Ghostscript was not present on the system.
+if [ ! -f $execname ] || [ ! -f $img ] || [ "x$hasghostscript" != "xyes" ]; then
+    exit 77;
 fi
 
 
 
 
 
-# Actual test script:
-#####################
-img=imgcrop_section.fits
+
+# Actual test script
+# ==================
 $execname $img --output=pdf --log
