@@ -1,4 +1,4 @@
-# Crop a box based on the --xc and --yc options.
+# Crop a box based on the --ra and --dec options.
 #
 # See the Tests subsection of the manual for a complete explanation
 # (in the Installing gnuastro section).
@@ -16,34 +16,37 @@
 
 
 
-# Preliminaries:
-################
+# Preliminaries
+# =============
+#
 # Set the variabels (The executable is in the build tree). Do the
 # basic checks to see if the executable is made or if the defaults
 # file exists (basicchecks.sh is in the source tree).
 prog=imgcrop
+img=mkprofcat*.fits
 execname=../src/$prog/ast$prog
 
 
 
 
-
-# If the executable was not made (the user chose to not install this
-# package), skip this test:
-if [ ! -f $execname ]; then
-    exit 77
-fi
+# Skip?
+# =====
+#
+# If the dependencies of the test don't exist, then skip it. There are two
+# types of dependencies:
+#
+#   - The executable was not made (for example due to a configure option),
+#
+#   - The input data was not made (for example the test that created the
+#     data file failed).
+if [ ! -f $execname ]; then exit 77; fi
+for fn in $img; do if [ ! -f $fn ]; then exit 77; fi; done
 
 
 
 
 
-# Actual test script:
-#####################
-
-# The number of threads is one so if CFITSIO does is not configured to
-# enable multithreaded access to files, the tests pass. It is the
-# users choice to enable this feature.
-
-img=mkprofcat*.fits
-$execname $img --ra=0.99917157 --dec=1.0008283 --wwidth=0.3 --output=imgcrop_radec.fits --numthreads=1
+# Actual test script
+# ==================
+$execname $img --ra=0.99917157 --dec=1.0008283 --wwidth=0.3 \
+          --output=imgcrop_radec.fits
