@@ -16,30 +16,37 @@
 
 
 
-# Preliminaries:
-################
+# Preliminaries
+# =============
+#
 # Set the variabels (The executable is in the build tree). Do the
 # basic checks to see if the executable is made or if the defaults
 # file exists (basicchecks.sh is in the source tree).
 prog=imgwarp
+img=convolve_spatial.fits
 execname=../src/$prog/ast$prog
 
 
 
 
 
-# If the executable was not made (the user chose to not install this
-# package), skip this test:
-if [ ! -f $execname ]; then
-    exit 77
-fi
+# Skip?
+# =====
+#
+# If the dependencies of the test don't exist, then skip it. There are two
+# types of dependencies:
+#
+#   - The executable was not made (for example due to a configure option),
+#
+#   - The input data was not made (for example the test that created the
+#     data file failed).
+if [ ! -f $execname ] || [ ! -f $img ]; then exit 77; fi
 
 
 
 
 
-# Actual test script:
-#####################
-img=convolve_spatial.fits
+# Actual test script
+# ==================
 $execname $img --output=homographic.fits \
-          --matrix="0.707106781,-0.707106781,0, 0.707106781, 0.707106781,0,      0.001,0.002,1"
+          --matrix="0.707106781,-0.707106781,0,  0.707106781, 0.707106781,0,  0.001,0.002,1"
