@@ -16,36 +16,36 @@
 
 
 
-# Preliminaries:
-################
+# Preliminaries
+# =============
+#
 # Set the variabels (The executable is in the build tree). Do the
 # basic checks to see if the executable is made or if the defaults
 # file exists (basicchecks.sh is in the source tree).
 prog=noisechisel
 execname=../src/$prog/ast$prog
-
-
-
-
-
-# If the executable was not made (the user chose to not install this
-# package), skip this test:
-if [ ! -f $execname ]; then
-    exit 77
-fi
-
-
-
-
-
-# Actual test script:
-#####################
 img=convolve_spatial_noised.fits
 
-# All of these conditions are put here because the image is relatively
-# small, so we can't get good statistics. But that doesn't matter too
-# much here, this is just a test to see if everything is working
-# properly.
-$execname $img --lmeshsize=100 --minbfrac=0.5 --detsnminarea=10   \
-          --minnumfalse=50 --numnearest=5 --segsnminarea=15       \
-          --checkdetection
+
+
+
+
+# Skip?
+# =====
+#
+# If the dependencies of the test don't exist, then skip it. There are two
+# types of dependencies:
+#
+#   - The executable was not made (for example due to a configure option),
+#
+#   - The input data was not made (for example the test that created the
+#     data file failed).
+if [ ! -f $execname ] || [ ! -f $img ]; then exit 77; fi
+
+
+
+
+
+# Actual test script
+# ==================
+$execname $img --checkdetection --checksegmentation
