@@ -16,40 +16,36 @@
 
 
 
-# Preliminaries:
-################
+# Preliminaries
+# =============
+#
 # Set the variabels (The executable is in the build tree). Do the
 # basic checks to see if the executable is made or if the defaults
 # file exists (basicchecks.sh is in the source tree).
 prog=subtractsky
 execname=../src/$prog/ast$prog
+img=convolve_spatial_noised.fits
 
 
 
 
 
-# If the executable was not made (the user chose to not install this
-# package), skip this test:
-if [ ! -f $execname ]; then
-    exit 77
-fi
-
-
-
-
-# This test relies on MakeNoise, so we have to check if it was built:
-if [ ! -f ../src/mknoise/astmknoise ]; then
-    exit 77
-fi
+# Skip?
+# =====
+#
+# If the dependencies of the test don't exist, then skip it. There are two
+# types of dependencies:
+#
+#   - The executable was not made (for example due to a configure option),
+#
+#   - The input data was not made (for example the test that created the
+#     data file failed).
+if [ ! -f $execname ] || [ ! -f $img ]; then exit 77; fi
 
 
 
 
 
-# Actual test script:
-#####################
-base=convolve_spatial
-export GSL_RNG_SEED=1
-export GSL_RNG_TYPE=ranlxs2
-../src/mknoise/astmknoise --envseed $base".fits"
-$execname $base"_noised.fits" --checksky
+# Actual test script
+# ==================
+$execname $img --checksky --numnearest=4
